/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_browse_services.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/08 00:05:45 $
 * 
 * $Revision: 1.38 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <unistd.h>

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_agents.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"
#include "gj_me.h"

#include "gj_gtk_browse_services.h"
#include "gj_gtk_register_service.h"
#include "gj_gtk_support.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_progress.h"


/* columns */
enum
  { 
    COL_BROWSE_SERVICE_JID,
    COL_BROWSE_SERVICE_NAME,
    COL_BROWSE_SERVICE_DESCRIPTION,
    COL_BROWSE_SERVICE_TRANSPORT,
    COL_BROWSE_SERVICE_GROUPCHAT,
    COL_BROWSE_SERVICE_SERVICE,
    COL_BROWSE_SERVICE_REGISTER,
    COL_BROWSE_SERVICE_SEARCH,
    COL_BROWSE_SERVICE_VISIBLE,
    COL_BROWSE_SERVICE_ARE_WE_REGISTERED,
    COL_BROWSE_SERVICE_COUNT
  };


typedef struct 
{
  GtkWidget *window;
  
  GtkWidget *combo_server;
  GtkWidget *combo_entry_server;
  GtkWidget *button_refresh;
    
  GtkWidget *treeview;
  GtkWidget *button_register;
  GtkWidget *button_unregister;

  GtkWidget *label_waiting;
  GtkWidget *progressbar_waiting;

  GtkWidget *button_close;
  GtkWidget *button_help;

    /* members */
  gchar *jid; /* jid that the register/unregister was done to */

    Prg p;
    

} GjBrowseServicesDialog;


static gboolean gj_gtk_bs_setup(GjBrowseServicesDialog *dialog);
static gboolean gj_gtk_bs_is_window_valid(GjBrowseServicesDialog *dialog);

/* void gj_gtk_bs_timer_cb(gboolean timeout_expired, void *userdata); */

static void gj_gtk_bs_enable_controls(GjBrowseServicesDialog *dialog, gboolean enabled);

/*
 * model functions 
 */
static gboolean gj_gtk_bs_model_setup(GjBrowseServicesDialog *dialog);
static gboolean gj_gtk_bs_model_populate_columns(GjBrowseServicesDialog *dialog);
static gboolean gj_gtk_bs_model_get_selection(GjBrowseServicesDialog *dialog, gchar **jid);

static gboolean gj_gtk_bs_set_agent_registered(GjBrowseServicesDialog *dialog, gchar *jid, gboolean registered);

/* internal callbacks */
static void gj_gtk_bs_agent_list_cb(GList *list, gint error_code, gchar *error_message, GjBrowseServicesDialog *dialog);
static void gj_gtk_bs_unregister_cb(guint event_id, guint related_event_id);
static void gj_gtk_bs_refresh_progress_cb(gboolean cancelled, GjBrowseServicesDialog *dialog);

/* gui callbacks */
static void on_treeview_selection_changed(GtkTreeSelection *treeselection, GjBrowseServicesDialog *dialog);

static void on_button_refresh_clicked(GtkButton *button, GjBrowseServicesDialog *dialog);

static void on_destroy(GtkWidget *button, GjBrowseServicesDialog *dialog);
static void on_response(GtkDialog *widget, gint response, GjBrowseServicesDialog *dialog);


static GjBrowseServicesDialog *current_dialog = NULL;


/*
 * window functions
 */
gboolean gj_gtk_bs_load()
{
  GladeXML *xml = NULL;
  GjBrowseServicesDialog *dialog = NULL;

  if(current_dialog != NULL)
    {
      gtk_window_present(GTK_WINDOW(current_dialog->window));
      return TRUE;
    }

  dialog = g_new0(GjBrowseServicesDialog, 1);

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "browse_services",
			      NULL,
			      "browse_services", &dialog->window,
			      "combo_server", &dialog->combo_server,
			      "combo-entry_server", &dialog->combo_entry_server,
			      "treeview", &dialog->treeview,
			      "button_refresh", &dialog->button_refresh,
			      "button_register", &dialog->button_register,
			      "button_unregister", &dialog->button_unregister,
			      "button_close", &dialog->button_close,
			      NULL);

  gj_gtk_glade_connect(xml,
		       dialog,
		       "browse_services", "response", on_response,
		       "browse_services", "destroy", on_destroy,
		       "button_refresh", "clicked", on_button_refresh_clicked,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(dialog->window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU | GDK_DECOR_RESIZEH);
  
  /* setup controls */
  gj_gtk_bs_setup(dialog);

  /* remember */
  current_dialog = dialog;

  return TRUE;
}

static gboolean gj_gtk_bs_setup(GjBrowseServicesDialog *dialog)
{
  GList *servers = NULL;
  const gchar **server_listing = NULL;
  const gchar *str = NULL;
  gint i = 0;

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return FALSE;
  }

  server_listing = gnome_jabber_get_server_list();
  
  /* set a limit of a 1000 */
  while((str = server_listing[i++]) != NULL && i < 1000)
      servers = g_list_append(servers, (gchar*)str);

  gtk_combo_set_popdown_strings(GTK_COMBO(dialog->combo_server), servers);

  /* cleanup */
  if(servers != NULL) g_list_free(servers);

  if(gj_me_get_connected_host() != NULL)
    gtk_entry_set_text(GTK_ENTRY(dialog->combo_entry_server), gj_me_get_connected_host());

  /* set up list */
  gj_gtk_bs_model_setup(dialog);

  return TRUE;
}

static gboolean gj_gtk_bs_is_window_valid(GjBrowseServicesDialog *dialog)
{
  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return FALSE;
  }

  return dialog->window != NULL ? TRUE : FALSE;
}

static void gj_gtk_bs_enable_controls(GjBrowseServicesDialog *dialog, gboolean enabled)
{
  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return;
  }

  gtk_widget_set_sensitive(GTK_WIDGET(dialog->treeview), enabled);

  gtk_widget_set_sensitive(GTK_WIDGET(dialog->combo_server), enabled);
  gtk_widget_set_sensitive(GTK_WIDGET(dialog->combo_entry_server), enabled);

  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_refresh), enabled);

  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_register), enabled);
  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_unregister), enabled);
  
  return;
}

/*
 * model functions 
 */
static gboolean gj_gtk_bs_model_setup(GjBrowseServicesDialog *dialog)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkListStore *store = NULL;
  GtkTreeSelection *selection = NULL;

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return FALSE;
  }

  view = GTK_TREE_VIEW(dialog->treeview);
  selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view));

  /* create store */
  store = gtk_list_store_new(COL_BROWSE_SERVICE_COUNT,
			     G_TYPE_STRING,    /* jid */
			     G_TYPE_STRING,    /* name */
			     G_TYPE_STRING,    /* description */	
			     G_TYPE_STRING,    /* transport */
			     G_TYPE_STRING,    /* groupchat */
			     G_TYPE_STRING,    /* service */
			     G_TYPE_STRING,    /* register */
			     G_TYPE_STRING,    /* search */
			     G_TYPE_BOOLEAN,   /* visible */
			     G_TYPE_BOOLEAN);  /* are we registered ? */

  /* model */
  model = GTK_TREE_MODEL(store);
  gtk_tree_view_set_model(view, model);

  /* selection */
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
  g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(on_treeview_selection_changed), dialog);

  /* columns */
  gj_gtk_bs_model_populate_columns(dialog);
  
  /* properties */
  gtk_tree_view_set_enable_search(view, TRUE);
  gtk_tree_view_set_search_column(view, COL_BROWSE_SERVICE_NAME); 
  gtk_tree_view_set_rules_hint(view, FALSE);
  gtk_tree_view_set_headers_visible(view, TRUE);
  gtk_tree_view_set_headers_clickable(view, TRUE);

  /* clean up */
  g_object_unref(G_OBJECT(store));

  return TRUE;
}

static gboolean gj_gtk_bs_model_populate_columns(GjBrowseServicesDialog *dialog)
{
    GtkTreeView *view = NULL;
  GtkTreeViewColumn *column = NULL; 
  GtkCellRenderer *renderer = NULL;
  guint col_offset = 0;

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return FALSE;
  }

  view = GTK_TREE_VIEW(dialog->treeview);

  /* COL_BROWSE_SERVICE_ARE_WE_REGISTERED */
  renderer = gtk_cell_renderer_toggle_new ();

  column = gtk_tree_view_column_new_with_attributes ("Registered",
						     renderer,
						     "active", COL_BROWSE_SERVICE_ARE_WE_REGISTERED,
						     NULL);

  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_ARE_WE_REGISTERED);
  gtk_tree_view_append_column(view, column);

  column = gtk_tree_view_get_column(view, col_offset);
/*   gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_ARE_WE_REGISTERED); */
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), FALSE);

  /* COL_BROWSE_SERVICE_NAME */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Name"),
							   renderer, 
							   "text", COL_BROWSE_SERVICE_NAME,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_NAME);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_NAME);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

#if 0
#if 0
  /* COL_BROWSE_SERVICE_SERVICE */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Service"),
							   renderer, 
							   "text", COL_BROWSE_SERVICE_SERVICE,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_SERVICE);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_SERVICE);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
#endif

#if 1
  /* COL_BROWSE_SERVICE_TRANSPORT */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Transport"),
							   renderer, 
							   "text", COL_BROWSE_SERVICE_TRANSPORT,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_TRANSPORT);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_TRANSPORT);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
#endif

  /* COL_BROWSE_SERVICE_REGISTER */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Register"),
							   renderer, 
							   "text", COL_BROWSE_SERVICE_REGISTER,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_REGISTER);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_REGISTER);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_BROWSE_SERVICE_GROUPCHAT */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Groupchat"),
							   renderer, 
							   "text", COL_BROWSE_SERVICE_GROUPCHAT,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_GROUPCHAT);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_GROUPCHAT);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

  /* COL_BROWSE_SERVICE_SEARCH */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Search"),
							   renderer, 
							   "text", COL_BROWSE_SERVICE_SEARCH,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_SEARCH);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_SEARCH);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);

#if 0
  /* COL_BROWSE_SERVICE_DESCRIPTION */
  renderer = gtk_cell_renderer_text_new();
  col_offset = gtk_tree_view_insert_column_with_attributes(view,
							   -1, _("Description"),
							   renderer, 
							   "text", COL_BROWSE_SERVICE_DESCRIPTION,
							   NULL);
  
  g_object_set_data(G_OBJECT(renderer), "column", (gint*)COL_BROWSE_SERVICE_DESCRIPTION);

  column = gtk_tree_view_get_column(view, col_offset - 1);
  gtk_tree_view_column_set_sort_column_id(column, COL_BROWSE_SERVICE_DESCRIPTION);
  gtk_tree_view_column_set_clickable(GTK_TREE_VIEW_COLUMN(column), TRUE);
#endif
#endif

  return TRUE;
}

static gboolean gj_gtk_bs_model_get_selection(GjBrowseServicesDialog *dialog, gchar **jid)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeIter iter;
  
  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return FALSE;
  }

  if(jid == NULL)
    {
      g_warning("%s: jid pointer passed was NULL", __FUNCTION__);
      return FALSE;
    }

  view = GTK_TREE_VIEW(dialog->treeview);
  model = GTK_TREE_MODEL(gtk_tree_view_get_model(view));
  selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view));

  if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
    return FALSE;
  
  gtk_tree_model_get(model, &iter, 
		     COL_BROWSE_SERVICE_JID, jid,
		     -1);     

  return TRUE;
}

/* sets */
static gboolean gj_gtk_bs_set_agent_registered(GjBrowseServicesDialog *dialog, gchar *jid, gboolean registered)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkListStore *store = NULL;
  GtkTreeIter iter;

  gboolean valid = FALSE;

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return FALSE;
  }
  
  if(jid == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }

  view = GTK_TREE_VIEW(dialog->treeview);
  model = GTK_TREE_MODEL(gtk_tree_view_get_model(view));
  store = GTK_LIST_STORE(gtk_tree_view_get_model(view));

  valid = gtk_tree_model_get_iter_first(model, &iter);
  
  for(;valid == TRUE; valid = gtk_tree_model_iter_next(model, &iter))
    {
      gchar *text = NULL;

      gtk_tree_model_get(model, &iter, 
			 COL_BROWSE_SERVICE_JID, &text,
			 -1);     

      if(text == NULL)
	continue;

      if(strcmp(text, jid) == 0)
	{
	  gtk_list_store_set(store, &iter, 
			     COL_BROWSE_SERVICE_ARE_WE_REGISTERED, registered,
			     -1);    
	}
      
      g_free(text);
    }

  return TRUE;
} 

/* internal callbacks */
static void gj_gtk_bs_agent_list_cb(GList *agents, gint error_code, gchar *error_reason, GjBrowseServicesDialog *dialog)
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GtkTreeIter iter;

  gint i = 0;

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return;
  }

  if(gj_gtk_bs_is_window_valid(dialog) == FALSE)
    {
      g_message("%s: window is no longer pertinant, not honoring response.", __FUNCTION__);
      return;
    }

  if(agents == NULL && error_code == 0 && error_reason == NULL)
    {
      g_warning("%s: agent list was NULL, error code was 0 and error message was NULL", __FUNCTION__);
      return;
    }

  view = GTK_TREE_VIEW(dialog->treeview);
  store = GTK_LIST_STORE(gtk_tree_view_get_model(view));

  /* clear list if already pressed */
  gtk_list_store_clear(store);

  for(i=0;i<g_list_length(agents);i++)
    {
      GjAgent ag = NULL;
      gboolean registered = FALSE;

      ag = (GjAgent) g_list_nth_data(agents, i);
      if(ag == NULL) continue;
      
      registered = gj_rosters_is_agent_registered(gj_agent_get_jid(ag));

      /* add */ 
      gtk_list_store_append(store, &iter);
      gtk_list_store_set(store, &iter,    
			 COL_BROWSE_SERVICE_JID, gj_agent_get_jid(ag),
			 COL_BROWSE_SERVICE_NAME, gj_agent_get_name(ag)?gj_agent_get_name(ag):"-",
			 COL_BROWSE_SERVICE_DESCRIPTION, gj_agent_get_description(ag)?gj_agent_get_description(ag):"-",
			 COL_BROWSE_SERVICE_TRANSPORT, gj_agent_get_transport(ag)?_("yes"):_("no"),
			 COL_BROWSE_SERVICE_GROUPCHAT, gj_agent_get_groupchat(ag)?_("yes"):_("no"),
			 COL_BROWSE_SERVICE_SERVICE, gj_agent_get_service(ag)?gj_agent_get_service(ag):"-",
			 COL_BROWSE_SERVICE_REGISTER, gj_agent_get_registers(ag)?_("yes"):_("no"),
			 COL_BROWSE_SERVICE_SEARCH, gj_agent_get_search(ag)?_("yes"):_("no"),
			 COL_BROWSE_SERVICE_VISIBLE, TRUE,
			 COL_BROWSE_SERVICE_ARE_WE_REGISTERED, registered,
			 -1);

      /* info */
      g_message("%s: agent %d, jid:'%s', name:'%s', desc:'%s', trans:'%s', groupchat:'%s', service:'%s', register:'%s', search:'%s', registered: %s",
		__FUNCTION__,
		i,
		gj_agent_get_jid(ag),
		gj_agent_get_name(ag),
		gj_agent_get_description(ag),
		gj_agent_get_transport(ag),
		gj_agent_get_groupchat(ag),
		gj_agent_get_service(ag),
		gj_agent_get_registers(ag),
		gj_agent_get_search(ag),
		registered?_("yes"):_("no"));
    }

  /* hide controls */
  gj_gtk_bs_enable_controls(dialog, TRUE);

  /* stop progress dialog */
  gj_gtk_progress_stop(dialog->p);
  dialog->p = NULL;

  /* set buttons disabled */
  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_register), FALSE);
  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_unregister), FALSE);

  /* handle error */
  if(error_code != 0 || error_reason != NULL)
    {
      gj_gtk_show_error(_("Failed to Browse Services."),
			error_code,
			error_reason);
    }
}

static void gj_gtk_bs_unregister_cb(guint event_id, guint related_event_id)
{
  GjInfoquery iq_related = NULL;
  GjBrowseServicesDialog *dialog = NULL;

  xmlNodePtr node = NULL;
  xmlChar *type = NULL;

  g_message("%s: response with event id:%d and related event id:%d", __FUNCTION__, event_id, related_event_id);

  if((iq_related = gj_iq_find(related_event_id)) == NULL)
    {
      g_warning("%s: could not find IQ from related event id:%d", __FUNCTION__, related_event_id);
      return;
    }
  
  dialog = (GjBrowseServicesDialog*) gj_iq_get_userdata(iq_related);

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return;
  }

  node = gj_iq_get_xml(iq_related);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *error_code = NULL;
      xmlChar *error_reason = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);
	  gj_iq_free_all_related_by_event_id(related_event_id);
	  return;
	}

      error_code = gj_parser_find_by_attr_and_ret(node, (guchar*)"code");
      error_reason = gj_parser_find_by_node_and_ret(node, (guchar*)"error");

      gj_gtk_show_error(_("Failed to Unregister Service."),
			atoi((gchar*)error_code),
			(gchar*)error_reason);

      /* clean up */
      g_free(detail);
      gj_iq_free_all_related_by_event_id(related_event_id);

      return;
    }

  /* update table of information */
  gj_gtk_bs_set_agent_registered(dialog, dialog->jid, FALSE);

  /* clean up */
  gj_iq_free_all_related_by_event_id(related_event_id);
}

static void gj_gtk_bs_refresh_progress_cb(gboolean cancelled, GjBrowseServicesDialog *dialog)
{
  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return;
  }

  if(cancelled == TRUE)
    {
      /* hide controls */
      gj_gtk_bs_enable_controls(dialog, TRUE);

      /* reset p */
      dialog->p = NULL;

      /* set buttons disabled */
      gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_register), FALSE);
      gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_unregister), FALSE);
    }
}


/* 
 * gui callbacks
 */
static void on_treeview_selection_changed(GtkTreeSelection *treeselection, GjBrowseServicesDialog *dialog)
{
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;

  gboolean registered = FALSE;

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return;
  }

  if(gtk_tree_selection_get_selected(treeselection, &model, &iter) == FALSE)
    {
      g_message("%s: there is nothing selected", __FUNCTION__);
      return;
    }

  if(GTK_TREE_MODEL(model) == NULL)
    {
      g_warning("%s: model was NULL", __FUNCTION__);
      return;
    }

  gtk_tree_model_get(model, &iter,
		     COL_BROWSE_SERVICE_ARE_WE_REGISTERED, &registered,
		     -1);

  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_register), !registered);
  gtk_widget_set_sensitive(GTK_WIDGET(dialog->button_unregister), registered);
}

static void on_button_refresh_clicked(GtkButton *button, GjBrowseServicesDialog *dialog)
{
  G_CONST_RETURN gchar *server = NULL;

  if(dialog == NULL)
  {
      g_warning("%s: dialog was NULL", __FUNCTION__);
      return;
  }

  server = gtk_entry_get_text(GTK_ENTRY(dialog->combo_entry_server));
  gj_agent_list_request((gchar*)server, (GjAgentCallback) gj_gtk_bs_agent_list_cb, dialog);

  dialog->p = gj_gtk_progress_start(TRUE,
				    GTK_WINDOW(dialog->window),
				    (PrgCb)gj_gtk_bs_refresh_progress_cb, 
				    dialog, 
				    _("Requesting Services"), 
				    _("Server is \"%s\""), 
				    server); 

  gj_gtk_bs_enable_controls(dialog, FALSE);
}

static void on_destroy(GtkWidget *widget, GjBrowseServicesDialog *dialog)
{
    current_dialog = NULL;
    if(dialog == NULL)
	return;

    if(dialog->jid != NULL)
    {
	g_free(dialog->jid);
	dialog->jid = NULL;
    }

    g_free(dialog);
}

static void on_response(GtkDialog *widget, gint response, GjBrowseServicesDialog *dialog)
{
    gchar *jid = NULL;

    if(dialog == NULL)
	return;

  switch(response) 
    {

    case 1:
      if(gj_gtk_bs_model_get_selection(dialog, &jid) == TRUE && jid != NULL)
      {
	  dialog->jid = g_strdup(jid);
	gj_gtk_rs_load(jid);
      }
      
      /* clean up */
      if(jid != NULL) g_free(jid);

      return;

    case 2:
      if(gj_gtk_bs_model_get_selection(dialog, &jid) == TRUE)
      {
	  dialog->jid = g_strdup(jid);
	  gj_iq_request_unregister(jid, gj_gtk_bs_unregister_cb, dialog);
      }
      
      /* clean up */
      if(jid != NULL) g_free(jid);

      return;

    case GTK_RESPONSE_OK:
    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
    case GTK_RESPONSE_APPLY:
      break;
    }

  gtk_widget_destroy(dialog->window);
}


#ifdef __cplusplus
}
#endif
