/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_connection_settings.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/05 16:11:12 $
 * 
 * $Revision: 1.18 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_me.h"
#include "gj_config.h"

#include "gj_gtk_connection_settings.h"
#include "gj_gtk_connection_progress.h"
#include "gj_gtk_password_change.h"
#include "gj_gtk_roster.h"
#include "gj_gtk_support.h"


struct 
{
  GtkWidget *window;

  GtkWidget *combo_server;
  GtkWidget *combo_entry_server;
/*   GtkWidget *entry_server; */
  GtkWidget *spinbutton_port;
  GtkWidget *entry_username;
  GtkWidget *entry_password;

  GtkWidget *entry_nickname;
  GtkWidget *entry_resource;
  GtkWidget *spinbutton_priority;
  
  GtkWidget *checkbutton_save_password;
  GtkWidget *checkbutton_use_plain_text_password;
  GtkWidget *checkbutton_use_ssl;

  GtkWidget *checkbutton_auto_connect_on_startup;
  GtkWidget *checkbutton_auto_reconnect_on_disconnect;

  GtkWidget *button_ok;
  GtkWidget *button_apply;
  GtkWidget *button_cancel;

} ctlCS;


static gboolean connect_on_close;


/* get/set preferences */
static gboolean gj_gtk_cs_open();
static gboolean gj_gtk_cs_save();

/* local functions */
static gboolean gj_gtk_cs_setup();

/*
 * gui callbacks
 */ 
static void on_checkbutton_cs_save_password_toggled(GtkToggleButton *togglebutton, gpointer user_data);

static void on_connection_settings_response(GtkDialog *dialog, gint response, gpointer user_data);

/*
 * code
 */
gboolean gj_gtk_cs_load()
{
  GladeXML *xml = NULL;

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "connection_settings",
			      NULL,
			      "connection_settings", &ctlCS.window,
			      "combo_server", &ctlCS.combo_server,
			      "combo-entry_server", &ctlCS.combo_entry_server,
			      "spinbutton_port", &ctlCS.spinbutton_port,
			      "entry_username", &ctlCS.entry_username,
			      "entry_password", &ctlCS.entry_password,
			      "entry_nickname", &ctlCS.entry_nickname,
			      "entry_resource", &ctlCS.entry_resource,
			      "spinbutton_priority", &ctlCS.spinbutton_priority,
			      "checkbutton_save_password", &ctlCS.checkbutton_save_password,
			      "checkbutton_use_plain_text_password", &ctlCS.checkbutton_use_plain_text_password,
			      "checkbutton_use_ssl", &ctlCS.checkbutton_use_ssl,
			      "checkbutton_auto_connect_on_startup", &ctlCS.checkbutton_auto_connect_on_startup,
			      "checkbutton_auto_reconnect_on_disconnect", &ctlCS.checkbutton_auto_reconnect_on_disconnect,
			      "button_ok", &ctlCS.button_ok,
			      "button_apply", &ctlCS.button_apply,
			      "button_cancel", &ctlCS.button_cancel,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "connection_settings", "response", on_connection_settings_response,
		       "checkbutton_save_password", "toggled", on_checkbutton_cs_save_password_toggled,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(ctlCS.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU);

  gj_gtk_cs_setup();

  /* get settings */
  gj_gtk_cs_open();

  /* set flags */
  connect_on_close = FALSE;

  return TRUE;
}

static gboolean gj_gtk_cs_setup()
{
  GList *servers = NULL;
  const gchar **server_listing = NULL;
  const gchar *str = NULL;
  gint i = 0;

  server_listing = gnome_jabber_get_server_list();
  
  /* set a limit of a 1000 */
  while((str = server_listing[i++]) != NULL && i < 1000)
      servers = g_list_append(servers, (gchar*)str);

  gtk_combo_set_popdown_strings(GTK_COMBO(ctlCS.combo_server), servers);

  /* cleanup */
  g_list_free(servers);
  return TRUE;
}

gboolean gj_gtk_cs_unload()
{
  if(ctlCS.window == NULL) 
    return TRUE;

  gtk_widget_destroy(ctlCS.window);

  ctlCS.window = NULL;

  return TRUE;
}

static gboolean gj_gtk_cs_save()
{
    G_CONST_RETURN gchar *server = NULL;
    gint16 port = 0;
    G_CONST_RETURN gchar *username = NULL;
    G_CONST_RETURN gchar *password = NULL;

    G_CONST_RETURN gchar *nickname = NULL;
    G_CONST_RETURN gchar *resource = NULL;
    gint8 priority = 0;

    gboolean save_password = FALSE;
    gboolean use_plain_text_password = FALSE;
    gboolean use_ssl = FALSE;
    gboolean auto_connect_on_startup = FALSE;
    gboolean auto_reconnect_on_disconnect = FALSE;

    /* get values */
    server = gtk_entry_get_text(GTK_ENTRY(ctlCS.combo_entry_server));
    port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlCS.spinbutton_port));
    username = gtk_entry_get_text(GTK_ENTRY(ctlCS.entry_username));
    password = gtk_entry_get_text(GTK_ENTRY(ctlCS.entry_password));

    nickname = gtk_entry_get_text(GTK_ENTRY(ctlCS.entry_nickname));
    resource = gtk_entry_get_text(GTK_ENTRY(ctlCS.entry_resource));
    priority = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlCS.spinbutton_priority));

    save_password = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_save_password));
    use_plain_text_password = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_use_plain_text_password));
    use_ssl = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_use_ssl));
    
    auto_connect_on_startup = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_auto_connect_on_startup));
    auto_reconnect_on_disconnect = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_auto_reconnect_on_disconnect));

  /* check values */
  if(server == NULL || g_utf8_strlen(server, -1) < 1)
    {
      g_warning("%s: server was NULL", __FUNCTION__);
      return FALSE;
    }

  if(port < 1)
    {
      g_warning("%s: port was < 1", __FUNCTION__);
      return FALSE;
    }

  if(username == NULL || g_utf8_strlen(username, -1) < 1)
    {
      g_warning("%s: username was NULL", __FUNCTION__);
      return FALSE;
    }  

  if(password == NULL || g_utf8_strlen(password, -1) < 1)
    {
      g_warning("%s: password was NULL", __FUNCTION__);
      return FALSE;
    }

  /* set config */
  gj_config_set_cs_server(server);
  gj_config_set_cs_port(port);
  gj_config_set_cs_username(username);
  gj_config_set_cs_password(password);

  gj_config_set_cs_nickname(nickname);
  gj_config_set_cs_resource(resource);
  gj_config_set_cs_priority(priority);

  gj_config_set_cs_save_password(save_password);
  gj_config_set_cs_use_plain_text_password(use_plain_text_password);
  gj_config_set_cs_use_ssl(use_ssl);
  gj_config_set_cs_auto_connect_on_startup(auto_connect_on_startup);
  gj_config_set_cs_auto_reconnect_on_disconnect(auto_reconnect_on_disconnect);  

  /* save */ 
  gj_config_write();

  return TRUE;
}

gboolean gj_gtk_cs_open()
{
  /* load */ 
  gj_config_read();

  /* set to widgets */
  if(gj_config_get_cs_server() != NULL)
    gtk_entry_set_text(GTK_ENTRY(ctlCS.combo_entry_server), gj_config_get_cs_server());

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctlCS.spinbutton_port), 
			    (gdouble)gj_config_get_cs_port());

  if(gj_config_get_cs_username() != NULL)
    gtk_entry_set_text(GTK_ENTRY(ctlCS.entry_username), gj_config_get_cs_username());
  if(gj_config_get_cs_password() != NULL)
    gtk_entry_set_text(GTK_ENTRY(ctlCS.entry_password), gj_config_get_cs_password());

  if(gj_config_get_cs_nickname() != NULL)
    gtk_entry_set_text(GTK_ENTRY(ctlCS.entry_nickname), gj_config_get_cs_nickname());
  if(gj_config_get_cs_resource() != NULL)
    gtk_entry_set_text(GTK_ENTRY(ctlCS.entry_resource), gj_config_get_cs_resource());

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctlCS.spinbutton_priority), 
(gdouble)gj_config_get_cs_priority());

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_save_password), 
			       gj_config_get_cs_save_password());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_use_plain_text_password), 
			       gj_config_get_cs_use_plain_text_password());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_use_ssl), 
			       gj_config_get_cs_use_ssl());
  
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_auto_connect_on_startup), 
			       gj_config_get_cs_auto_connect_on_startup());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlCS.checkbutton_auto_reconnect_on_disconnect), 
			       gj_config_get_cs_auto_reconnect_on_disconnect());
  
  return TRUE;
}

/*
 * GUI events
 */
static void on_checkbutton_cs_save_password_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
    gtk_widget_set_sensitive(GTK_WIDGET(ctlCS.checkbutton_auto_reconnect_on_disconnect), 
			     gtk_toggle_button_get_active(togglebutton));
}

static void on_connection_settings_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_APPLY:
      gj_gtk_cs_save();
      return;

    case GTK_RESPONSE_OK:
      gj_gtk_cs_save();
      gj_gtk_cs_unload();
      
      if(connect_on_close == TRUE)
	gj_gtk_cp_load(gj_gtk_roster_cb_connected, gj_gtk_roster_cb_disconnected);

    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
      break;
    }

  gj_gtk_cs_unload();
}


#ifdef __cplusplus
}
#endif
