/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_docklet.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/08 15:36:36 $
 * 
 * $Revision: 1.11 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <gtk/gtk.h>

#include "common.h"

#include "gj_main.h"
#include "gj_config.h"

#ifndef WIN32
#include "eggtrayicon.h"
#endif

#include "gj_gtk_support.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_my_preferences.h"
#include "gj_gtk_roster.h"


/*
 * DOCKLET menu
 */
enum
  {
    docklet_mnu_connection_settings = 1,
    docklet_mnu_preferences,
    docklet_mnu_quit
  } gj_gtk_docklet_mnu_ids;


struct 
{

#ifndef WIN32
  EggTrayIcon *docklet;
#endif

  GtkWidget *box;
  GtkWidget *image;

} ctlDocklet;


static gboolean is_active = FALSE;
/* static gboolean is_image_shown = TRUE; */


static gboolean gj_gtk_docklet_set_image_from_filename(const gchar *filename);

/* callbacks */
/* static gboolean gj_gtk_docklet_active_timeout_cb(gpointer user_data); */

static void gj_gtk_docklet_embedded(GtkWidget *widget, void *data); 
static void gj_gtk_docklet_destroyed(GtkWidget *widget, void *data); 
static void gj_gtk_docklet_clicked(GtkWidget *button, GdkEventButton *event, void *data);

static void gj_gtk_docklet_menu_cb(GtkMenuItem *menuitem, gpointer user_data);


/*
 * code
 */
#ifndef WIN32 

gboolean gj_gtk_docklet_load() 
{
  if(ctlDocklet.docklet != NULL)
    {
      g_warning("gj_gtk_docklet_load: not loading docklet, it is already loaded.");
      return FALSE;
    }

  if(gj_config_get_mpf_use_docklet() == FALSE)
    {
      g_message("gj_gtk_docklet_load: not loading docklet, see preferences.");
      return TRUE;
    }

  ctlDocklet.docklet = egg_tray_icon_new("gnome-jabber");
  ctlDocklet.box = gtk_event_box_new();
  ctlDocklet.image = gtk_image_new();

  gj_gtk_docklet_set_image_from_filename(FILE_JSF);
  
  gtk_container_add(GTK_CONTAINER(ctlDocklet.box), ctlDocklet.image);
  gtk_container_add(GTK_CONTAINER(ctlDocklet.docklet), ctlDocklet.box);
  gtk_widget_show_all(GTK_WIDGET(ctlDocklet.docklet));

  /* hook up signals */
  g_signal_connect(G_OBJECT(ctlDocklet.docklet), "embedded", G_CALLBACK(gj_gtk_docklet_embedded), NULL);
  g_signal_connect(G_OBJECT(ctlDocklet.docklet), "destroy", G_CALLBACK(gj_gtk_docklet_destroyed), NULL);
  g_signal_connect(G_OBJECT(ctlDocklet.box), "button-press-event", G_CALLBACK(gj_gtk_docklet_clicked), NULL);
  
  /* ref the docklet before we bandy it about the place */
  g_object_ref(G_OBJECT(ctlDocklet.docklet));
  
  return TRUE;
}

gboolean gj_gtk_docklet_unload() 
{
  if(ctlDocklet.docklet == NULL)
    {
      g_warning("gj_gtk_docklet_load: not unloading docklet, it is already unloaded.");
      return FALSE;
    }

  g_object_unref(G_OBJECT(ctlDocklet.docklet));
  gtk_widget_destroy(ctlDocklet.box);

  ctlDocklet.docklet = NULL;
  ctlDocklet.box = NULL;
  ctlDocklet.image = NULL;

  return TRUE;
}

#else

gboolean gj_gtk_docklet_load()
{
  return TRUE;
}

gboolean gj_gtk_docklet_unload()
{
  return TRUE;
}

#endif /* WIN32 */

gboolean gj_gtk_docklet_set_active(gboolean active)
{
/*     if(is_active != active) */
/* 	g_timeout_add(1000, gj_gtk_docklet_active_timeout_cb, NULL); */

    is_active = active;

    return TRUE;
}

/* gboolean gj_gtk_docklet_active_timeout_cb(gpointer user_data) */
/* { */
/*     if(!is_active) */
/*     { */
/* 	is_image_shown = TRUE; */
/* 	gtk_widget_show(GTK_WIDGET(ctlDocklet.image)); */
/* 	return FALSE; */
/*     } */

/*     if(is_image_shown) */
/* 	gtk_widget_show(GTK_WIDGET(ctlDocklet.image)); */
/*     else */
/* 	gtk_widget_hide(GTK_WIDGET(ctlDocklet.image)); */

/*     is_image_shown = !is_image_shown; */

/*     return TRUE; */
/* } */

static gboolean gj_gtk_docklet_set_image_from_filename(const gchar *filename)
{
  gint w = 0;
  gint h = 0;

  GdkPixbuf *pb = NULL;
  GdkPixbuf *pb_scaled = NULL;
	      
  if((pb = gj_gtk_pixbuf_new_from_file(filename)) == NULL)
    {
      g_warning("gj_gtk_docklet_set_image_from_file: pixbuf was NULL");
      return FALSE;
    }
    
  /* get size */
  gtk_icon_size_lookup(GTK_ICON_SIZE_LARGE_TOOLBAR, &w, &h);

  /* scale the image */
  pb_scaled = gdk_pixbuf_scale_simple(pb, w, h, GDK_INTERP_BILINEAR);

  /* set the image */
  gtk_image_set_from_pixbuf(GTK_IMAGE(ctlDocklet.image), pb_scaled);

  /* clean up */
  g_object_unref(G_OBJECT(pb));

  return TRUE;
}

static gboolean gj_gtk_docklet_generate_mnu_rightclick(GdkEventButton *event)
{
  GtkWidget *menu = NULL;
  GtkWidget *item = NULL;

  if(event == NULL)
    {
      g_warning("gj_gtk_docklet_generate_mnu_rightclick: gdk event was NULL");
      return FALSE;
    }

  menu = gtk_menu_new();
  
  item = gtk_menu_item_new_with_mnemonic(_("Connection _Settings"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
  gtk_signal_connect (GTK_OBJECT (item), "activate", 
		      GTK_SIGNAL_FUNC(gj_gtk_docklet_menu_cb), 
		      GINT_TO_POINTER(docklet_mnu_connection_settings));

  item = gtk_image_menu_item_new_from_stock(GTK_STOCK_PREFERENCES, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  gtk_signal_connect (GTK_OBJECT (item), "activate", 
		      GTK_SIGNAL_FUNC(gj_gtk_docklet_menu_cb), 
		      GINT_TO_POINTER(docklet_mnu_preferences));

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  
  item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  gtk_signal_connect (GTK_OBJECT (item), "activate", 
		      GTK_SIGNAL_FUNC(gj_gtk_docklet_menu_cb), 
		      GINT_TO_POINTER(docklet_mnu_quit));

  /* show */
  gtk_widget_show_all(GTK_WIDGET(menu));
  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);

  /* clean up */
  gtk_object_sink(GTK_OBJECT(menu));

  return TRUE;
}


static void gj_gtk_docklet_embedded(GtkWidget *widget, void *data) 
{
  g_message("gj_gtk_docklet_embedded: docklet embedded");
}

static void gj_gtk_docklet_destroyed(GtkWidget *widget, void *data) 
{
  g_message("gj_gtk_docklet_destroyed: docklet destroyed");
}

static void gj_gtk_docklet_clicked(GtkWidget *button, GdkEventButton *event, void *data)
{
  g_message("gj_gtk_docklet_clicked: docklet clicked");

  if(event->button == 1)
    {
      /* just present the application */
      gj_gtk_roster_show(TRUE);
    }
  else if(event->button == 3)
    {
      /* show the menu */
      gj_gtk_docklet_generate_mnu_rightclick(event);
    }
}

static void gj_gtk_docklet_menu_cb(GtkMenuItem *menuitem, gpointer user_data)
{
  gint callback_action = -1;

  if(user_data == NULL)
    {
      g_warning("gj_gtk_docklet_menu_cb: user data was NULL");
      return;
    }

  /* get action */
  callback_action = GPOINTER_TO_INT(user_data);

  g_message ("gj_gtk_docklet_menu_cb: agent menu activated menuitem:0x%.8x, callback action:%d", 
	     (gint)menuitem, callback_action);

  switch(callback_action)
    {
    case docklet_mnu_connection_settings:
      gj_gtk_cs_load();
      break;
    case docklet_mnu_preferences:
      gj_gtk_mpf_load();
      break;
    case docklet_mnu_quit:
      gnome_jabber_quit();
      break;
    }
}


#ifdef __cplusplus
}
#endif
