/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_group_chat.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/08 00:05:45 $
 * 
 * $Revision: 1.20 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gdk/gdkkeysyms.h>

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_roster_item.h"
#include "gj_message.h"
#include "gj_presence.h"
#include "gj_event.h"
#include "gj_support.h"
#include "gj_config.h"
#include "gj_jid.h"

#include "gj_gtk_group_chat.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_support.h"


/* columns */
enum
  {
    COL_GROUP_CHAT_IMAGE,
    COL_GROUP_CHAT_JID,
    COL_GROUP_CHAT_NAME,
    COL_GROUP_CHAT_ENABLED,
    COL_GROUP_CHAT_COUNT
  };


struct t_controls
{
  GtkWidget *window;

  GtkWidget *entry;
  GtkWidget *textview_receive;
  GtkWidget *textview_send;
  GtkWidget *treeview;

};

struct t_GroupChatWindow
{
  struct t_controls controls;

  gchar *host;
  gchar *room;
  gchar *nickname;

  gint ri_id;
};

GList *group_chat_window_list = NULL;

/* list functions */
static gboolean gj_gtk_gcw_list_add(GroupChatWindow gcw);
static gboolean gj_gtk_gcw_list_del(GroupChatWindow gcw);
static GList *gj_gtk_gcw_list_get();

/* item functions */
static GroupChatWindow gj_gtk_gcw_new(const gchar *host, const gchar *room, const gchar *nickname);
static gboolean gj_gtk_gcw_free(GroupChatWindow gcw);
static void gj_gtk_gcw_destroy(GroupChatWindow gcw);

/* find functions */
static GroupChatWindow gj_gtk_gcw_find_by_host_and_room(const gchar *host, const gchar *room);
static GroupChatWindow gj_gtk_gcw_find_by_widget(GtkWidget *widget);
static GroupChatWindow gj_gtk_gcw_find_by_ri_id(gint id);


/*
 * member functions
 */
     
/* gets */
static const gint gj_gtk_gcw_get_ri_id();

static const gchar *gj_gtk_gcw_get_host();
static const gchar *gj_gtk_gcw_get_room();
static const gchar *gj_gtk_gcw_get_nickname();

/* sets */
static gboolean gj_gtk_gcw_set_ri_id(GroupChatWindow gcw, const gint ri_id);

/*
 * window functions 
 */
static gboolean gj_gtk_gcw_load();
static gboolean gj_gtk_gcw_unload(GroupChatWindow gcw);

static gboolean gj_gtk_gcw_setup(GroupChatWindow gcw);
static gboolean gj_gtk_gcw_send();

static gboolean gj_gtk_gcw_disconnect(GroupChatWindow gcw);

static gboolean gj_gtk_gcw_handle_presence(GroupChatWindow gcw, GjPresence pres);
static gboolean gj_gtk_gcw_handle_message(GroupChatWindow gcw, GjMessage m);

static gboolean gj_gtk_gcw_send(GroupChatWindow gcw, gchar *text);
static gboolean gj_gtk_gcw_receive(GroupChatWindow gcw, GjMessage m);


/* 
 * model functions 
 */

static gboolean gj_gtk_gcw_model_setup(GroupChatWindow gcw);
static gboolean gj_gtk_gcw_model_populate_columns(GtkTreeView *view);

static gboolean gj_gtk_gcw_model_find_iter_by_jid(GroupChatWindow gcw, const gchar *jid, GtkTreeIter *ret_iter);

static gboolean gj_gtk_gcw_model_add(GroupChatWindow gcw, GjPresence pres);
static gboolean gj_gtk_gcw_model_remove(GroupChatWindow gcw, GjPresence pres);
static gboolean gj_gtk_gcw_model_update(GroupChatWindow gcw,  GjPresence pres);

#if 0

/*
 * debugging
 */
static void gj_gtk_gcw_print_all();
static void gj_gtk_gcw_print(GroupChatWindow gcw);

#endif


/*
 * gui functions 
 */
static void on_textbuffer_gcw_receive_changed(GtkTextBuffer *textbuffer, gpointer user_data);
static gboolean on_textview_gcw_receive_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data);
static gboolean on_textview_gcw_receive_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
static gboolean on_textview_gcw_send_key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static gboolean on_textview_gcw_send_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);

static gboolean on_group_chat_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);


/*
 * code
 */

/* list handling */
static gboolean gj_gtk_gcw_list_add(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: chatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  group_chat_window_list = g_list_append(group_chat_window_list, gcw);
  return TRUE;
}

static gboolean gj_gtk_gcw_list_del(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: chatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  group_chat_window_list = g_list_remove(group_chat_window_list, gcw);
  return TRUE;
}

static GList *gj_gtk_gcw_list_get()
{
  return group_chat_window_list;
}

static GroupChatWindow gj_gtk_gcw_new(const gchar *host, const gchar *room, const gchar *nickname)
{
  GroupChatWindow gcw = NULL;

  if(host == NULL)
    {
      g_warning("%s: host was NULL", __FUNCTION__);
      return NULL;
    }

  if(room == NULL)
    {
      g_warning("%s: room was NULL", __FUNCTION__);
      return NULL;
    }

  if(nickname == NULL)
    {
      g_warning("%s: nickname was NULL", __FUNCTION__);
      return NULL;
    }

  if((gcw = (GroupChatWindow) g_new(struct t_GroupChatWindow,1)) != NULL)
    {
      gcw->ri_id = -1;

      gcw->host = g_strdup(host);
      gcw->room = g_strdup(room);
      gcw->nickname = g_strdup(nickname);

      gj_gtk_gcw_list_add(gcw);
    }

  return gcw;
}

static gboolean gj_gtk_gcw_free(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  /* free from list */
  gj_gtk_gcw_list_del(gcw);
  
  /* destroy members */
  gj_gtk_gcw_destroy(gcw);

  /* free struct */
  g_free(gcw);
  
  return TRUE;
}

static void gj_gtk_gcw_destroy(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return;
    }

  gcw->ri_id = -1;

  if(gcw->host != NULL)
    {
      g_free(gcw->host);
      gcw->host = NULL;
    }

  if(gcw->room != NULL)
    {
      g_free(gcw->room);
      gcw->room = NULL;
    }

  if(gcw->nickname != NULL)
    {
      g_free(gcw->nickname);
      gcw->nickname = NULL;
    }
}

static GroupChatWindow gj_gtk_gcw_find_by_host_and_room(const gchar *host, const gchar *room)
{
  gint index = 0;

  if(host == NULL)
    {
      g_warning("%s: host was NULL", __FUNCTION__);
      return NULL;
    }

  if(room == NULL)
    {
      g_warning("%s: room was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(gj_gtk_gcw_list_get());index++)
    {
      GroupChatWindow gcw = g_list_nth_data(gj_gtk_gcw_list_get(), index);

      if(gcw == NULL) continue;
      if(gj_gtk_gcw_get_host(gcw) == NULL) continue;
      if(gj_gtk_gcw_get_room(gcw) == NULL) continue;

      if(strcmp(gj_gtk_gcw_get_host(gcw), host) == 0 &&
	 strcmp(gj_gtk_gcw_get_room(gcw), room) == 0)
	return gcw;
    }

  return NULL;  
}

static GroupChatWindow gj_gtk_gcw_find_by_widget(GtkWidget *widget)
{
  gint index = 0;

  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("%s: widget was NULL", __FUNCTION__);
      return NULL;
    }

  /* get top level widget */
  toplevel = gtk_widget_get_toplevel(widget);

  if(toplevel == NULL)
    {
      g_warning("%s: top level widget was NULL", __FUNCTION__);
      return NULL;
    }

  for(index=0;index<g_list_length(gj_gtk_gcw_list_get());index++)
    {
      GroupChatWindow gcw = g_list_nth_data(gj_gtk_gcw_list_get(), index);

      if(gcw == NULL) continue;
      if(gcw->controls.window == toplevel) return gcw;
    }

  return NULL;  
}

static GroupChatWindow gj_gtk_gcw_find_by_ri_id(gint id)
{
  gint index = 0;

  if(id < 0)
    {
      g_warning("%s: id:%d was NULL", __FUNCTION__, id);
      return NULL;
    }

  for(index=0;index<g_list_length(gj_gtk_gcw_list_get());index++)
    {
      GroupChatWindow gcw = g_list_nth_data(gj_gtk_gcw_list_get(), index);

      if(gcw == NULL) continue;
      if(gcw->ri_id == id) return gcw;
    }

  return NULL;  
}

/* 
 * member functions 
 */

/* gets */ 
static const gint gj_gtk_gcw_get_ri_id(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return -1;
    }

  return gcw->ri_id;
}

static const gchar *gj_gtk_gcw_get_host(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return NULL;
    }

  return gcw->host;
}

static const gchar *gj_gtk_gcw_get_room(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return NULL;
    }

  return gcw->room;
}

static const gchar *gj_gtk_gcw_get_nickname(GroupChatWindow gcw)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return NULL;
    }

  return gcw->nickname;
}

/* sets */
static gboolean gj_gtk_gcw_set_ri_id(GroupChatWindow gcw, const gint ri_id)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }  
  
  if(ri_id < 0)
    {
      g_warning("%s: id:%d was < 0", __FUNCTION__, ri_id);
      return FALSE;
    }    

  gcw->ri_id = ri_id;
  return TRUE;
}

#if 0

static void gj_gtk_gcw_print_all()
{
  gint index = 0;

  /* loop through all contacts on roster */
  for(index=0;index<g_list_length(gj_gtk_gcw_list_get());index++)
    {
      GroupChatWindow gcw = g_list_nth_data(gj_gtk_gcw_list_get(), index);
      if(gcw == NULL) continue;

      gj_gtk_gcw_print(gcw);
    }  
}

static void gj_gtk_gcw_print(GroupChatWindow gcw)
{
  if(gcw == NULL)
    return;

  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
  g_message("++++");
  g_message("++++ ri id:'%d'", gcw->ri_id);
  g_message("++++");
  g_message("++++ host:'%s'", gcw->host);
  g_message("++++ room:'%s'", gcw->room);
  g_message("++++ nickname:'%s'", gcw->nickname);
  g_message("++++");
  g_message("++++++++++++++++++++++++++++++++++++++++++++++++");
}

#endif

/*
 * window functions
 */
gboolean gj_gtk_gcw_load(GroupChatWindow gcw)
{
  GladeXML *xml = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "group_chat",
			      NULL,
			      "group_chat", &gcw->controls.window,
			      "entry", &gcw->controls.entry,
			      "textview_receive", &gcw->controls.textview_receive,
			      "textview_send", &gcw->controls.textview_send,
			      "treeview", &gcw->controls.treeview,
			      NULL);

  gtk_widget_set_events(GTK_WIDGET(gcw->controls.textview_receive),
			GDK_EXPOSURE_MASK | 
			GDK_POINTER_MOTION_MASK | 
			GDK_POINTER_MOTION_HINT_MASK | 
			GDK_BUTTON_RELEASE_MASK | 
			GDK_STRUCTURE_MASK | 
			GDK_PROPERTY_CHANGE_MASK);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "group_chat", "delete_event", on_group_chat_delete_event,
		       "textview_receive", "button_press_event", on_textview_gcw_receive_button_press_event,
		       "textview_receive", "motion_notify_event", on_textview_gcw_receive_motion_notify_event,
		       "textview_send", "key_press_event", on_textview_gcw_send_key_press_event,
		       "textview_send", "key_release_event", on_textview_gcw_send_key_release_event,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(gcw->controls.window->window), 
			      GDK_DECOR_ALL);

  gj_gtk_gcw_setup(gcw);

  return TRUE;
}

static gboolean gj_gtk_gcw_unload(GroupChatWindow gcw)
{
  GjRosterItem ri = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  /* remove roster item */
  if((ri = gj_rosters_find_item_by_id(gj_gtk_gcw_get_ri_id(gcw))) != NULL)
    gj_roster_item_free(ri);
  
  /* kill the window */
  gtk_widget_destroy(gcw->controls.window);
  
  /* free the mem */
  gj_gtk_gcw_free(gcw);
  
  g_message("%s: unloaded...", __FUNCTION__);
  
  return TRUE;
}

static gboolean gj_gtk_gcw_setup(GroupChatWindow gcw)
{
  GtkTextBuffer *buffer = NULL;

  gchar *text = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  /* set title */
  text = g_strdup_printf("%s %s@%s", 
			 _("Chatting in"), 
			 gj_gtk_gcw_get_room(gcw),
			 gj_gtk_gcw_get_host(gcw));

  gtk_window_set_title(GTK_WINDOW(gcw->controls.window), text);
  g_free(text);

  /* set buffer changed callback */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(gcw->controls.textview_receive));

  g_signal_connect(G_OBJECT(buffer), "changed",  
 		   G_CALLBACK(on_textbuffer_gcw_receive_changed),  
 		   NULL); 

  /* set styles */
  gtk_text_buffer_create_tag(buffer, "bold", 
			     "weight", PANGO_WEIGHT_BOLD, 
			     NULL);
  gtk_text_buffer_create_tag(buffer, "italic", 
			     "style", PANGO_STYLE_ITALIC, 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "blue_foreground", 
			     "foreground", "darkblue", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "red_foreground", 
			     "foreground", "darkred", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "green_foreground", 
			     "foreground", "darkgreen", 
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "black_foreground", 
			     "foreground", "black", 
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "url", 
			     "foreground", "blue", 
			     NULL); 

  /* set up model */
  gj_gtk_gcw_model_setup(gcw);

  return TRUE;
}

gboolean gj_gtk_gcw_join(const gchar *host, const gchar *room, const gchar *nickname)
{
  GroupChatWindow gcw = NULL;
  GjRosterItem ri = NULL;
  gchar *jid = NULL;

  if(nickname == NULL || g_utf8_strlen(nickname, -1) < 1)
    {
      g_warning("%s: nickname was NULL", __FUNCTION__);
      return FALSE;
    }

  if(host == NULL || g_utf8_strlen(host, -1) < 1)
    {
      g_warning("%s: host was NULL", __FUNCTION__);
      return FALSE;
    }

  if(room == NULL || g_utf8_strlen(room, -1) < 1)
    {
      g_warning("%s: room was NULL", __FUNCTION__);
      return FALSE;
    }

  if((gcw = gj_gtk_gcw_find_by_host_and_room(host, room)) != NULL)
    {
      g_message("%s: there is already a GroupChatWindow open to host:'%s', room:'%s'", __FUNCTION__, host, room);
      gtk_window_present(GTK_WINDOW(gcw->controls.window));

      return TRUE;
    }

  if((gcw = gj_gtk_gcw_new(host, room, nickname)) == NULL)
    return FALSE;

  if(gj_gtk_gcw_load(gcw) == FALSE)
    return FALSE;

  /* set up jid */
  jid = g_strdup_printf("%s@%s/%s", room, host, nickname);

  if((ri = gj_roster_item_new(jid)) == NULL)
    {
      if(jid != NULL) g_free(jid);
      return FALSE;
    }

  /* remember so we can free it up after */
  gj_gtk_gcw_set_ri_id(gcw, gj_roster_item_get_id(ri));
  
  /* send presence... and effectively join */
  gj_presence_send_to(gj_roster_item_get_jid(ri), GjPresenceTypeAvailable, GjPresenceShowNormal, "Online", 0);

  /* clean up */
  if(jid != NULL) g_free(jid);

  return TRUE;
}

static gboolean gj_gtk_gcw_disconnect(GroupChatWindow gcw)
{
  gchar *jid = NULL;
  GjJID j = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  /* set up jid */
  jid = g_strdup_printf("%s@%s/%s", 
			gj_gtk_gcw_get_room(gcw), 
			gj_gtk_gcw_get_host(gcw), 
			gj_gtk_gcw_get_nickname(gcw));

  j = gj_jid_new(jid);
  
  /* send unavailable presence back */
  gj_presence_send_to(j, GjPresenceTypeUnAvailable, GjPresenceShowNormal, NULL, 0);

  /* clean up */
  if(jid != NULL) g_free(jid);
  if(j != NULL) gj_jid_unref(j);
  
  return TRUE;
}

gboolean gj_gtk_gcw_post_by_event(GjEvent ev)
{
  GroupChatWindow gcw = NULL;

  if(ev == NULL)
    {
      g_warning("%s: event was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if((gcw = gj_gtk_gcw_find_by_ri_id(gj_event_get_ri_id(ev))) == NULL)
    {
      g_warning("%s: could not find GroupChatWindow by roster item id:%d", __FUNCTION__, gj_event_get_ri_id(ev));
      return FALSE;
    }

  switch(gj_event_get_type(ev))
    {
    case GjEventTypePresence:
      gj_gtk_gcw_handle_presence(gcw, (GjPresence)gj_event_get_data(ev));
      break;

    case GjEventTypeMessage:
      gj_gtk_gcw_handle_message(gcw, (GjMessage)gj_event_get_data(ev));
      break;

    default:
      break;
    }

  return TRUE;
}

static gboolean gj_gtk_gcw_handle_presence(GroupChatWindow gcw, GjPresence pres)
{
  GtkTreeIter iter;

  GjJID j = NULL;
  j = gj_presence_get_from_jid(pres);

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_presence_get_type(pres) == GjPresenceTypeAvailable)
    {
      if(gj_gtk_gcw_model_find_iter_by_jid(gcw, gj_jid_get_full(j), &iter) == FALSE)
	{
	  /* not found, add contact */
	  gj_gtk_gcw_model_add(gcw, pres);
	}
      else
	{
	  /* update presence */
	  gj_gtk_gcw_model_update(gcw, pres);
	}
    }
  else
    {
      gchar *jid = NULL;

      /* set up jid */
      jid = g_strdup_printf("%s@%s/%s", gj_gtk_gcw_get_room(gcw), gj_gtk_gcw_get_host(gcw), gj_gtk_gcw_get_nickname(gcw));
      
      /* remove */
      if(strcmp(gj_jid_get_full(j), jid) == 0)
	gj_gtk_gcw_unload(gcw);
      else
	gj_gtk_gcw_model_remove(gcw, pres);

      /* clean up */
      if(jid != NULL) g_free(jid);
    }

  return TRUE;
}

static gboolean gj_gtk_gcw_handle_message(GroupChatWindow gcw, GjMessage m)
{
  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if(m == NULL)
    { 
      g_warning("%s: message was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_message_get_type(m) != GjMessageTypeGroupChat)
    return TRUE;
  
  gj_gtk_gcw_receive(gcw, m);
  return TRUE;
}

static gboolean gj_gtk_gcw_send(GroupChatWindow gcw, gchar *text)
{
  gchar *jid = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if(text == NULL)
    {
      g_warning("%s: text was NULL, can not send NULL", __FUNCTION__);
      return FALSE;
    }

  /* set up jid */
  jid = g_strdup_printf("%s@%s", gj_gtk_gcw_get_room(gcw), gj_gtk_gcw_get_host(gcw));

  /* send message */
  gj_message_send(GjMessageTypeGroupChat, 
		  jid, 
		  NULL, 
		  text, 
		  NULL);
  
#if 0
  /* log to history */
  gj_history_write_sent_by_ri_id(gj_gtk_cw_get_ri_id(cw), 
				 (gchar*)gj_get_timestamp_nice(), 
				 NULL, 
				 text);
#endif

  /* clean up */
  if(jid != NULL) g_free(jid);

  return TRUE;
}

static gboolean gj_gtk_gcw_receive(GroupChatWindow gcw, GjMessage m)
{
  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextMark *mark = NULL;
  GtkTextIter iter;

  GjJID j = NULL;

  gchar *message = NULL;
  gchar *time = NULL;
  gchar *colour_time_and_name = NULL;
  gchar *colour_content = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if(m == NULL)
    {
      g_warning("%s: message was NULL", __FUNCTION__);
      return FALSE;
    }

  if((textview = GTK_TEXT_VIEW(gcw->controls.textview_receive)) == NULL)
    {
      g_warning("%s: textview was NULL", __FUNCTION__);
      return FALSE;
    }

  j = gj_message_get_from_jid(m);
  
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(gcw->controls.textview_receive));
  time = g_strdup_printf("[%s]", gj_get_timestamp_from_timet(gj_message_get_time(m)));

  /* set details */
  message = g_strdup_printf("%s%s\n", gj_jid_get_resource(j) == NULL ? "" : ": ", gj_message_get_body(m));

  /* set colours */
  if(gj_jid_get_resource(j) != NULL)
    {
      if(strcmp(gj_jid_get_resource(j), gj_gtk_gcw_get_nickname(gcw)) == 0)
	colour_time_and_name = g_strdup_printf("green_foreground");
      else
	colour_time_and_name = g_strdup_printf("blue_foreground");

      colour_content = g_strdup_printf("black_foreground");
    }
  else
    {
      colour_time_and_name = g_strdup_printf("red_foreground");
      colour_content = g_strdup_printf("red_foreground");
    }


#if 0
  /* get text direction */
  if(gj_gtk_cw_get_reverse_text_direction(cw) == TRUE)
    gtk_text_buffer_get_start_iter(buffer, &iter);
  else
    gtk_text_buffer_get_end_iter(buffer, &iter);
#else
    gtk_text_buffer_get_end_iter(buffer, &iter);
#endif

    if(gj_config_get_mpf_hide_time() == FALSE)
      {
	/* insert at iter */
	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
						 time, -1,
						 colour_time_and_name,
						 NULL);
	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
						 " ", -1,
						 colour_time_and_name,
						 NULL);
      }

    if(gj_jid_get_resource(j) != NULL)
      {
	gtk_text_buffer_insert_with_tags_by_name(buffer, &iter,
						 gj_jid_get_resource(j), -1,
						 "bold", 
						 colour_time_and_name,
						 NULL);
      }

  if(gj_config_get_mpf_use_smileys() == TRUE)
    gj_gtk_textbuffer_insert_with_emote_icons(buffer, &iter, colour_content, message);
  else
    gj_gtk_textbuffer_insert(buffer, &iter, colour_content, message);

  /* set subject */
  if(gj_message_get_subject(m) != NULL)
    gtk_entry_set_text(GTK_ENTRY(gcw->controls.entry), gj_message_get_subject(m));

#if 0
  /* log to history */
  gj_history_write_received_by_ri_id(gj_gtk_cw_get_ri_id(cw), 
				     (gchar*)gj_get_timestamp_nice_from_timet(gj_message_get_time(m)), 
				     NULL, 
				     gj_message_get_body(m));

  if(gj_gtk_cw_get_reverse_text_direction(cw) == FALSE)
    {
      mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, FALSE);
      gtk_text_view_scroll_mark_onscreen(textview, mark);
      gtk_text_buffer_delete_mark(buffer, mark);
    }

#endif

  mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, FALSE);
  gtk_text_view_scroll_mark_onscreen(textview, mark);
  gtk_text_buffer_delete_mark(buffer, mark);

  /* clean up */
  if(colour_time_and_name != NULL) g_free(colour_time_and_name);
  if(colour_content != NULL) g_free(colour_content);
  if(time != NULL) g_free(time);
  if(message != NULL) g_free(message);

  return TRUE;
}


/*
 * model functions
 */
static gboolean gj_gtk_gcw_model_setup(GroupChatWindow gcw)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkListStore *store = NULL;
  GtkTreeSelection *selection = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(gcw->controls.treeview)) == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }

  if((selection = GTK_TREE_SELECTION(gtk_tree_view_get_selection(view))) == NULL)
    {
      g_warning("%s: selection was NULL", __FUNCTION__);
      return FALSE;
    }

  /* store */
  store = gtk_list_store_new(COL_GROUP_CHAT_COUNT,
			     GDK_TYPE_PIXBUF,   /* presence */
			     G_TYPE_STRING,     /* jid */
			     G_TYPE_STRING,     /* name */
			     G_TYPE_BOOLEAN);   /* include */
			     
  /* model */
  model = GTK_TREE_MODEL(store);
  gtk_tree_view_set_model(view, model);

  /* selection */
  gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

  gj_gtk_gcw_model_populate_columns(view);

  /* properties */
  gtk_tree_view_set_enable_search(view, TRUE);
  gtk_tree_view_set_search_column(view, COL_GROUP_CHAT_NAME); 
  gtk_tree_view_set_rules_hint(view, FALSE);
  gtk_tree_view_set_headers_visible(view, FALSE);
  gtk_tree_view_set_headers_clickable(view, FALSE);
  gtk_tree_view_columns_autosize(view);

  /* clean up */
  g_object_unref(G_OBJECT(store));

  return TRUE;
}

static gboolean gj_gtk_gcw_model_populate_columns(GtkTreeView *view)
{
  GtkTreeViewColumn *column = NULL;
  GtkCellRenderer *renderer = NULL;

  if(view == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }

  /* cell renderer's */
  column = gtk_tree_view_column_new();

  /* COL_ROSTER_IMAGE */  
  renderer = gtk_cell_renderer_pixbuf_new();
  gtk_tree_view_column_pack_start(column, renderer, FALSE);
  gtk_tree_view_column_set_attributes(column,
				      renderer,
				      "pixbuf", COL_GROUP_CHAT_IMAGE,
				      NULL);

  /* COL_ROSTER_TEXT */
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_column_pack_start(column, renderer, TRUE);
  gtk_tree_view_column_set_attributes(column,
				      renderer,
				      "text", COL_GROUP_CHAT_NAME,
				      NULL);

  /* insert column */
  gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);

  return TRUE;
}

static gboolean gj_gtk_gcw_model_find_iter_by_jid(GroupChatWindow gcw, const gchar *jid, GtkTreeIter *ret_iter)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;
  
  gboolean valid = FALSE;
  gboolean found = FALSE;
  gchar *text = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if((view = GTK_TREE_VIEW(gcw->controls.treeview)) == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if((model = GTK_TREE_MODEL(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("%s: model was NULL", __FUNCTION__);
      return FALSE;
    }

  if(jid == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }

  if(ret_iter == NULL)
    {
      g_warning("%s: ret_iter pointer was NULL", __FUNCTION__);
      return FALSE;
    }

  /* Get the first iter in the list */
  valid = gtk_tree_model_get_iter_first(model, &iter);

  for(found = FALSE; valid == TRUE && found == FALSE; valid = gtk_tree_model_iter_next(model, &iter))
    {
      gtk_tree_model_get(model, &iter, 
			 COL_GROUP_CHAT_JID, &text,
			 -1);

      if(text != NULL && strcmp(jid, text) == 0)
	{
	  *ret_iter = iter;
	  found = TRUE;
	}

      if(text == NULL) g_free(text);
    }

  return found;
}

static gboolean gj_gtk_gcw_model_add(GroupChatWindow gcw, GjPresence pres)
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GtkTreeIter iter;

  GdkPixbuf *pb = NULL;

  GjJID j = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    } 

  j = gj_presence_get_from_jid(pres);

  if((view = GTK_TREE_VIEW(gcw->controls.treeview)) == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("%s: model was NULL", __FUNCTION__);
      return FALSE;
    }

  pb = gj_gtk_presence_to_pixbuf_new(gj_presence_get_type(pres), gj_presence_get_show(pres));

  gtk_list_store_append(store, &iter);
  gtk_list_store_set(store, &iter, 
		     COL_GROUP_CHAT_IMAGE, pb,
		     COL_GROUP_CHAT_JID, gj_jid_get_full(j),
		     COL_GROUP_CHAT_NAME, gj_jid_get_resource(j),
		     -1);

  /* clean up */
  if(pb != NULL) g_object_unref(G_OBJECT(pb));

  return TRUE;
}

static gboolean gj_gtk_gcw_model_remove(GroupChatWindow gcw, GjPresence pres)
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GtkTreeIter iter;

  GjJID j = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    } 

  j = gj_presence_get_from_jid(pres);

  if((view = GTK_TREE_VIEW(gcw->controls.treeview)) == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("%s: model was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_gtk_gcw_model_find_iter_by_jid(gcw, gj_jid_get_full(j), &iter) == FALSE)
    {
      g_warning("%s: user not found in model with jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
      return TRUE;
    }

  gtk_list_store_remove(store, &iter);
  return TRUE;
}

static gboolean gj_gtk_gcw_model_update(GroupChatWindow gcw, GjPresence pres)
{
  GtkTreeView *view = NULL;
  GtkListStore *store = NULL;
  GtkTreeIter iter;

  GdkPixbuf *pb = NULL;

  GjJID j = NULL;

  if(gcw == NULL)
    {
      g_warning("%s: GroupChatWindow was NULL", __FUNCTION__);
      return FALSE;
    }

  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    } 

  j = gj_presence_get_from_jid(pres);
  
  if((view = GTK_TREE_VIEW(gcw->controls.treeview)) == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if((store = GTK_LIST_STORE(gtk_tree_view_get_model(view))) == NULL)
    {
      g_warning("%s: model was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_gtk_gcw_model_find_iter_by_jid(gcw, gj_jid_get_full(j), &iter) == FALSE)
    {
      g_warning("%s: user not found in model with jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
      return TRUE;
    }

  pb = gj_gtk_presence_to_pixbuf_new(gj_presence_get_type(pres), gj_presence_get_show(pres));
  
  gtk_list_store_set(store, &iter, 
		     COL_GROUP_CHAT_IMAGE, pb,
		     COL_GROUP_CHAT_JID, gj_jid_get_full(j),
		     COL_GROUP_CHAT_NAME, gj_jid_get_resource(j),
		     -1);

  /* clean up */
  if(pb != NULL) g_object_unref(G_OBJECT(pb));

  return TRUE;
}

/*
 * GUI events
 */
static void on_textbuffer_gcw_receive_changed(GtkTextBuffer *textbuffer, gpointer user_data)
{
  gj_gtk_textbuffer_handle_changed(textbuffer, user_data);
}

static gboolean on_textview_gcw_receive_motion_notify_event(GtkWidget *widget, GdkEventMotion *event, gpointer user_data)
{
  return gj_gtk_textview_handle_motion_notify_event(widget, event, user_data);
}

static gboolean on_textview_gcw_receive_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
  return gj_gtk_textview_handle_button_press_event(widget, event, user_data);
}

static gboolean on_textview_gcw_send_key_release_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  GroupChatWindow gcw = NULL;

  if((buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget))) == NULL)
    {
      g_warning("%s: buffer was NULL", __FUNCTION__);
      return FALSE;
    }

  if((gcw = gj_gtk_gcw_find_by_widget(GTK_WIDGET(widget))) == NULL)
    {
      g_warning("%s: failed to find window based on widget:0x%.8x", __FUNCTION__, (gint)widget);
      return FALSE;
    }

  /* if was enter then clear box */
  if(event != NULL && event->keyval == GDK_Return && (event->state & GDK_SHIFT_MASK) == FALSE)
    {
      GtkTextIter iter_start;
      GtkTextIter iter_end; 

      gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
      gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
    }
  else
    {
      gint length = 0;

      /* set toolbar icon */  
      length = gtk_text_buffer_get_char_count(buffer);
    }

  return FALSE;
}

static gboolean on_textview_gcw_send_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;  

  gchar *text = NULL;
  gint length = 0;

  GroupChatWindow gcw = NULL;
    
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
  length = gtk_text_buffer_get_char_count(buffer);

  if((gcw = gj_gtk_gcw_find_by_widget(GTK_WIDGET(widget))) == NULL)
    {
      g_warning("%s: failed to find window based on button widget", __FUNCTION__);
      return FALSE;
    }
  
  if(length < 1)
    {
      /* if enter was pressed... warn user that we can not do this */
      if(event != NULL && event->keyval == GDK_Return && (event->state & GDK_SHIFT_MASK) == FALSE)
	g_warning("%s: can't send NULL or 0 length message", __FUNCTION__);
      
      return FALSE;
    }

  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  text = gtk_text_buffer_get_text(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end, TRUE);
   
  /* send message and clear text box */
  if(event->keyval == GDK_Return && (event->state & GDK_SHIFT_MASK) == FALSE)
    gj_gtk_gcw_send(gcw, text);
    
  /* cleanup */
  if(text != NULL) g_free(text);
    
  return FALSE;
}

static gboolean on_group_chat_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  GroupChatWindow gcw = NULL;

  if(widget == NULL)
    {
      g_warning("%s: widget was NULL", __FUNCTION__);
      return FALSE;
    }

  if((gcw = gj_gtk_gcw_find_by_widget(widget)) == NULL)
    {
      g_warning("%s: could not find GroupChatWindow from widget:0x%.8x", __FUNCTION__, (gint)widget);
      return FALSE;
    }
    
  gj_gtk_gcw_disconnect(gcw);
  return TRUE;
}

#ifdef __cplusplus
}
#endif
