/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_group_chat_join.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/05 16:11:12 $
 * 
 * $Revision: 1.9 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_iq_requests.h"
#include "gj_config.h"

#include "gj_gtk_group_chat_join.h"
#include "gj_gtk_group_chat.h"
#include "gj_gtk_support.h"


struct 
{
  GtkWidget *window;

  GtkWidget *entry_nickname;
  GtkWidget *entry_host;
  GtkWidget *entry_room;

  GtkWidget *button_ok;

} ctlGCJ;


static gboolean gj_gtk_gcj_join();
static gboolean gj_gtk_gcj_check();


/*
 * gui functions 
 */
static void on_entry_gcj_nickname_changed(GtkEditable *editable, gpointer user_data);
static void on_entry_gcj_host_changed(GtkEditable *editable, gpointer user_data);
static void on_entry_gcj_room_changed(GtkEditable *editable, gpointer user_data);

static void on_group_chat_join_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * window functions
 */

gboolean gj_gtk_gcj_load()
{
  GladeXML *xml = NULL;

  if(ctlGCJ.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlGCJ.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "group_chat_join",
			      NULL,
			      "group_chat_join", &ctlGCJ.window,
			      "entry_nickname", &ctlGCJ.entry_nickname,
			      "combo-entry_host", &ctlGCJ.entry_host,
			      "entry_room", &ctlGCJ.entry_room,
			      "button_ok", &ctlGCJ.button_ok,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "group_chat_join", "response", on_group_chat_join_response,
		       "entry_nickname", "changed", on_entry_gcj_nickname_changed,
		       "combo-entry_host", "changed", on_entry_gcj_host_changed,
		       "entry_room", "changed", on_entry_gcj_room_changed,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);
 
  gdk_window_set_decorations (GDK_WINDOW(ctlGCJ.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  /* set up window */
  gtk_entry_set_text(GTK_ENTRY(ctlGCJ.entry_nickname), gj_config_get_cs_nickname());
/*   gtk_entry_set_text(GTK_ENTRY(ctlGCJ.entry_host), "conference.jabber.org"); */
/*   gtk_entry_set_text(GTK_ENTRY(ctlGCJ.entry_room), "jdev"); */

  return TRUE;
}

gboolean gj_gtk_gcj_unload()
{
  if(ctlGCJ.window == NULL)
    return FALSE;

  gtk_widget_destroy(GTK_WIDGET(ctlGCJ.window));

  ctlGCJ.window = NULL;
  ctlGCJ.entry_nickname = NULL;
  ctlGCJ.entry_host = NULL;
  ctlGCJ.entry_room = NULL;
  ctlGCJ.button_ok = NULL;

  return TRUE;
}

static gboolean gj_gtk_gcj_join()
{
  G_CONST_RETURN gchar *nickname = NULL;
  G_CONST_RETURN gchar *host = NULL;
  G_CONST_RETURN gchar *room = NULL;

  nickname = gtk_entry_get_text(GTK_ENTRY(ctlGCJ.entry_nickname));
  host = gtk_entry_get_text(GTK_ENTRY(ctlGCJ.entry_host));
  room = gtk_entry_get_text(GTK_ENTRY(ctlGCJ.entry_room));

  if(gj_gtk_gcj_check() == FALSE)
    return FALSE;

  return gj_gtk_gcw_join(host, room, nickname);
}

static gboolean gj_gtk_gcj_check()
{
  G_CONST_RETURN gchar *nickname = NULL;
  G_CONST_RETURN gchar *host = NULL;
  G_CONST_RETURN gchar *room = NULL;

  nickname = gtk_entry_get_text(GTK_ENTRY(ctlGCJ.entry_nickname));
  host = gtk_entry_get_text(GTK_ENTRY(ctlGCJ.entry_host));
  room = gtk_entry_get_text(GTK_ENTRY(ctlGCJ.entry_room));

  if((nickname == NULL || g_utf8_strlen(nickname, -1) < 1) ||
     (host == NULL || g_utf8_strlen(host, -1) < 1) ||
     (room == NULL || g_utf8_strlen(room, -1) < 1))
    {
      /* disable ok */
      gtk_widget_set_sensitive(GTK_WIDGET(ctlGCJ.button_ok), FALSE);
      return FALSE;
    }

  gtk_widget_set_sensitive(GTK_WIDGET(ctlGCJ.button_ok), TRUE);
  return TRUE;
}


/*
 * GUI events
 */
static void on_entry_gcj_nickname_changed(GtkEditable *editable, gpointer user_data)
{
  gj_gtk_gcj_check();
}

static void on_entry_gcj_host_changed(GtkEditable *editable, gpointer user_data)
{
  gj_gtk_gcj_check();
}

static void on_entry_gcj_room_changed(GtkEditable *editable, gpointer user_data)
{
  gj_gtk_gcj_check();
}

static void on_group_chat_join_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_OK:
      gj_gtk_gcj_join();

    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
    case GTK_RESPONSE_APPLY:
      break;
    }

  gj_gtk_gcj_unload();
}


#ifdef __cplusplus
}
#endif
