/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_log.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/08 15:45:03 $
 * 
 * $Revision: 1.16 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdio.h>
#include <string.h>

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_log.h"
#include "gj_support.h"

#include "gj_gtk_log.h"
#include "gj_gtk_support.h"

#define GJ_GTK_LOG_TAG_ERROR "error"
#define GJ_GTK_LOG_TAG_CRITICAL "critical"
#define GJ_GTK_LOG_TAG_WARNING "warning"
#define GJ_GTK_LOG_TAG_MESSAGE "message"
#define GJ_GTK_LOG_TAG_OTHER "other"


struct 
{
  GtkWidget *window;

  GtkWidget *textview;
  GtkWidget *button_clear;
/*   GtkWidget *button_close; */
  GtkWidget *togglebutton_scrolling;
  
} ctlLog;


struct
{
  gboolean stop_scrolling;

} optLog;


/*
 * window functions 
 */ 

static gboolean gj_gtk_log_setup();
static gboolean gj_gtk_log_clear();

/*
 * member functions 
 */ 

/* gets */
static gboolean gj_gtk_log_get_stop_scrolling();

/* sets */
static gboolean gj_gtk_log_set_stop_scrolling(gboolean stop_scrolling);

/*
 * GUI functions
 */
static void on_togglebutton_log_scrolling_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_button_log_clear_clicked(GtkButton *button, gpointer user_data);
/* static void on_button_log_close_clicked(GtkButton *button, gpointer user_data); */

static gboolean on_log_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);


gboolean gj_gtk_log_load()
{
  GladeXML *xml = NULL;

  if(ctlLog.window != NULL)
    {
      gj_gtk_log_show();
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "log",
			      NULL,
			      "log", &ctlLog.window,
 			      "textview", &ctlLog.textview,
 			      "button_clear", &ctlLog.button_clear,
/*  			      "button_close", &ctlLog.button_close, */
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "log", "delete_event", on_log_delete_event,
		       "togglebutton_scrolling", "toggled", on_togglebutton_log_scrolling_toggled,
		       "button_clear", "clicked", on_button_log_clear_clicked,
/* 		       "button_close", "clicked", on_button_log_close_clicked, */
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  /* set up tags for log */
  gj_gtk_log_setup();
  gj_gtk_log_set_stop_scrolling(FALSE);

  return TRUE;
}

gboolean gj_gtk_log_unload()
{
  if(ctlLog.window == NULL)
    return FALSE;

  gtk_widget_destroy(ctlLog.window);
  
  ctlLog.window = NULL;

  ctlLog.textview = NULL;
  ctlLog.button_clear = NULL;
  ctlLog.togglebutton_scrolling = NULL;

  return TRUE;
}

gboolean gj_gtk_log_show()
{
  gtk_widget_show(GTK_WIDGET(ctlLog.window));

  /* if we do it on load, the window is hidden, and we get warnings */
  gdk_window_set_decorations (GDK_WINDOW(ctlLog.window->window), 
			      GDK_DECOR_ALL);

  return TRUE;
}

gboolean gj_gtk_log_hide()
{
  gtk_widget_hide(GTK_WIDGET(ctlLog.window));
  return TRUE;
}

static gboolean gj_gtk_log_setup()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  /* set buffer iterator at position 0 */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlLog.textview));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  /* set styles */
  gtk_text_buffer_create_tag(buffer, "bold", 
			     "weight", PANGO_WEIGHT_BOLD, 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);
  gtk_text_buffer_create_tag(buffer, "italic", 
			     "style", PANGO_STYLE_ITALIC, 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "critical", 
			     "foreground", "darkred", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "error", 
			     "foreground", "red", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "warning", 
			     "foreground", "darkorange", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "message", 
			     "foreground", "darkblue", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  
  gtk_text_buffer_create_tag(buffer, "other", 
			     "foreground", "black", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "debug", 
			     "foreground", "grey", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL); 
  gtk_text_buffer_create_tag(buffer, "details", 
			     "foreground", "darkgreen", 
			     "family", "monospace",
			     "scale", PANGO_SCALE_SMALL,
			     NULL);  

  return TRUE;
}

/*
 * member functions 
 */ 

/* gets */
static gboolean gj_gtk_log_get_stop_scrolling()
{
  return optLog.stop_scrolling;
}

/* sets */
static gboolean gj_gtk_log_set_stop_scrolling(gboolean stop_scrolling)
{
  optLog.stop_scrolling = stop_scrolling;
  return TRUE;
}

static gboolean gj_gtk_log_clear()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlLog.textview));
  gtk_text_buffer_get_bounds(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  /* clear buffer */
  gtk_text_buffer_delete(GTK_TEXT_BUFFER(buffer), &iter_start, &iter_end);
  
  return TRUE;
}

gboolean gj_gtk_log_new(const gchar *log_domain, 
			GLogLevelFlags log_level, 
			const gchar *message, 
			gpointer user_data)
{
  GtkTextView *textview = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkTextMark *mark = NULL;
  GtkTextIter iter_start;
  GtkTextIter iter_end;

  gchar *details = NULL;
  gchar *format = NULL;
  gchar *tag = NULL;

  if(log_level < 0 || message == NULL)
    {
      g_print("%s: failed to add new message, log level:%d was < 0 OR message was NULL\n", __FUNCTION__, log_level);
      return FALSE;
    }

  if((textview = GTK_TEXT_VIEW(ctlLog.textview)) == NULL)
    {
      fprintf(stderr, "%s: textview was NULL", __FUNCTION__);
      return FALSE;
    }

  buffer = gtk_text_view_get_buffer(textview);
  gtk_text_buffer_get_bounds(buffer, &iter_start, &iter_end);

  /* format strings */
  details = g_strdup_printf("%s - %s\t",
			    gj_get_timestamp(),
			    log_domain?log_domain:"GJ");

  format = g_strdup_printf("%s\n",message);

  /* print initial part of message */
  switch(log_level)
    {
    case G_LOG_LEVEL_ERROR: 
      tag = g_strdup_printf("error");
      break;
    case G_LOG_LEVEL_CRITICAL:
      tag = g_strdup_printf("critical");
      break;
    case G_LOG_LEVEL_WARNING:
      tag = g_strdup_printf("warning");
      break;
    case G_LOG_LEVEL_MESSAGE:
      tag = g_strdup_printf("message");
      break;
    case G_LOG_LEVEL_DEBUG:
      tag = g_strdup_printf("debug");
      break;
    case G_LOG_LEVEL_INFO:
    default:
      tag = g_strdup_printf("other");
      break;
    }

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter_end,
					   details, -1,
					   "details",
					   NULL);

  gtk_text_buffer_insert_with_tags_by_name(buffer, &iter_end,
					   format, -1,
				 	   tag,
					   NULL);

  if(gj_gtk_log_get_stop_scrolling() == FALSE)
    {
      mark = gtk_text_buffer_create_mark(buffer, NULL, &iter_end, FALSE);
      gtk_text_view_scroll_mark_onscreen(textview, mark);
      gtk_text_buffer_delete_mark(buffer, mark);
    }

  if(details != NULL) g_free(details);
  if(format != NULL) g_free(format);
  if(tag != NULL) g_free(tag);

  return TRUE;
}

/*
 * GUI events
 */
static void on_togglebutton_log_scrolling_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
  gj_gtk_log_set_stop_scrolling(gtk_toggle_button_get_active(togglebutton));
}

static void on_button_log_clear_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_log_clear();
}

/* static void on_button_log_close_clicked(GtkButton *button, gpointer user_data) */
/* { */
/*   gj_gtk_log_hide(); */
/* } */

static gboolean on_log_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  gj_gtk_log_hide();

  return TRUE;
}

#ifdef __cplusplus
}
#endif
