/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $RCSfile: gj_gtk_msg.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/05 16:11:12 $
 *
 * $Revision: 1.15 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_main.h"

#include "gj_gtk_msg.h"
#include "gj_gtk_support.h"

/* list functions */
static gboolean gj_gtk_msg_list_add(Msg m);
static gboolean gj_gtk_msg_list_del(Msg m);

/* item functions */
static gboolean gj_gtk_msg_free(Msg m);
static void gj_gtk_msg_destroy(Msg m);

/* find */
static Msg gj_gtk_msg_find_by_widget(GtkWidget *widget);

/* member functions - gets */
static const gchar *gj_gtk_msg_get_primary(Msg m);
static const gchar *gj_gtk_msg_get_secondary(Msg m);

static gint gj_gtk_msg_get_type(Msg m);
static MsgCb gj_gtk_msg_get_cb(Msg m);

/* member functions - sets */

/* local functions */
static gboolean gj_gtk_msg_function(Msg m, gint type);
static gboolean gj_gtk_msg_setup(Msg m);

/*
 * actual GUI functions
 */
static void on_msg_response(GtkDialog *dialog, gint response, gpointer user_data);


struct t_ctlMsg 
{
  /* search */
  GtkWidget *window;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;
  GtkWidget *button_yes;
  GtkWidget *button_no;
  GtkWidget *button_close;

  GtkWidget *label_primary;
  GtkWidget *label_secondary;

  GtkWidget *image;

};

struct t_msg
{
  guint id;
  void *userdata;

  /* properties */
  gchar *primary;
  gchar *title;
  gchar *secondary; 

  gboolean modal;

  gint type;

  MsgCb cb;

  gint response;

  /* controls */
  struct t_ctlMsg controls;
};


guint msg_id = 0;
GList *msg_list = NULL;


gboolean gj_gtk_msg_init()
{
  /* nothing to do here */
  return TRUE;
}

gboolean gj_gtk_msg_term()
{
  gint index = 0;
    
  /* list */
  if(msg_list == NULL)
    return TRUE;
    
  /* free all items on the list */
  for(index=0;index<g_list_length(msg_list);index++)
    {
      Msg m = NULL;
	
      if((m = (Msg) g_list_nth_data(msg_list,index)) != NULL)
	gj_gtk_msg_free(m);
    }
    
  g_list_free(msg_list);
  msg_list = NULL;
    
  return TRUE;
}

/*
 * list functions
 */
static gboolean gj_gtk_msg_list_add(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_list_add: failed to add item to list, it was NULL");
      return FALSE;
    }

  msg_list = g_list_append(msg_list,m);

  return TRUE;
}

static gboolean gj_gtk_msg_list_del(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_list_del: failed to del item from list, it was NULL");
      return FALSE;
    }

  msg_list = g_list_remove(msg_list, m);
  return TRUE;
}

/*
 * item functions
 */
Msg gj_gtk_msg_new(gboolean modal, 
		   gint type, 
		   MsgCb cb, 
		   void *userdata, 
		   const gchar *primary, 
		   const gchar *secondary, ...)
{
  Msg m = NULL;
  
  va_list args;
  gchar *buffer = NULL;

  va_start(args,secondary);
  buffer = g_strdup_vprintf(secondary,args);
  va_end(args);

  /* check vars */
  if(primary == NULL)
    {
      g_warning("gj_gtk_msg_new: primary was NULL");
      return NULL;
    }
  
/*   if(secondary == NULL) */
/*     { */
/*       g_warning("gj_gtk_msg_new: secondary was NULL"); */
/*       return NULL; */
/*     } */

/*   if(title == NULL) */
/*     { */
/*       g_warning("gj_gtk_msg_new: title was NULL"); */
/*       return NULL; */
/*     } */
  
  if(type < 1)
    {
      g_warning("gj_gtk_msg_new: type:%d was < 1",type);
      return NULL;
    }
  
  g_message("gj_gtk_msg_new: Msg id:%d", msg_id);

  /* create new struct */
  if((m = g_new(struct t_msg,1)))
    {
      m->id = msg_id++;
      m->userdata = userdata;

      m->primary = g_strdup(primary);
      m->secondary = buffer;

/*       m->title = g_strdup(title); */

      m->modal = modal;
      m->type = type;
      m->cb = cb;

      m->response = 0;
    }

  /* add to list */
  gj_gtk_msg_list_add(m);

  /* show dialog */
  gj_gtk_msg_load(m);

  return m;
}

static gboolean gj_gtk_msg_free(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_free: failed to free, Msg was NULL");
      return FALSE;
    }

  g_message("gj_gtk_msg_free: Msg id:%d", gj_gtk_msg_get_id(m));

  /* free mem */
  gj_gtk_msg_list_del(m);  

  /* destroy members */
  gj_gtk_msg_destroy(m);

  /* free structure */
  g_free(m);

  return TRUE;
}

static void gj_gtk_msg_destroy(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_destroy: failed to destroy, Msg was NULL");
      return;
    }

  m->id = 0;

  m->userdata = NULL;

  if(m->primary != NULL)
    {
      g_free(m->primary);
      m->primary = NULL;
    }

  if(m->secondary != NULL)
    {
      g_free(m->secondary);
      m->secondary = NULL;
    }

/*   if(m->title != NULL) */
/*     { */
/*       g_free(m->title); */
/*       m->title = NULL; */
/*     } */

  m->modal = FALSE;
  m->type = 0;
  m->cb = NULL;

  m->response = 0;

  /* controls */
  m->controls.window = NULL;
  
  m->controls.button_ok = NULL;
  m->controls.button_cancel = NULL;
  m->controls.button_yes = NULL;
  m->controls.button_no = NULL;
  m->controls.button_close = NULL;
  
  m->controls.label_secondary = NULL;
  m->controls.label_secondary = NULL;
  
  m->controls.image = NULL;
}

static Msg gj_gtk_msg_find_by_widget(GtkWidget *widget)
{
  gint index = 0;
  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("gj_gtk_msg_find_by_widget: widget was NULL");
      return NULL;
    }

  toplevel = gtk_widget_get_toplevel(widget);
  if(!GTK_WIDGET_TOPLEVEL(toplevel))
    {
      g_warning("gj_gtk_msg_find_by_widget: could not find toplevel widget for widget:0x%.8x", (gint)widget);
      return NULL;
    }

  for(index=0;index<g_list_length(msg_list);index++)
    {
      Msg m = g_list_nth_data(msg_list, index);
      
      if(m == NULL)
	{
	  g_warning("gj_gtk_msg_find_by_widget: getting nth item(%d) in user information list was NULL",index);
	  continue;
	}
      
      if(m->controls.window == toplevel)
	return m;
    }
 
  return NULL;
}

/*
 * member functions 
 */

/* gets */
guint gj_gtk_msg_get_id(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_id: Msg was NULL");
      return -1;
    }
  
  return m->id;
}

void *gj_gtk_msg_get_userdata(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_userdata: Msg was NULL");
      return NULL;
    }
  
  return m->userdata;
}

static const gchar *gj_gtk_msg_get_primary(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_primary: Msg was NULL");
      return NULL;
    }  

  return m->primary;
}

static const gchar *gj_gtk_msg_get_secondary(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_secondary: Msg was NULL");
      return NULL;
    }

  return m->secondary;
}

gboolean gj_gtk_msg_get_modal(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_modal: Msg was NULL");
      return FALSE;
    }

  return m->modal;
}

static gint gj_gtk_msg_get_type(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_type: Msg was NULL");
      return -1;
    }    

  return m->type;
}

static MsgCb gj_gtk_msg_get_cb(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_cb: Msg was NULL");
      return NULL;
    } 
    
  return m->cb;
}

gint gj_gtk_msg_get_response(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_get_response: Msg was NULL");
      return -1;
    }
  
  return m->response;
}

/* sets */
gboolean gj_gtk_msg_set_id(Msg m, guint id)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_id: Msg was NULL");
      return FALSE;
    }
  
  m->id = id;
  return TRUE;
}

gboolean gj_gtk_msg_set_userdata(Msg m, void *userdata)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_userdata: Msg was NULL");
      return FALSE;
    }
  
  m->userdata = userdata;
  return TRUE;
}

gboolean gj_gtk_msg_set_title(Msg m, const gchar *title)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_title: Msg was NULL");
      return FALSE;
    }
  
  if(title != NULL)
    {
      g_warning("gj_gtk_msg_set_title: title was NULL");
      return FALSE;
    }

  if(m->title != NULL)
    {
      g_free(m->title);
      m->title = NULL;
    }

  m->title = g_strdup(title);
  
  return TRUE;
}

gboolean gj_gtk_msg_set_modal(Msg m, gboolean modal)
{
  return TRUE;
}

/*
 * actual function to modify the msg gui  
 */
gboolean gj_gtk_msg_load(Msg m)
{
  GladeXML *xml = NULL;

  if(m == NULL)
    {
      g_warning("gj_gtk_msg_load: Msg was NULL");
      return FALSE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "msg",
			      NULL,
			      "msg", &m->controls.window,
			      "button_ok", &m->controls.button_ok,
			      "button_cancel", &m->controls.button_cancel,
			      "button_yes", &m->controls.button_yes,
			      "button_no", &m->controls.button_no,
			      "button_close", &m->controls.button_close,
			      "label_primary", &m->controls.label_primary,
			      "label_secondary", &m->controls.label_secondary,
			      "image", &m->controls.image,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "msg", "response", on_msg_response,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  /* check window is created ok */
  if(m->controls.window == NULL)
    {
      g_warning("gj_gtk_msg_load: failed to create message box, window was NULL");
      return FALSE;
    }

  gj_gtk_msg_setup(m);

  return TRUE;
}

gboolean gj_gtk_msg_unload(Msg m)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_unload: Msg was NULL");
      return FALSE;
    }

  gtk_widget_destroy(m->controls.window);
  m->controls.window = NULL;

  gj_gtk_msg_free(m);

  return TRUE;
}

static gboolean gj_gtk_msg_setup(Msg m)
{
  gchar *text = NULL;
  static guint id = -1;

  if(m == NULL)
    {
      g_warning("gj_gtk_msg_setup: Msg was NULL");
      return FALSE;
    }

  /* set text */
  g_message("gj_gtk_msg_setup: setting primary...");
  text = g_strdup_printf("<span size=\"11000\" weight=\"heavy\">%s</span>", gj_gtk_msg_get_primary(m));
  gtk_label_set_markup(GTK_LABEL(m->controls.label_primary), text);
  g_free(text);

  if(gj_gtk_msg_get_secondary(m) != NULL)
    {
      g_message("gj_gtk_msg_setup: setting secondary...");
      text = g_strdup_printf("%s", gj_gtk_msg_get_secondary(m));
      gtk_label_set_markup(GTK_LABEL(m->controls.label_secondary), text);
      g_free(text);
    }

  /* set title prop */
/*   g_message("gj_gtk_msg_setup: setting window title to:'%s'...", gj_gtk_msg_get_title(m)); */
/*   gtk_window_set_title(GTK_WINDOW(m->controls.window), gj_gtk_msg_get_title(m)); */
  
  /* set modal prop */
  g_message("gj_gtk_msg_setup: setting window to %s", gj_gtk_msg_get_modal(m)?"be modal":"NOT be modal");
  gtk_window_set_modal(GTK_WINDOW(m->controls.window), gj_gtk_msg_get_modal(m));
  
  /* set icon - TODO */
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_ERROR)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image),GTK_STOCK_DIALOG_ERROR,GTK_ICON_SIZE_DIALOG);
  else if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_EXCLAIMATION)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image),GTK_STOCK_DIALOG_WARNING,GTK_ICON_SIZE_DIALOG);
  else if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_QUESTION)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image),GTK_STOCK_DIALOG_QUESTION,GTK_ICON_SIZE_DIALOG);
  else if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_ICON_INFORMATION)
    gtk_image_set_from_stock(GTK_IMAGE(m->controls.image),GTK_STOCK_DIALOG_INFO,GTK_ICON_SIZE_DIALOG);

  id = gj_gtk_msg_get_id(m);

  g_message("gj_gtk_msg_setup: using id:%d", id);
  g_message("gj_gtk_msg_setup: using userdata: 0x%.8x",(gint)gj_gtk_msg_get_userdata(m));
  g_message("gj_gtk_msg_setup: using callback: 0x%.8x",(gint)gj_gtk_msg_get_cb(m));

  g_message("gj_gtk_msg_setup: setting up buttons...");

  /* set buttons */
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_OK)
    {
      g_message("gj_gtk_msg_setup: using 'ok' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_ok), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_ok));
    }
    
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_CANCEL)
    {
      g_message("gj_gtk_msg_setup: using 'cancel' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_cancel), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_cancel));
    }
  
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_YES)
    {
      g_message("gj_gtk_msg_setup: using 'yes' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_yes), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_yes));
    }

  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_NO)
    {
      g_message("gj_gtk_msg_setup: using 'no' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_no), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_no));
    }
  
  if(gj_gtk_msg_get_type(m) & GJ_GTK_MSG_BUTTON_CLOSE)
    {
      g_message("gj_gtk_msg_setup: using 'close' button");
      gtk_widget_set_sensitive(GTK_WIDGET(m->controls.button_close), TRUE);
      gtk_widget_show(GTK_WIDGET(m->controls.button_close));
    }

  g_message("gj_gtk_msg_setup: setting up buttons complete!");
  gtk_widget_show(m->controls.window);

  gdk_window_set_decorations (GDK_WINDOW(m->controls.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  return TRUE;
}

gboolean gj_gtk_msg_set_transient_window(Msg m, GtkWindow *parent_window)
{
  if(m == NULL)
    {
      g_warning("gj_gtk_msg_set_transient_window: Msg was NULL");
      return FALSE;
    }

  gtk_window_set_transient_for(GTK_WINDOW(m->controls.window), 
			       GTK_WINDOW(parent_window));

  g_message("gj_gtk_msg_set_transient_window: setting transient parent window:0x%.8x for Msg with id:%d", 
	    (gint)parent_window, gj_gtk_msg_get_id(m));

  return TRUE;
}

static gboolean gj_gtk_msg_function(Msg m, gint type)
{
  if(type < 0)
    {
      g_warning("%s: type:%d was invalid (<1)", __FUNCTION__, type);
      return FALSE;
    }

  if(m == NULL)
    {
      g_warning("%s: Msg was NULL", __FUNCTION__);
      return FALSE;
    }

  /* set buttons */
  if(type & GJ_GTK_MSG_BUTTON_OK)
    g_message("%s: 'ok' button pressed", __FUNCTION__);
  else if(type & GJ_GTK_MSG_BUTTON_CANCEL)
    g_message("%s: 'cancel' button pressed", __FUNCTION__);
  else if(type & GJ_GTK_MSG_BUTTON_YES)
    g_message("%s: 'yes' button pressed", __FUNCTION__);
  else if(type & GJ_GTK_MSG_BUTTON_NO)
    g_message("%s: 'no' button pressed", __FUNCTION__);
  else if(type & GJ_GTK_MSG_BUTTON_CLOSE)
    g_message("%s: 'close' button pressed", __FUNCTION__);
  else if(type & GJ_GTK_MSG_BUTTON_DELETE_EVENT)
    g_message("%s: delete event occured", __FUNCTION__);

  if(gj_gtk_msg_get_cb(m) != NULL)
    {
      g_message("%s: using callback:0x%.8x, with userdata:0x%.8x", 
		__FUNCTION__, (gint)gj_gtk_msg_get_cb(m), (gint)gj_gtk_msg_get_userdata);

      /* call callback function */
      (gj_gtk_msg_get_cb(m))(type, gj_gtk_msg_get_userdata(m));
    }
     
  return TRUE;
}

/*
 * actual gui functions
 */
static void on_msg_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  Msg m = NULL;
    
  if((m = gj_gtk_msg_find_by_widget(GTK_WIDGET(dialog))) == NULL)
    {
      g_warning("%s: could not find window from widget:0x%.8x", __FUNCTION__, (gint)dialog);
      return;
    }

  switch(response) 
    {
    case GTK_RESPONSE_OK:
      gj_gtk_msg_function(m, GJ_GTK_MSG_BUTTON_OK);
      break;

    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
      gj_gtk_msg_function(m, GJ_GTK_MSG_BUTTON_DELETE_EVENT);
      break;

    case GTK_RESPONSE_CANCEL:
      gj_gtk_msg_function(m, GJ_GTK_MSG_BUTTON_CANCEL);
      break;

    case GTK_RESPONSE_CLOSE:
      gj_gtk_msg_function(m, GJ_GTK_MSG_BUTTON_CLOSE);
      break;

    case GTK_RESPONSE_YES:
      gj_gtk_msg_function(m, GJ_GTK_MSG_BUTTON_YES);
      break;

    case GTK_RESPONSE_NO:
      gj_gtk_msg_function(m, GJ_GTK_MSG_BUTTON_NO);
      break;

    case GTK_RESPONSE_APPLY:
      break;
    }

  gj_gtk_msg_unload(m);
}


#ifdef __cplusplus
}
#endif
