/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_my_information.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/30 20:38:27 $
 * 
 * $Revision: 1.20 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef HAVE_GNOME
#include <gnome.h> 
#endif

#include "gj_main.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"
#include "gj_config.h"

#include "gj_gtk_my_information.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_support.h"


struct 
{
  GtkWidget *window;
  
  GtkWidget *entry_full_name;
  GtkWidget *entry_name_given;
  GtkWidget *entry_name_family;
  GtkWidget *entry_name_nickname;

  GtkWidget *entry_email;
  GtkWidget *entry_url;
  GtkWidget *entry_tel;
  
  GtkWidget *entry_street;
  GtkWidget *entry_extadd;
  GtkWidget *entry_locality;
  GtkWidget *entry_region;
  GtkWidget *entry_pcode;
  GtkWidget *entry_country;

  GtkWidget *entry_org_orgname;
  GtkWidget *entry_org_orgunit;
  GtkWidget *entry_title;
  GtkWidget *entry_role;
  
  GtkWidget *calendar_dob;
  GtkWidget *textview_description;
  
  GtkWidget *checkbutton_publish_in_jud;

} ctlMI;


static gboolean gj_gtk_mi_set();
static gboolean gj_gtk_mi_get();


/*
 * internal callbacks 
 */
static void gj_gtk_mi_vcard_response(guint event_id, guint related_event_id);


/*
 * gui callbacks
 */
static void on_my_information_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * code
 */
gboolean gj_gtk_mi_load()
{
  GladeXML *xml = NULL;

  if(ctlMI.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlMI.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "my_information",
			      NULL,
			      "my_information", &ctlMI.window,
			      "entry_full_name", &ctlMI.entry_full_name,
			      "entry_name_given", &ctlMI.entry_name_given,
			      "entry_name_family", &ctlMI.entry_name_family,
			      "entry_name_nickname", &ctlMI.entry_name_nickname,
			      "entry_email", &ctlMI.entry_email,
			      "entry_url", &ctlMI.entry_url,
			      "entry_tel", &ctlMI.entry_tel,
			      "entry_street", &ctlMI.entry_street,
			      "entry_extadd", &ctlMI.entry_extadd,
			      "entry_locality", &ctlMI.entry_locality,
			      "entry_region", &ctlMI.entry_region,
			      "entry_pcode", &ctlMI.entry_pcode,
			      "entry_country", &ctlMI.entry_country,
			      "entry_org_orgname", &ctlMI.entry_org_orgname,
			      "entry_org_orgunit", &ctlMI.entry_org_orgunit,
			      "entry_title", &ctlMI.entry_title,
			      "entry_role", &ctlMI.entry_role,
			      "calendar_dob", &ctlMI.calendar_dob,
			      "textview_description", &ctlMI.textview_description,
			      "checkbutton_publish_in_jud", &ctlMI.checkbutton_publish_in_jud,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "my_information", "response", on_my_information_response,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(ctlMI.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU);

  /* get my information */
  gj_gtk_mi_get();

  return TRUE;
}

gboolean gj_gtk_mi_unload()
{
  if(ctlMI.window == NULL)
    return TRUE;

  gtk_widget_destroy(ctlMI.window);

  ctlMI.window = NULL;

  ctlMI.entry_full_name = NULL;
  ctlMI.entry_name_given = NULL;
  ctlMI.entry_name_family = NULL;
  ctlMI.entry_name_nickname = NULL;

  ctlMI.entry_email = NULL;
  ctlMI.entry_url = NULL;
  ctlMI.entry_tel = NULL;
  
  ctlMI.entry_street = NULL;
  ctlMI.entry_extadd = NULL;
  ctlMI.entry_locality = NULL;
  ctlMI.entry_region = NULL;
  ctlMI.entry_pcode = NULL;
  ctlMI.entry_country = NULL;

  ctlMI.entry_org_orgname = NULL;
  ctlMI.entry_org_orgunit = NULL;
  ctlMI.entry_title = NULL;
  ctlMI.entry_role = NULL;
  
  ctlMI.calendar_dob = NULL;
  ctlMI.textview_description = NULL;
  
  ctlMI.checkbutton_publish_in_jud = NULL;
  
  return TRUE;
}

static gboolean gj_gtk_mi_set()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter bufStart;
  GtkTextIter bufStop;

  G_CONST_RETURN gchar *full_name = NULL;
  G_CONST_RETURN gchar *name_given = NULL;
  G_CONST_RETURN gchar *name_family = NULL;
  G_CONST_RETURN gchar *name_nickname = NULL;

  G_CONST_RETURN gchar *email = NULL;
  G_CONST_RETURN gchar *url = NULL;
  G_CONST_RETURN gchar *tel = NULL;
  
  G_CONST_RETURN gchar *street = NULL;
  G_CONST_RETURN gchar *extadd = NULL;
  G_CONST_RETURN gchar *locality = NULL;
  G_CONST_RETURN gchar *region = NULL;
  G_CONST_RETURN gchar *pcode = NULL;
  G_CONST_RETURN gchar *country = NULL;
  
  G_CONST_RETURN gchar *org_orgname = NULL;
  G_CONST_RETURN gchar *org_orgunit = NULL;
  G_CONST_RETURN gchar *title = NULL;
  G_CONST_RETURN gchar *role = NULL;

  guint dob_day = 0;
  guint dob_month = 0;
  guint dob_year = 0;

  gchar *dob = NULL;
  gchar *description = NULL;
  
  gboolean publish_in_jud = FALSE;

  /* get values */
  if((publish_in_jud = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMI.checkbutton_publish_in_jud))) == FALSE)
    {
      g_message("gj_gtk_mi_save: publishing information on 'personal' page in Jabber User Directory!");

      full_name = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_full_name));
      name_given = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_name_given)); 
      name_family = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_name_family));
      name_nickname = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_name_nickname));
      email = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_email));
      url = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_url));
      tel = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_tel));
    }

  street = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_street));
  extadd = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_extadd));
  locality = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_locality));
  region = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_region));
  pcode = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_pcode));
  country = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_country));

  org_orgname = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_org_orgname));
  org_orgunit = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_org_orgunit));
  title = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_title));
  role = gtk_entry_get_text(GTK_ENTRY(ctlMI.entry_role));

  gtk_calendar_get_date(GTK_CALENDAR(ctlMI.calendar_dob), &dob_year, &dob_month, &dob_day);
  dob = g_strdup_printf("%d-%d-%d", dob_year, ++dob_month, dob_day);

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMI.textview_description));
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStart, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStop, -1);
  description = gtk_text_buffer_get_text(buffer,&bufStart,&bufStop,TRUE);

  /* save vcard */
  gj_iq_request_vcard_change(full_name,
			     name_given,
			     name_family,
			     name_nickname, /* nickname */
			     url,
			     tel,
			     email,
			     description,
			     dob,
			     street,
			     extadd,
			     locality,
			     region,
			     pcode,
			     country,
			     org_orgname,
			     org_orgunit,
			     title,
			     role,
			     gj_gtk_mi_set_response,
			     NULL);
  
  /* print to log changes... */
  g_message("gj_gtk_mi_save: saving VCard:\n\t"
	    "name - full:'%s'\n\t"
	    "name - given:'%s'\n\t"
	    "name - family:'%s'\n\t"
	    "name - nickname:'%s'\n\t"
	    "url:'%s'\n\t"
	    "telephone:'%s'\n\t"
	    "email:'%s'\n\t"
	    "description:'%s'\n\t"
	    "dob:'%s'\n\t"
	    "street:'%s'\n\t"
	    "extadd:'%s'\n\t"
	    "locality:'%s'\n\t"
	    "region:'%s'\n\t"
	    "pcode:'%s'\n\t"
	    "country:'%s'\n\t"
	    "org_name:'%s'\n\t"
	    "org_unit:'%s'\n\t"
	    "title:'%s'\n\t"
	    "role:'%s'",
	    full_name,
	    name_given,
	    name_family,
	    name_nickname,
	    url,
	    tel,
	    email,
	    description,
	    dob,
	    street,
	    extadd,
	    locality,
	    region,
	    pcode,
	    country,
	    org_orgname,
	    org_orgunit,
	    title,
	    role);

  /* clean up */
  if(dob != NULL) g_free(dob);
  if(description != NULL) g_free(description);

  return TRUE;
}

static gboolean gj_gtk_mi_get()
{
  gchar *jid = NULL;

  /* get information from config */
  jid = g_strdup_printf("%s@%s", gj_config_get_cs_username(), gj_config_get_cs_server());
  gj_iq_request_vcard(jid, gj_gtk_mi_vcard_response, NULL);

  /* cleanup */
  if(jid != NULL) g_free(jid);

  return TRUE;
}

/*
 * internal callbacks
 */
void gj_gtk_mi_set_response(guint event_id, guint related_event_id)
{
  /* !!!!!!!! NEED TO CHECK FOR ERRORS here */
  GjInfoquery iq = NULL;
  GjInfoquery iq_related = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  g_message("%s: received related_event_id:%d", __FUNCTION__, event_id);

  if((iq = gj_iq_find(event_id)) == NULL)
    {
      g_warning("%s: failed to find IQ based on event_id:%d", __FUNCTION__, event_id);
      return;
    }

  if((iq_related = gj_iq_find(related_event_id)) == NULL)
    {
      g_warning("%s: failed to find IQ based on related_event_id:%d", __FUNCTION__, event_id);
      return;
    }

  node = gj_iq_get_xml(iq_related);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *errorCode = NULL;
      xmlChar *errorMessage = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);

	  gj_iq_free_by_event_id(event_id);
	  return;
	}

      if((errorCode = gj_parser_find_by_attr_and_ret(node, (guchar*)"code")) != NULL)
	{
	  detail = g_strdup_printf("Error:%s",errorCode);
	}

      if((errorMessage = gj_parser_find_by_node_and_ret(node, (guchar*)"error")) != NULL) 
	{
	  if(detail != NULL)
	    detail = g_strdup_printf("%s - %s",detail,errorMessage);
	}

      g_warning("%s: IQ type is error, event_id:%d", __FUNCTION__, event_id);

     /* clean up */
      g_free(detail);
    }

  /* clean up */
  gj_iq_free_all_related_by_event_id(event_id);
}

static void gj_gtk_mi_vcard_response(guint event_id, guint related_event_id)
{
  GtkTextBuffer *buffer = NULL;

  xmlChar *tmp = NULL;
  GjInfoquery iq = NULL;

  g_message("%s: response with event_id:%d, related event id:%d", __FUNCTION__, event_id, related_event_id);

  if((iq = gj_iq_find(related_event_id)) == NULL)
    {
      g_warning("%s: failed to find IQ based on related event id:%d", __FUNCTION__, event_id);
      return;
    }

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"fn");
  gj_gtk_set_entry_with_args(ctlMI.entry_full_name, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"given");
  gj_gtk_set_entry_with_args(ctlMI.entry_name_given, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"family");
  gj_gtk_set_entry_with_args(ctlMI.entry_name_family, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"nickname");
  gj_gtk_set_entry_with_args(ctlMI.entry_name_nickname, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"url");
  gj_gtk_set_entry_with_args(ctlMI.entry_url, "%s",tmp?(gchar*)tmp:"");
  
  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"tel");
  gj_gtk_set_entry_with_args(ctlMI.entry_tel, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"email");
  gj_gtk_set_entry_with_args(ctlMI.entry_email, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"desc");
  if(tmp != NULL && xmlStrlen(tmp) > 0)
    {
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMI.textview_description));
      gtk_text_buffer_set_text(buffer,(gchar*)tmp,-1);
    }

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"bday");
  if(tmp != NULL && xmlStrlen(tmp) > 0)
    {
      /* NOTE: this must comply with ISO 8601 which shows the date
	       in the format YYYY-MM-DD
      */
      gchar **vDate = NULL;
      vDate = g_strsplit((gchar*)tmp,"-",4);
     
      if(vDate != NULL && vDate[0] != NULL && vDate[1] != NULL && vDate[2] != NULL)
	{
	  gtk_calendar_select_day(GTK_CALENDAR(ctlMI.calendar_dob), atoi(vDate[2]));
	  gtk_calendar_select_month(GTK_CALENDAR(ctlMI.calendar_dob), atoi(vDate[1])-1, atoi(vDate[0]));
				    
	  g_strfreev(vDate);
	}
      else
	g_warning("%s: could not get birth date from:'%s' on vcard", __FUNCTION__, tmp);
    }

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"street");
  gj_gtk_set_entry_with_args(ctlMI.entry_street, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"extadd");
  gj_gtk_set_entry_with_args(ctlMI.entry_extadd, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"locality");
  gj_gtk_set_entry_with_args(ctlMI.entry_locality, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"region");
  gj_gtk_set_entry_with_args(ctlMI.entry_region, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"pcode");
  gj_gtk_set_entry_with_args(ctlMI.entry_pcode, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"country");
  gj_gtk_set_entry_with_args(ctlMI.entry_country, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"orgname");
  gj_gtk_set_entry_with_args(ctlMI.entry_org_orgname, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"orgunit");
  gj_gtk_set_entry_with_args(ctlMI.entry_org_orgunit, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"title");
  gj_gtk_set_entry_with_args(ctlMI.entry_title, "%s",tmp?(gchar*)tmp:"");

  tmp = gj_parser_find_by_node_and_ret(gj_iq_get_xml(iq), (guchar*)"role");
  gj_gtk_set_entry_with_args(ctlMI.entry_role, "%s",tmp?(gchar*)tmp:"");

  /* clean up */
  gj_iq_free_all_related_by_event_id(related_event_id);
}

/*
 * GUI events
 */
static void on_my_information_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_APPLY:
      gj_gtk_mi_set();
      return;

    case GTK_RESPONSE_OK:
      gj_gtk_mi_set();
      break;

    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
      break;
    }

  gj_gtk_mi_unload();
}


#ifdef __cplusplus
}
#endif
