/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_my_preferences.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/22 13:44:59 $
 * 
 * $Revision: 1.25 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_roster.h"
#include "gj_config.h"
#include "gj_me.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_my_preferences.h"
#include "gj_gtk_docklet.h"
#include "gj_gtk_support.h"


struct 
{
  GtkWidget *window;

  GtkWidget *checkbutton_hide_offline_contacts;
  GtkWidget *checkbutton_hide_offline_group;
  GtkWidget *checkbutton_hide_agents;
  GtkWidget *checkbutton_hide_contact_presence_details;

  GtkWidget *radiobutton_double_click_for_chat;
  GtkWidget *radiobutton_double_click_for_message;

  GtkWidget *checkbutton_open_window_on_new_message;
  GtkWidget *checkbutton_raise_window_on_message;
  GtkWidget *checkbutton_hide_time;
  GtkWidget *checkbutton_reverse_text_direction;
  GtkWidget *checkbutton_use_smileys;

  GtkWidget *checkbutton_use_spell_checker;

  GtkWidget *checkbutton_use_auto_away;
  GtkWidget *spinbutton_auto_away_after;
  GtkWidget *spinbutton_auto_xa_after;
  GtkWidget *textview_auto_away_status;
  GtkWidget *textview_auto_xa_status;

  GtkWidget *entry_browse_log_location;

  GtkWidget *file_selector;

  GtkWidget *button_browse_sound_on_user_online;
  GtkWidget *button_browse_sound_on_user_offline;
  GtkWidget *button_browse_sound_on_receive_message;
  GtkWidget *button_browse_sound_on_receive_chat;
  GtkWidget *button_browse_sound_on_receive_headline;
  GtkWidget *button_browse_sound_on_receive_error;

  GtkWidget *table_sound;
  GtkWidget *checkbutton_sound_enabled;
  GtkWidget *checkbutton_sound_on_user_online;
  GtkWidget *checkbutton_sound_on_user_offline;
  GtkWidget *checkbutton_sound_on_receive_message;
  GtkWidget *checkbutton_sound_on_receive_chat;
  GtkWidget *checkbutton_sound_on_receive_headline;
  GtkWidget *checkbutton_sound_on_receive_error;
  GtkWidget *checkbutton_sound_beep_if_no_file;

  GtkWidget *entry_sound_file_on_user_online;
  GtkWidget *entry_sound_file_on_user_offline;
  GtkWidget *entry_sound_file_on_receive_message;
  GtkWidget *entry_sound_file_on_receive_chat;
  GtkWidget *entry_sound_file_on_receive_headline;
  GtkWidget *entry_sound_file_on_receive_error;
  GtkWidget *entry_sound_beep_if_no_file;

  GtkWidget *vbox_docklet;
  GtkWidget *checkbutton_use_docklet;

  GtkWidget *button_browse_log_location;
  GtkWidget *button_browse_history_location;
  GtkWidget *checkbutton_use_history;
  GtkWidget *entry_browse_history_location;

  GtkWidget *notebook;
    
} ctlMPF;


#define BROWSE_SOUND_ON_USER_ONLINE 1
#define BROWSE_SOUND_ON_USER_OFFLINE 2
#define BROWSE_SOUND_ON_RECEIVE_MESSAGE 3
#define BROWSE_SOUND_ON_RECEIVE_CHAT 4
#define BROWSE_SOUND_ON_RECEIVE_HEADLINE 5
#define BROWSE_SOUND_ON_RECEIVE_ERROR 6
#define BROWSE_LOG_LOCATION 7
#define BROWSE_HISTORY_LOCATION 8


static gboolean gj_gtk_mpf_get();
static gboolean gj_gtk_mpf_set();

static void gj_gtk_mpf_store_filename(GtkFileSelection *file_selector, gpointer user_data);
static void gj_gtk_mpf_create_file_selection(guint sound);

/*
 * GUI functions
 */
static void on_checkbutton_mpf_hide_offline_contacts_toggled(GtkToggleButton *togglebutton, gpointer user_data);

static void on_button_mpf_browse_sound_on_user_online_clicked(GtkButton *button, gpointer user_data);
static void on_button_mpf_browse_sound_on_user_offline_clicked(GtkButton *button, gpointer user_data);
static void on_button_mpf_browse_sound_on_receive_message_clicked(GtkButton *button, gpointer user_data);
static void on_button_mpf_browse_sound_on_receive_chat_clicked(GtkButton *button, gpointer user_data);
static void on_button_mpf_browse_sound_on_receive_headline_clicked(GtkButton *button, gpointer user_data);
static void on_button_mpf_browse_sound_on_receive_error_clicked(GtkButton *button, gpointer user_data);

static void on_button_mpf_browse_log_location_clicked(GtkButton *button, gpointer user_data);

static void on_checkbutton_mpf_use_history_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_button_mpf_browse_history_location_clicked(GtkButton *button, gpointer user_data);

/* static void on_checkbutton_mpf_sound_enabled_toggled(GtkToggleButton *togglebutton, gpointer user_data); */
static void on_checkbutton_mpf_hide_offline_contacts_toggled(GtkToggleButton *togglebutton, gpointer user_data);

static void on_my_preferences_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * code
 */
gboolean gj_gtk_mpf_load()
{
  GladeXML *xml = NULL;
/*   GtkWidget *page = NULL; */

  if(ctlMPF.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlMPF.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "my_preferences",
			      NULL,
			      "my_preferences", &ctlMPF.window,
			      "checkbutton_hide_offline_contacts", &ctlMPF.checkbutton_hide_offline_contacts, 
			      "checkbutton_hide_offline_group", &ctlMPF.checkbutton_hide_offline_group,
			      "checkbutton_hide_agents", &ctlMPF.checkbutton_hide_agents,
			      "checkbutton_hide_contact_presence_details", &ctlMPF.checkbutton_hide_contact_presence_details,
			      "radiobutton_double_click_for_chat", &ctlMPF.radiobutton_double_click_for_chat,
			      "radiobutton_double_click_for_message", &ctlMPF.radiobutton_double_click_for_message,
			      
			      "checkbutton_open_window_on_new_message", &ctlMPF.checkbutton_open_window_on_new_message,
			      "checkbutton_raise_window_on_message", &ctlMPF.checkbutton_raise_window_on_message ,
			      "checkbutton_hide_time", &ctlMPF.checkbutton_hide_time, 
			      "checkbutton_reverse_text_direction", &ctlMPF.checkbutton_reverse_text_direction,
			      "checkbutton_use_smileys", &ctlMPF.checkbutton_use_smileys,
			      "checkbutton_use_spell_checker", &ctlMPF.checkbutton_use_spell_checker,
			      "checkbutton_use_auto_away", &ctlMPF.checkbutton_use_auto_away,
			      "spinbutton_auto_away_after", &ctlMPF.spinbutton_auto_away_after,
			      "spinbutton_auto_xa_after", &ctlMPF.spinbutton_auto_xa_after,
			      "textview_auto_away_status", &ctlMPF.textview_auto_away_status,
			      "textview_auto_xa_status", &ctlMPF.textview_auto_xa_status,
			      "entry_browse_log_location", &ctlMPF.entry_browse_log_location,
			      "button_browse_sound_on_user_online", &ctlMPF.button_browse_sound_on_user_online,
			      "button_browse_sound_on_user_offline", &ctlMPF.button_browse_sound_on_user_offline, 
			      "button_browse_sound_on_receive_message", &ctlMPF.button_browse_sound_on_receive_message,
			      "button_browse_sound_on_receive_chat", &ctlMPF.button_browse_sound_on_receive_chat,
			      "button_browse_sound_on_receive_headline", &ctlMPF.button_browse_sound_on_receive_headline,
			      "button_browse_sound_on_receive_error", &ctlMPF.button_browse_sound_on_receive_error,
			      "table_sound", &ctlMPF.table_sound,
/* 			      "checkbutton_sound_enabled", &ctlMPF.checkbutton_sound_enabled, */
			      "checkbutton_sound_on_user_online", &ctlMPF.checkbutton_sound_on_user_online, 
			      "checkbutton_sound_on_user_offline", &ctlMPF.checkbutton_sound_on_user_offline,
			      "checkbutton_sound_on_receive_message", &ctlMPF.checkbutton_sound_on_receive_message,
			      "checkbutton_sound_on_receive_chat", &ctlMPF.checkbutton_sound_on_receive_chat,
			      "checkbutton_sound_on_receive_headline", &ctlMPF.checkbutton_sound_on_receive_headline,
			      "checkbutton_sound_on_receive_error", &ctlMPF.checkbutton_sound_on_receive_error,
/* 			      "checkbutton_sound_beep_if_no_file", &ctlMPF.checkbutton_sound_beep_if_no_file, */
			      "entry_browse_sound_on_user_online", &ctlMPF.entry_sound_file_on_user_online,
			      "entry_browse_sound_on_user_offline", &ctlMPF.entry_sound_file_on_user_offline,
			      "entry_browse_sound_on_receive_message", &ctlMPF.entry_sound_file_on_receive_message,
			      "entry_browse_sound_on_receive_chat", &ctlMPF.entry_sound_file_on_receive_chat,
			      "entry_browse_sound_on_receive_headline", &ctlMPF.entry_sound_file_on_receive_headline,
			      "entry_browse_sound_on_receive_error", &ctlMPF.entry_sound_file_on_receive_error,
			      "checkbutton_use_docklet", &ctlMPF.checkbutton_use_docklet,
			      "vbox_docklet", &ctlMPF.vbox_docklet,
			      "button_browse_log_location", &ctlMPF.button_browse_log_location,
			      "button_browse_history_location", &ctlMPF.button_browse_history_location,
			      "checkbutton_use_history", &ctlMPF.checkbutton_use_history,
			      "entry_browse_history_location", &ctlMPF.entry_browse_history_location,
			      "notebook",  &ctlMPF.notebook,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "my_preferences", "response", on_my_preferences_response,
		       "checkbutton_hide_offline_contacts", "toggled", on_checkbutton_mpf_hide_offline_contacts_toggled,
		       "button_browse_sound_on_user_online", "clicked", on_button_mpf_browse_sound_on_user_online_clicked,
		       "button_browse_sound_on_user_offline", "clicked", on_button_mpf_browse_sound_on_user_offline_clicked,
		       "button_browse_sound_on_receive_message", "clicked", on_button_mpf_browse_sound_on_receive_message_clicked,
		       "button_browse_sound_on_receive_chat", "clicked", on_button_mpf_browse_sound_on_receive_chat_clicked,
		       "button_browse_sound_on_receive_headline", "clicked", on_button_mpf_browse_sound_on_receive_headline_clicked,
		       "button_browse_sound_on_receive_error", "clicked",on_button_mpf_browse_sound_on_receive_error_clicked,
		       "button_browse_log_location", "clicked", on_button_mpf_browse_log_location_clicked,
		       "checkbutton_use_history", "toggled", on_checkbutton_mpf_use_history_toggled,
		       "button_browse_history_location", "clicked", on_button_mpf_browse_history_location_clicked,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(ctlMPF.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU);

#ifdef WIN32
#if 0
  /* if windows hide the notification tab */
  page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(ctlMPF.notebook), 3);
  gtk_widget_hide(page);
#else
  gtk_widget_hide(ctlMPF.vbox_docklet);
#endif
#endif

  /* get settings */
  gj_gtk_mpf_get();

  return TRUE;
}

gboolean gj_gtk_mpf_unload()
{
  if(ctlMPF.window == NULL)
    return TRUE;

  gtk_widget_destroy(ctlMPF.window);
  ctlMPF.window = NULL;

  return TRUE;
}

static gboolean gj_gtk_mpf_set()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter bufStart;
  GtkTextIter bufStop;

  /* get values */
  gj_config_set_mpf_hide_offline_contacts(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_offline_contacts)));
  gj_config_set_mpf_hide_offline_group(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_offline_group)));
  gj_config_set_mpf_hide_agents(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_agents)));
  gj_config_set_mpf_hide_contact_presence_details(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_contact_presence_details)));

  gj_config_set_mpf_double_click_for_chat(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.radiobutton_double_click_for_chat)));

  gj_config_set_mpf_open_window_on_new_message(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_open_window_on_new_message)));
  gj_config_set_mpf_raise_window_on_message(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_raise_window_on_message)));
  gj_config_set_mpf_hide_time(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_time)));
  gj_config_set_mpf_reverse_text_direction(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_reverse_text_direction)));
  gj_config_set_mpf_use_smileys(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_smileys)));

  gj_config_set_mpf_use_spell_checker(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_spell_checker)));

  gj_config_set_mpf_use_auto_away(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_auto_away)));
  gj_config_set_mpf_auto_away_after(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlMPF.spinbutton_auto_away_after)));
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMPF.textview_auto_away_status));
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStart, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStop, -1);
  gj_config_set_mpf_auto_away_status(gtk_text_buffer_get_text(buffer,&bufStart,&bufStop,TRUE));

  gj_config_set_mpf_auto_xa_after(gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlMPF.spinbutton_auto_xa_after)));
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMPF.textview_auto_xa_status));
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStart, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStop, -1);
  gj_config_set_mpf_auto_xa_status(gtk_text_buffer_get_text(buffer,&bufStart,&bufStop,TRUE));

  gj_config_set_mpf_sound_on_user_online(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_user_online)));
  gj_config_set_mpf_sound_on_user_offline(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_user_offline)));
  gj_config_set_mpf_sound_on_receive_message(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_message)));
  gj_config_set_mpf_sound_on_receive_chat(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_chat)));
  gj_config_set_mpf_sound_on_receive_headline(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_headline)));
  gj_config_set_mpf_sound_on_receive_error(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_error)));

  gj_config_set_mpf_sound_file_on_user_online((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_user_online)));
  gj_config_set_mpf_sound_file_on_user_offline((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_user_offline)));
  gj_config_set_mpf_sound_file_on_receive_message((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_message)));
  gj_config_set_mpf_sound_file_on_receive_chat((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_chat)));
  gj_config_set_mpf_sound_file_on_receive_headline((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_headline)));
  gj_config_set_mpf_sound_file_on_receive_error((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_error)));
  gj_config_set_mpf_use_docklet(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_docklet)));

  gj_config_set_mpf_log_location((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_browse_log_location)));
  gj_config_set_mpf_history_location((gchar*)gtk_entry_get_text(GTK_ENTRY(ctlMPF.entry_browse_history_location)));
  gj_config_set_mpf_use_history(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_history)));

  if(gj_config_write() == FALSE)
    return FALSE;

  if(gj_me_get_is_connected() == TRUE)
    gj_gtk_roster_model_reload();

  if(gj_config_get_mpf_use_docklet() == TRUE)
    gj_gtk_docklet_load();
  else
    gj_gtk_docklet_unload();

  return TRUE;
}

static gboolean gj_gtk_mpf_get()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter bufStart;

  if(gj_config_read() == FALSE)
    g_message("gj_gtk_mpf_get: config read failed, continuing anyway...");

  /* set up settings */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_offline_contacts), gj_config_get_mpf_hide_offline_contacts());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_offline_group), gj_config_get_mpf_hide_offline_group());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_agents), gj_config_get_mpf_hide_agents());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_contact_presence_details), gj_config_get_mpf_hide_contact_presence_details());

  if(gj_config_get_mpf_double_click_for_chat() == TRUE)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.radiobutton_double_click_for_chat), TRUE);
  else
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.radiobutton_double_click_for_message), TRUE);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_open_window_on_new_message),
			       gj_config_get_mpf_open_window_on_new_message());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_raise_window_on_message),
			       gj_config_get_mpf_raise_window_on_message());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_hide_time),
			       gj_config_get_mpf_hide_time());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_reverse_text_direction),
			       gj_config_get_mpf_reverse_text_direction());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_smileys),
			       gj_config_get_mpf_use_smileys());

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_spell_checker),
			       gj_config_get_mpf_use_spell_checker());

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_auto_away),
			       gj_config_get_mpf_use_auto_away());
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctlMPF.spinbutton_auto_away_after), (gdouble)gj_config_get_mpf_auto_away_after());
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctlMPF.spinbutton_auto_xa_after), (gdouble)gj_config_get_mpf_auto_xa_after());
  
  if(gj_config_get_mpf_auto_away_status() != NULL && g_utf8_strlen(gj_config_get_mpf_auto_away_status(),-1) > 0)
    {
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMPF.textview_auto_away_status));
      gtk_text_buffer_get_start_iter(buffer, &bufStart);
      gtk_text_buffer_insert(buffer, &bufStart, gj_config_get_mpf_auto_away_status(), -1);
    }

  if(gj_config_get_mpf_auto_xa_status() != NULL && g_utf8_strlen(gj_config_get_mpf_auto_xa_status(),-1) > 0)
    {
      buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMPF.textview_auto_xa_status));
      gtk_text_buffer_get_start_iter(buffer, &bufStart);
      gtk_text_buffer_insert(buffer, &bufStart, gj_config_get_mpf_auto_xa_status(), -1);
    }

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_user_online), gj_config_get_mpf_sound_on_user_online());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_user_offline), gj_config_get_mpf_sound_on_user_offline());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_message), gj_config_get_mpf_sound_on_receive_message());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_chat), gj_config_get_mpf_sound_on_receive_chat());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_headline), gj_config_get_mpf_sound_on_receive_headline());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_sound_on_receive_error), gj_config_get_mpf_sound_on_receive_error());

  if(gj_config_get_mpf_sound_file_on_user_online() && g_utf8_strlen(gj_config_get_mpf_sound_file_on_user_online(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_user_online), gj_config_get_mpf_sound_file_on_user_online());
  if(gj_config_get_mpf_sound_file_on_user_offline() != NULL && g_utf8_strlen(gj_config_get_mpf_sound_file_on_user_offline(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_user_offline), gj_config_get_mpf_sound_file_on_user_offline());
  if(gj_config_get_mpf_sound_file_on_receive_message() != NULL && g_utf8_strlen(gj_config_get_mpf_sound_file_on_receive_message(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_message), gj_config_get_mpf_sound_file_on_receive_message());
  if(gj_config_get_mpf_sound_file_on_receive_chat() != NULL && g_utf8_strlen(gj_config_get_mpf_sound_file_on_receive_chat(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_chat), gj_config_get_mpf_sound_file_on_receive_chat());
  if(gj_config_get_mpf_sound_file_on_receive_headline() != NULL && g_utf8_strlen(gj_config_get_mpf_sound_file_on_receive_headline(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_headline), gj_config_get_mpf_sound_file_on_receive_headline());
  if(gj_config_get_mpf_sound_file_on_receive_error() != NULL && g_utf8_strlen(gj_config_get_mpf_sound_file_on_receive_error(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_error), gj_config_get_mpf_sound_file_on_receive_error());

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_docklet), gj_config_get_mpf_use_docklet());

  if(gj_config_get_mpf_log_location() != NULL && g_utf8_strlen(gj_config_get_mpf_log_location(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_browse_log_location), gj_config_get_mpf_log_location());
  if(gj_config_get_mpf_history_location() != NULL && g_utf8_strlen(gj_config_get_mpf_history_location(), -1) > 0)
    gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_browse_history_location), gj_config_get_mpf_history_location());

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctlMPF.checkbutton_use_history), gj_config_get_mpf_use_history());

  return TRUE;
}

static void gj_gtk_mpf_store_filename(GtkFileSelection *file_selector, gpointer user_data) 
{
   const gchar *selected_filename;
   guint browse = 0;
 
   browse = GPOINTER_TO_INT(user_data);
   selected_filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(ctlMPF.file_selector));

   if(selected_filename == NULL || g_utf8_strlen(selected_filename, -1) < 1)
     {
       g_warning("gj_gtk_mpf_store_filename: not saving filename for browse id:%d, filename was NULL or length was < 1", browse);
       return;
     }

   g_message("gj_gtk_mpf_store_filename: selected filename: %s for browse:%d", selected_filename, browse);

   switch(browse)
     {
     case BROWSE_SOUND_ON_USER_ONLINE:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_user_online), selected_filename);
       break;
     case BROWSE_SOUND_ON_USER_OFFLINE:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_user_offline), selected_filename);
       break;
     case BROWSE_SOUND_ON_RECEIVE_MESSAGE:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_message), selected_filename);
       break;
     case BROWSE_SOUND_ON_RECEIVE_CHAT:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_chat), selected_filename);
       break;
     case BROWSE_SOUND_ON_RECEIVE_HEADLINE:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_headline), selected_filename);
       break;
     case BROWSE_SOUND_ON_RECEIVE_ERROR:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_sound_file_on_receive_error), selected_filename);
       break;
     case BROWSE_LOG_LOCATION:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_browse_log_location), selected_filename);
       break;
     case BROWSE_HISTORY_LOCATION:
       gtk_entry_set_text(GTK_ENTRY(ctlMPF.entry_browse_history_location), selected_filename);
       break;


     default:
       g_warning("gj_gtk_mpf_store_filename: default reached when checking browse id");
       break;
     }

   ctlMPF.file_selector = NULL;
}

static void gj_gtk_mpf_create_file_selection(guint sound) 
{
   /* Create the selector */
   ctlMPF.file_selector = gtk_file_selection_new(_("Select File for Sound"));
   
   g_signal_connect((gpointer) GTK_FILE_SELECTION(ctlMPF.file_selector)->ok_button,
		    "clicked",
		    G_CALLBACK(gj_gtk_mpf_store_filename),
		    GINT_TO_POINTER(sound));
   			   
   /* Ensure that the dialog box is destroyed when the user clicks a button. */
   g_signal_connect_swapped((gpointer) GTK_FILE_SELECTION(ctlMPF.file_selector)->ok_button,
			    "clicked",
			    G_CALLBACK(gtk_widget_destroy), 
			    (gpointer)ctlMPF.file_selector); 
   
   g_signal_connect_swapped((gpointer) GTK_FILE_SELECTION(ctlMPF.file_selector)->cancel_button,
			    "clicked",
			    G_CALLBACK(gtk_widget_destroy),
			    (gpointer)ctlMPF.file_selector); 
   
   /* Display that dialog */
   gtk_widget_show(ctlMPF.file_selector);
}

/*
 * GUI events
 */
static void on_button_mpf_browse_sound_on_user_online_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_SOUND_ON_USER_ONLINE);
}

static void on_button_mpf_browse_sound_on_user_offline_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_SOUND_ON_USER_OFFLINE);
}

static void on_button_mpf_browse_sound_on_receive_message_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_SOUND_ON_RECEIVE_MESSAGE);
}

static void on_button_mpf_browse_sound_on_receive_chat_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_SOUND_ON_RECEIVE_CHAT);
}

static void on_button_mpf_browse_sound_on_receive_headline_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_SOUND_ON_RECEIVE_HEADLINE);
}

static void on_button_mpf_browse_sound_on_receive_error_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_SOUND_ON_RECEIVE_ERROR);
}

static void on_button_mpf_browse_log_location_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_LOG_LOCATION);
}

static void on_button_mpf_browse_history_location_clicked(GtkButton *button, gpointer user_data)
{
  gj_gtk_mpf_create_file_selection(BROWSE_HISTORY_LOCATION);
}

static void on_checkbutton_mpf_use_history_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
  gboolean active = FALSE;
  active = gtk_toggle_button_get_active(togglebutton);
  gtk_widget_set_sensitive(GTK_WIDGET(ctlMPF.entry_browse_history_location), active);
  gtk_widget_set_sensitive(GTK_WIDGET(ctlMPF.button_browse_history_location), active);
}

/* static void on_checkbutton_mpf_sound_enabled_toggled(GtkToggleButton *togglebutton, gpointer user_data) */
/* { */
/*   gboolean active = FALSE; */
/*   active = gtk_toggle_button_get_active(togglebutton); */
/*   gtk_widget_set_sensitive(GTK_WIDGET(ctlMPF.table_sound), active); */
/* } */

static void on_checkbutton_mpf_hide_offline_contacts_toggled(GtkToggleButton *togglebutton, gpointer user_data)
{
  gtk_widget_set_sensitive(GTK_WIDGET(ctlMPF.checkbutton_hide_offline_group), 
			   !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(togglebutton)));
}

static void on_my_preferences_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_OK:
      gj_gtk_mpf_set();
      break;

    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
      break;
    }

  gj_gtk_mpf_unload();
}


#ifdef __cplusplus
}
#endif
