/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_my_presence.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/08 15:36:36 $
 * 
 * $Revision: 1.27 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_presence.h"
#include "gj_me.h"
#include "gj_support.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_my_presence.h"
#include "gj_gtk_support.h"

struct 
{
  GtkWidget *window;

  GtkWidget *spinbutton;
  GtkWidget *textview;
  
  GtkWidget *image;
  GtkWidget *label;
  GtkWidget *entry;

  GtkWidget *optionmenu_presence;
  GtkWidget *optionmenu_preset;

  GtkWidget *button_ok;

} ctlMP;


struct
{
  GjPresenceType type;
  GjPresenceShow show;

  gboolean is_custom_message;

} optMP;
  

static gboolean gj_gtk_mp_setup();
static gboolean gj_gtk_mp_set();

/* 
 * member functions 
 */

/* sets */
static gboolean gj_gtk_mp_set_type(GjPresenceType type);
static gboolean gj_gtk_mp_set_show(GjPresenceShow show);
static gboolean gj_gtk_mp_set_is_custom_message(gboolean custom);

/* gets */
static GjPresenceType gj_gtk_mp_get_type();
static GjPresenceShow gj_gtk_mp_get_show();
static gboolean gj_gtk_mp_get_is_custom_message();

static void gj_gtk_mp_presence_menu_cb(GtkMenuItem *menuitem, gpointer user_data);
static void gj_gtk_mp_presence_preset_menu_cb(GtkMenuItem *menuitem, gpointer user_data);


/* 
 * gui callbacks 
 */
static void on_textbuffer_mp_changed(GtkTextBuffer *textbuffer, gpointer user_data);
static void on_my_presence_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * code
 */
gboolean gj_gtk_mp_load(GjPresenceType type, GjPresenceShow show)
{
  GladeXML *xml = NULL;

  if(ctlMP.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlMP.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "my_presence",
			      NULL,
			      "my_presence", &ctlMP.window,
			      "spinbutton", &ctlMP.spinbutton,
/* 			      "image", &ctlMP.image, */
/* 			      "label_presence", &ctlMP.label, */
			      "textview", &ctlMP.textview,
			      "optionmenu_presence", &ctlMP.optionmenu_presence,
			      "optionmenu_preset", &ctlMP.optionmenu_preset,
			      "button_ok", &ctlMP.button_ok,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "my_presence", "response", on_my_presence_response,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(ctlMP.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | GDK_DECOR_MENU);

  /* set defaults */
  if(type <= 0)
    gj_gtk_mp_set_type(GjPresenceTypeAvailable);
  else
    gj_gtk_mp_set_type(type);

  if(show <= 0)
    gj_gtk_mp_set_show(GjPresenceShowNormal);
  else
    gj_gtk_mp_set_show(show);

  gj_gtk_mp_set_is_custom_message(TRUE);

  gj_gtk_mp_setup();

  return TRUE;
}

gboolean gj_gtk_mp_unload()
{
  if(ctlMP.window == NULL)
    return TRUE;

  gtk_widget_destroy(ctlMP.window);

  ctlMP.window = NULL;

  ctlMP.spinbutton = NULL;
  ctlMP.textview = NULL;

  ctlMP.image = NULL;
  ctlMP.label = NULL;

  ctlMP.button_ok = NULL;

  return TRUE;
}

static gboolean gj_gtk_mp_setup()
{
  gchar *text = NULL;

  GdkPixbuf *pb = NULL;
  GtkTextBuffer *buffer = NULL;
  GtkWidget *menu = NULL;

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMP.textview));
  
  pb = gj_gtk_presence_to_pixbuf_new(gj_gtk_mp_get_type(), gj_gtk_mp_get_show());
  text = g_strconcat("<b>",
		     gj_gtk_presence_to_show_text_new(gj_gtk_mp_get_type(), gj_gtk_mp_get_show()),
		     "</b>",
		     NULL);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(ctlMP.spinbutton), 
			    (gint8)gj_me_get_presence_priority());

/*   gtk_image_set_from_pixbuf(GTK_IMAGE(ctlMP.image), pb); */
/*   gtk_label_set_markup(GTK_LABEL(ctlMP.label),text); */


  g_signal_connect((gpointer) buffer, "changed", 
		   G_CALLBACK(on_textbuffer_mp_changed), 
		   NULL);

  if(gj_me_get_presence_status() != NULL)
    gtk_text_buffer_set_text(buffer, gj_me_get_presence_status(), -1);
 
  /* presence option menu */
  menu = gj_gtk_presence_menu_new(gj_gtk_mp_presence_menu_cb);
  gtk_option_menu_set_menu(GTK_OPTION_MENU(ctlMP.optionmenu_presence), menu);

  menu = gj_gtk_presence_preset_menu_new(gj_gtk_mp_presence_preset_menu_cb);
  gtk_option_menu_set_menu(GTK_OPTION_MENU(ctlMP.optionmenu_preset), menu);

  /* clean up */
  if(text != NULL) g_free(text);
  if(pb != NULL) g_object_unref(G_OBJECT(pb));

  return TRUE;
}

/* 
 * member functions 
 */

/* sets */
static gboolean gj_gtk_mp_set_type(GjPresenceType type)
{
  optMP.type = type;
  return TRUE;
}

static gboolean gj_gtk_mp_set_show(GjPresenceShow show)
{
  optMP.show = show;
  return TRUE;
}

static gboolean gj_gtk_mp_set_is_custom_message(gboolean custom)
{
  optMP.is_custom_message = custom;
  return TRUE;
}

/* gets */
static GjPresenceType gj_gtk_mp_get_type()
{
  return optMP.type;
}

static GjPresenceShow gj_gtk_mp_get_show()
{
  return optMP.show;
}

static gboolean gj_gtk_mp_get_is_custom_message()
{
  return optMP.is_custom_message;
}

static gboolean gj_gtk_mp_set()
{
  GtkTextBuffer *buffer = NULL;
  GtkTextIter bufStart;
  GtkTextIter bufStop;

  gchar *status = NULL;
  gint8 priority = 0;

  /* get priority */
  priority = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlMP.spinbutton));

  /* get status */
  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMP.textview));
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStart, 0);
  gtk_text_buffer_get_iter_at_offset(buffer, &bufStop, -1);
  status = gtk_text_buffer_get_text(buffer,&bufStart,&bufStop,TRUE);

  if(status != NULL && g_utf8_strlen(status, -1) < 1)
    status = NULL;

  /* update myself */
  gj_gtk_roster_set_my_presence_and_save(gj_gtk_mp_get_type(), 
					 gj_gtk_mp_get_show(), 
					 status, 
					 priority,
					 TRUE);

  return TRUE;
}

static void gj_gtk_mp_presence_menu_cb(GtkMenuItem *menuitem, gpointer user_data)
{
  gint callback_action = -1;

  if(user_data == NULL)
    {
      g_warning("%s: user data was NULL", __FUNCTION__);
      return;
    }

  /* get action */
  callback_action = GPOINTER_TO_INT(user_data);

  g_message("%s: agent menu activated menuitem:0x%.8x, callback action:%d", 
	    __FUNCTION__, (gint)menuitem, callback_action);

  switch(callback_action)
    {
    case gj_presence_menu_available:
      gj_gtk_mp_set_type(GjPresenceTypeAvailable);
      gj_gtk_mp_set_show(GjPresenceShowNormal);
      break;
    case gj_presence_menu_chat:
      gj_gtk_mp_set_type(GjPresenceTypeAvailable);
      gj_gtk_mp_set_show(GjPresenceShowChat);
      break;
    case gj_presence_menu_away:
      gj_gtk_mp_set_type(GjPresenceTypeAvailable);
      gj_gtk_mp_set_show(GjPresenceShowAway);
      break;
    case gj_presence_menu_xa:
      gj_gtk_mp_set_type(GjPresenceTypeAvailable);
      gj_gtk_mp_set_show(GjPresenceShowXA);
      break;
    case gj_presence_menu_dnd:
      gj_gtk_mp_set_type(GjPresenceTypeAvailable);
      gj_gtk_mp_set_show(GjPresenceShowDND);
      break;
    case gj_presence_menu_invisible:
      gj_gtk_mp_set_type(GjPresenceTypeAvailable);
      gj_gtk_mp_set_show(GjPresenceShowInvisible);
      break;

    default:
      break;
    }
}

static void gj_gtk_mp_presence_preset_menu_cb(GtkMenuItem *menuitem, gpointer user_data)
{
  gint callback_action = -1;
  GtkTextBuffer *buffer = NULL;

  if(user_data == NULL)
    {
      g_warning("%s: user data was NULL", __FUNCTION__);
      return;
    }

  buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(ctlMP.textview));

  /* get action */
  callback_action = GPOINTER_TO_INT(user_data);

  g_message("%s: agent menu activated menuitem:0x%.8x, callback action:%d", 
	    __FUNCTION__, (gint)menuitem, callback_action);

  switch(callback_action)
    {
    case gj_presence_preset_menu_custom:
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), "", -1);
      gj_gtk_mp_set_is_custom_message(TRUE);
      break;
    case gj_presence_preset_menu_around:
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), _("About the place..."), -1);
      gj_gtk_mp_set_is_custom_message(FALSE);
      break;
    case gj_presence_preset_menu_eating:
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), _("Eating"), -1);
      gj_gtk_mp_set_is_custom_message(FALSE);
      break;
    case gj_presence_preset_menu_out:
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), _("Out for a bit"), -1);
      gj_gtk_mp_set_is_custom_message(FALSE);
      break;
    case gj_presence_preset_menu_showering:
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), _("In the shower"), -1);
      gj_gtk_mp_set_is_custom_message(FALSE);
      break;
    case gj_presence_preset_menu_sleeping:
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), _("Sleeping... zZzZ"), -1);
      gj_gtk_mp_set_is_custom_message(FALSE);
      break;
    case gj_presence_preset_menu_working:
      gtk_text_buffer_set_text(GTK_TEXT_BUFFER(buffer), _("Working!") ,-1);
      gj_gtk_mp_set_is_custom_message(FALSE);
      break;

    default:
      break;
    }
}


/*
 * GUI events
 */
static void on_textbuffer_mp_changed(GtkTextBuffer *textbuffer, gpointer user_data)
{
  if(gj_gtk_mp_get_is_custom_message() == FALSE)
    {
      gtk_option_menu_set_history(GTK_OPTION_MENU(ctlMP.optionmenu_preset), 0);
      gj_gtk_mp_set_is_custom_message(TRUE);
    }


/*   gtk_entry_set_text(GTK_ENTRY(ctlMP.entry), _("Custom")); */
}

static void on_my_presence_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_OK:
      gj_gtk_mp_set();
      break;

    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
      break;
    }

  gj_gtk_mp_unload();
}


#ifdef __cplusplus
}
#endif
