/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_password_change.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/22 13:44:59 $
 * 
 * $Revision: 1.23 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include "gj_main.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"
#include "gj_config.h"

#include "gj_gtk_password_change.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_support.h"


struct 
{
  GtkWidget *window;

  GtkWidget *entry1;
  GtkWidget *entry2;

  GtkWidget *button_ok;

} ctlPC;


static gboolean gj_gtk_pc_changed();
static gboolean gj_gtk_pc_set();


/*
 * gui functions 
 */
static void on_entry_pc_password_1_changed(GtkEditable *editable, gpointer user_data);
static void on_entry_pc_password_2_changed(GtkEditable *editable, gpointer user_data);

static void on_password_change_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * code
 */
gboolean gj_gtk_pc_load()
{
  GladeXML *xml = NULL;

  if(ctlPC.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlPC.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "password_change",
			      NULL,
			      "password_change", &ctlPC.window,
			      "entry1", &ctlPC.entry1,
			      "entry2", &ctlPC.entry2,
			      "button_ok", &ctlPC.button_ok,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "password_change", "response", on_password_change_response,
		       "entry1", "changed", on_entry_pc_password_1_changed,
		       "entry2", "changed", on_entry_pc_password_2_changed,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);
  
  gdk_window_set_decorations (GDK_WINDOW(ctlPC.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  /* set up dialog */
  gtk_widget_set_sensitive(ctlPC.button_ok, FALSE);

  return TRUE;
}

gboolean gj_gtk_pc_unload()
{
  gtk_widget_destroy(ctlPC.window);

  return TRUE;
}

static gboolean gj_gtk_pc_set()
{
  G_CONST_RETURN gchar *password = NULL;

  password = gtk_entry_get_text(GTK_ENTRY(ctlPC.entry1));
  gj_iq_request_password_change(gj_config_get_cs_server(), 
				password, 
				gj_gtk_pc_set_response,
				NULL);
  
  return TRUE;
}

void gj_gtk_pc_set_response(guint event_id, guint related_event_id)
{
  GjInfoquery iq = NULL;
  GjInfoquery iq_related = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  g_message("%s: response with event_id:%d, related event id:%d", __FUNCTION__, event_id, related_event_id);

  iq = gj_iq_find(event_id);
  iq_related = gj_iq_find(related_event_id);

  if(iq == NULL || iq_related == NULL)
    {
      if(iq == NULL)
	g_warning("%s: failed to find IQ based on event id:%d", __FUNCTION__, event_id);
      else if(iq_related == NULL)
	g_warning("%s: failed to find IQ based on related event id:%d", __FUNCTION__, related_event_id);
      
      gj_gtk_show_error(_("Internal Error."),
			1,
			NULL);

      return;
    }

  node = gj_iq_get_xml(iq_related);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *error_code = NULL;
      xmlChar *error_message = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);
	  return;
	}

      error_code = gj_parser_find_by_attr_and_ret(node, (guchar*)"code");
      error_message = gj_parser_find_by_node_and_ret(node, (guchar*)"error");

      g_message("%s: IQ type is error, password change failed for event id:%d", __FUNCTION__, event_id);
      g_message("%s: error code:%d, reason:'%s'", __FUNCTION__, atoi((gchar*)error_code), error_message);

      gj_gtk_show_error(_("Error Changing Password."),
			atoi((gchar*)error_code),
			(gchar*)error_message);

      g_free(detail);

      return;
    }
  
  g_message("%s: received response, event id:%d", __FUNCTION__, event_id);

  gj_gtk_show_success(_("Password Change."));
}

static gboolean gj_gtk_pc_changed()
{
  gboolean match = FALSE;

  G_CONST_RETURN gchar *password1 = NULL;
  G_CONST_RETURN gchar *password2 = NULL;

  password1 = gtk_entry_get_text(GTK_ENTRY(ctlPC.entry1));
  password2 = gtk_entry_get_text(GTK_ENTRY(ctlPC.entry2));

  if(password1 != NULL && g_utf8_strlen(password1, -1) > 0 && 
     password2 != NULL && g_utf8_strlen(password2, -1) > 0)
    {
      if(strcmp(password1,password2) == 0)
	match = TRUE;
    }
  else
    {
      gtk_widget_set_sensitive(GTK_WIDGET(ctlPC.button_ok), FALSE);
      return TRUE;
    }
  
  /* change gui based on match */
  if(match == TRUE)
      gtk_widget_set_sensitive(GTK_WIDGET(ctlPC.button_ok), TRUE);
  else
      gtk_widget_set_sensitive(GTK_WIDGET(ctlPC.button_ok), FALSE);

  return TRUE;
}


/*
 * GUI events
 */

static void on_entry_pc_password_1_changed(GtkEditable *editable, gpointer user_data)
{
  gj_gtk_pc_changed();
}

static void on_entry_pc_password_2_changed(GtkEditable *editable, gpointer user_data)
{
  gj_gtk_pc_changed();
}

/*
 * GUI events
 */
static void on_password_change_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_OK:
      gj_gtk_pc_set();
      break;

    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
      break;
    }

  gj_gtk_pc_unload();
}

#ifdef __cplusplus
}
#endif
