/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_password_required.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/05 16:11:13 $
 * 
 * $Revision: 1.7 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <glade/glade.h>

#include "common.h"

#include "gj_main.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"

#include "gj_gtk_password_required.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_support.h"


struct 
{
  GtkWidget *window;

  GtkWidget *entry;

  GtkWidget *button_ok;

} ctlPR;

struct 
{
  gboolean check_password;
  gchar *password;
  gj_gtk_pr_callback cb;

} optPR;


/*
 * member functions
 */

/* gets */
static gboolean gj_gtk_pr_get_check_password();
static const gchar *gj_gtk_pr_get_password();
static gj_gtk_pr_callback gj_gtk_pr_get_callback();

/* sets */
static gboolean gj_gtk_pr_set_check_password(gboolean check_password);
static gboolean gj_gtk_pr_set_password(const gchar *password);
static gboolean gj_gtk_pr_set_callback(gj_gtk_pr_callback cb);


/*
 * utils 
 */
static gboolean gj_gtk_pr_changed();
static gboolean gj_gtk_pr_call_callback(gboolean password_correct, const gchar *password);
static gboolean gj_gtk_pr_check();

/*
 * gui functions 
 */
static void on_entry_pr_password_changed(GtkEditable *editable, gpointer user_data);
static void on_entry_pr_password_activate(GtkEntry *entry, gpointer user_data);

static void on_password_required_response(GtkDialog *dialog, gint response, gpointer user_data);


/*
 * member functions
 */

/* gets */
static gboolean gj_gtk_pr_get_check_password()
{
  return optPR.check_password;
}

static const gchar *gj_gtk_pr_get_password()
{
  return optPR.password;
}

static gj_gtk_pr_callback gj_gtk_pr_get_callback()
{
  return optPR.cb;
}

/* sets */
static gboolean gj_gtk_pr_set_check_password(gboolean check_password)
{
  optPR.check_password = check_password;
  return TRUE;
}

static gboolean gj_gtk_pr_set_password(const gchar *password)
{
  if(password == NULL)
    {
      g_warning("%s: password was NULL", __FUNCTION__);
      return FALSE;
    }

  if(optPR.password != NULL)
    {
      g_free(optPR.password);
      optPR.password = NULL;
    }

  optPR.password = g_strdup(password);
  return TRUE;
}

static gboolean gj_gtk_pr_set_callback(gj_gtk_pr_callback cb)
{
  optPR.cb = cb;
  return TRUE;
}


/*
 * window functions
 */
gboolean gj_gtk_pr_load(gboolean check_password, const gchar *password, gj_gtk_pr_callback cb)
{
  GladeXML *xml = NULL;

  if(check_password == TRUE && password == NULL)
    {
      g_warning("%s: password was NULL", __FUNCTION__);
      return FALSE;
    }

  if(cb == NULL) 
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  if(ctlPR.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlPR.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "password_required",
			      NULL,
			      "password_required", &ctlPR.window,
			      "entry", &ctlPR.entry,
			      "button_ok", &ctlPR.button_ok,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "password_required", "response", on_password_required_response,
		       "entry", "changed", on_entry_pr_password_changed,
		       "entry", "activate", on_entry_pr_password_activate,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);
  
  gdk_window_set_decorations (GDK_WINDOW(ctlPR.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  /* save password and callback */
  gj_gtk_pr_set_check_password(check_password);
  if(check_password == TRUE) gj_gtk_pr_set_password((gchar*)password);
  gj_gtk_pr_set_callback(cb);

  /* set up dialog */
  gtk_widget_set_sensitive(ctlPR.button_ok, FALSE);

  return TRUE;
}

gboolean gj_gtk_pr_unload()
{
  if(ctlPR.window == NULL)
    return TRUE;

  gtk_widget_destroy(GTK_WIDGET(ctlPR.window));

  ctlPR.window = NULL;
  
  return TRUE;
}

static gboolean gj_gtk_pr_changed()
{
  G_CONST_RETURN gchar *password = NULL;
  password = gtk_entry_get_text(GTK_ENTRY(ctlPR.entry));

  if(password != NULL && g_utf8_strlen(password, -1) > 0) 
    gtk_widget_set_sensitive(GTK_WIDGET(ctlPR.button_ok), TRUE);
  else
    gtk_widget_set_sensitive(GTK_WIDGET(ctlPR.button_ok), FALSE);

  return TRUE;
}

static gboolean gj_gtk_pr_call_callback(gboolean success, const gchar *password)
{
  gj_gtk_pr_callback cb = NULL;

  if((cb = gj_gtk_pr_get_callback()) == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return FALSE;
    }

  (cb)(success, password);

  return TRUE;
}

static gboolean gj_gtk_pr_check()
{
  G_CONST_RETURN gchar *password = NULL;
  password = gtk_entry_get_text(GTK_ENTRY(ctlPR.entry));

  if(gj_gtk_pr_get_check_password() == FALSE)
    return TRUE;

  if(password == NULL || g_utf8_strlen(password, -1) < 1)
    return FALSE;

  if(gj_gtk_pr_get_password() == NULL)
    {
      g_warning("%s: password was NULL", __FUNCTION__);
      return FALSE;
    }
    
  if(strcmp(gj_gtk_pr_get_password(), password) == 0)
    return TRUE;

  return FALSE;
}

/*
 * GUI events
 */
static void on_entry_pr_password_changed(GtkEditable *editable, gpointer user_data)
{
  gj_gtk_pr_changed();
}

static void on_entry_pr_password_activate(GtkEntry *entry, gpointer user_data)
{
  G_CONST_RETURN gchar *password = NULL; 
  password = gtk_entry_get_text(GTK_ENTRY(entry));

  if(password == NULL || g_utf8_strlen(password, -1) < 1)
    return;

  on_password_required_response(GTK_DIALOG(ctlPR.window), GTK_RESPONSE_OK, NULL); 
}

static void on_password_required_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  G_CONST_RETURN gchar *password = NULL; 

  switch(response) 
    {
    case GTK_RESPONSE_OK:
      password = gtk_entry_get_text(GTK_ENTRY(ctlPR.entry));
      gj_gtk_pr_call_callback(gj_gtk_pr_check(), password);
      break;

    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
      gj_gtk_pr_call_callback(FALSE, NULL);
      break;

    default:
      g_message("%s: response:%d not handled", __FUNCTION__, response);
      break;
    }

  /* we close the window on ALL responses */
  gj_gtk_pr_unload();
}

#ifdef __cplusplus
}
#endif
