/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $RCSfile: gj_gtk_progress.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/08 11:22:07 $
 *
 * $Revision: 1.4 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_main.h"
#include "gj_support.h"

#include "gj_gtk_progress.h"
#include "gj_gtk_support.h"

/* list functions */
static gboolean gj_gtk_progress_list_add(Prg p);
static gboolean gj_gtk_progress_list_del(Prg p);

/* item functions */
static gboolean gj_gtk_progress_free(Prg p);
static void gj_gtk_progress_destroy(Prg p);

/* find */
static Prg gj_gtk_progress_find_by_id(guint id);
static Prg gj_gtk_progress_find_by_widget(GtkWidget *widget);

/* member functions - gets */
static const gchar *gj_gtk_progress_get_primary(Prg p);
static const gchar *gj_gtk_progress_get_secondary(Prg p);

static PrgCb gj_gtk_progress_get_cb(Prg p);

static guint gj_gtk_progress_get_id(Prg p);
static void *gj_gtk_progress_get_userdata(Prg p);

static gboolean gj_gtk_progress_get_modal(Prg p);

/* member functions - sets */

/* local functions */
static gboolean gj_gtk_progress_load(Prg p);
static gboolean gj_gtk_progress_unload(Prg p);

static gboolean gj_gtk_progress_cancel(Prg p);
static gboolean gj_gtk_progress_setup(Prg p);

/* internal callbacks */
static gboolean gj_gtk_progress_pulse_cb(gpointer userdata);

/*
 * actual GUI functions
 */
static void on_progress_response(GtkDialog *dialog, gint response, gpointer user_data);


struct t_ctlPrg 
{
  /* search */
  GtkWidget *window;

  GtkWidget *button_cancel;

  GtkWidget *label_primary;
  GtkWidget *label_secondary;

  GtkWidget *progressbar;
};

struct t_prg
{
  guint id;
  void *userdata;

  /* properties */
  gchar *primary;
  gchar *title;
  gchar *secondary; 

  gboolean modal;
  GtkWindow *transient_window;

  PrgCb cb;

  gint response;

  /* controls */
  struct t_ctlPrg controls;
};


guint prg_id = 0;
GList *prg_list = NULL;


gboolean gj_gtk_progress_init()
{
  /* nothing to do here */
  return TRUE;
}

gboolean gj_gtk_progress_term()
{
  gint index = 0;
    
  /* list */
  if(prg_list == NULL)
    return TRUE;
    
  /* free all items on the list */
  for(index=0;index<g_list_length(prg_list);index++)
    {
      Prg p = NULL;
	
      if((p = (Prg) g_list_nth_data(prg_list,index)) != NULL)
	gj_gtk_progress_free(p);
    }
    
  g_list_free(prg_list);
  prg_list = NULL;
    
  return TRUE;
}

/*
 * list functions
 */
static gboolean gj_gtk_progress_list_add(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: failed to add item to list, it was NULL", __FUNCTION__);
      return FALSE;
    }

  prg_list = g_list_append(prg_list, p);

  return TRUE;
}

static gboolean gj_gtk_progress_list_del(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: failed to delete item from list, it was NULL", __FUNCTION__);
      return FALSE;
    }

  prg_list = g_list_remove(prg_list, p);
  return TRUE;
}

/*
 * item functions
 */
Prg gj_gtk_progress_start(gboolean modal, 
			  GtkWindow *transient_window,
			  PrgCb cb, 
			  void *userdata, 
			  const gchar *primary, 
			  const gchar *secondary, ...)
{
  Prg p = NULL;
  
  va_list args;
  gchar *buffer = NULL;

  va_start(args,secondary);
  buffer = g_strdup_vprintf(secondary,args);
  va_end(args);

  /* check vars */
  if(primary == NULL)
    {
      g_warning("%s: primary was NULL", __FUNCTION__);
      return NULL;
    }
  
  if(cb == NULL)
    {
      g_warning("%s: callback was NULL", __FUNCTION__);
      return NULL;
    }
  
  /* create new struct */
  if((p = g_new(struct t_prg,1)))
    {
      p->id = gj_get_unique_id();
      p->userdata = userdata;

      p->primary = g_strdup(primary);
      p->secondary = buffer;

      p->transient_window = transient_window;
      
      p->modal = modal;
      p->cb = cb;

      p->response = 0;
    }

  /* add to list */
  gj_gtk_progress_list_add(p);

  /* show dialog */
  gj_gtk_progress_load(p);

  return p;
}

static gboolean gj_gtk_progress_free(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return FALSE;
    }

  g_message("%s: Prg id:%d", __FUNCTION__, gj_gtk_progress_get_id(p));

  /* free mem */
  gj_gtk_progress_list_del(p);  

  /* destroy members */
  gj_gtk_progress_destroy(p);

  /* free structure */
  g_free(p);

  return TRUE;
}

static void gj_gtk_progress_destroy(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return;
    }

  p->id = 0;

  p->userdata = NULL;

  if(p->primary != NULL)
    {
      g_free(p->primary);
      p->primary = NULL;
    }

  if(p->secondary != NULL)
    {
      g_free(p->secondary);
      p->secondary = NULL;
    }

  p->modal = FALSE;
  p->transient_window = NULL;

  p->cb = NULL;

  p->response = 0;

  /* controls */
  p->controls.window = NULL;
  
  p->controls.button_cancel = NULL;
  
  p->controls.label_primary = NULL;
  p->controls.label_secondary = NULL;
  
  p->controls.progressbar = NULL;
}

static Prg gj_gtk_progress_find_by_id(guint id)
{
  gint index = 0;

  if(id < 1)
    {
      g_warning("%s: id:%d < 1", __FUNCTION__, id);
      return NULL;
    }

  for(index=0;index<g_list_length(prg_list);index++)
    {
      Prg p = g_list_nth_data(prg_list, index);
      
      if(p == NULL) continue;
      if(p->id == id) return p;
    }
 
  return NULL;
}

static Prg gj_gtk_progress_find_by_widget(GtkWidget *widget)
{
  gint index = 0;
  GtkWidget *toplevel = NULL;

  if(widget == NULL)
    {
      g_warning("%s: widget was NULL", __FUNCTION__);
      return NULL;
    }

  toplevel = gtk_widget_get_toplevel(widget);
  if(!GTK_WIDGET_TOPLEVEL(toplevel))
    {
      g_warning("%s: could not find toplevel widget for widget:0x%.8x", __FUNCTION__, (gint)widget);
      return NULL;
    }

  for(index=0;index<g_list_length(prg_list);index++)
    {
      Prg p = g_list_nth_data(prg_list, index);
      
      if(p == NULL) continue;
      if(p->controls.window == toplevel) return p;
    }
 
  return NULL;
}

/*
 * member functions 
 */

/* gets */
static guint gj_gtk_progress_get_id(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return -1;
    }
  
  return p->id;
}

static void *gj_gtk_progress_get_userdata(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return NULL;
    }
  
  return p->userdata;
}

static const gchar *gj_gtk_progress_get_primary(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return NULL;
    }  

  return p->primary;
}

static const gchar *gj_gtk_progress_get_secondary(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return NULL;
    }

  return p->secondary;
}

static gboolean gj_gtk_progress_get_modal(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return FALSE;
    }

  return p->modal;
}

static GtkWindow *gj_gtk_progress_get_transient_window(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return FALSE;
    }

  return p->transient_window;
}

static PrgCb gj_gtk_progress_get_cb(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return NULL;
    } 
    
  return p->cb;
}

/* sets */


/*
 * actual function to modify the prg gui  
 */
static gboolean gj_gtk_progress_load(Prg p)
{
  GladeXML *xml = NULL;

  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return FALSE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "progress",
			      NULL,
			      "progress", &p->controls.window,
			      "button_cancel", &p->controls.button_cancel,
			      "label_primary", &p->controls.label_primary,
			      "label_secondary", &p->controls.label_secondary,
			      "progressbar", &p->controls.progressbar,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "progress", "response", on_progress_response,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gj_gtk_progress_setup(p);

  return TRUE;
}

static gboolean gj_gtk_progress_unload(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return FALSE;
    }

  gtk_widget_destroy(p->controls.window);
  p->controls.window = NULL;

  gj_gtk_progress_free(p);

  return TRUE;
}

static gboolean gj_gtk_progress_setup(Prg p)
{
  gchar *text = NULL;
  static guint id = -1;

  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return FALSE;
    }

  /* details */
  id = gj_gtk_progress_get_id(p);

  g_message("%s: using id:%d", __FUNCTION__, id);
  g_message("%s: using userdata: 0x%.8x", __FUNCTION__, (gint)gj_gtk_progress_get_userdata(p));
  g_message("%s: using callback: 0x%.8x", __FUNCTION__, (gint)gj_gtk_progress_get_cb(p));

  /* set text */
  g_message("%s: setting primary...", __FUNCTION__);
  text = g_strdup_printf("<span size=\"11000\" weight=\"heavy\">%s</span>", gj_gtk_progress_get_primary(p));
  gtk_label_set_markup(GTK_LABEL(p->controls.label_primary), text);
  g_free(text);

  if(gj_gtk_progress_get_secondary(p) != NULL)
    {
      g_message("%s: setting secondary...", __FUNCTION__);
      text = g_strdup_printf("%s", gj_gtk_progress_get_secondary(p));
      gtk_label_set_markup(GTK_LABEL(p->controls.label_secondary), text);
      g_free(text);
    }

  /* set modal prop */
  g_message("%s: setting window to %s", __FUNCTION__, gj_gtk_progress_get_modal(p)?"be modal":"NOT be modal");
  gtk_window_set_modal(GTK_WINDOW(p->controls.window), gj_gtk_progress_get_modal(p));


  /* set transient window */
  if(p->transient_window != NULL)
    {
      g_message("%s: setting transient window: 0x%.8x", __FUNCTION__, (gint)gj_gtk_progress_get_transient_window(p));
      gtk_window_set_transient_for(GTK_WINDOW(p->controls.window), 
				   GTK_WINDOW(gj_gtk_progress_get_transient_window(p)));
    }

  /* set timeout */
  g_timeout_add(50, gj_gtk_progress_pulse_cb, GINT_TO_POINTER(id));

  gdk_window_set_decorations (GDK_WINDOW(p->controls.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  return TRUE;
}

static gboolean gj_gtk_progress_cancel(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_gtk_progress_get_cb(p) != NULL)
    {
      g_message("%s: using callback:0x%.8x, with userdata:0x%.8x", 
		__FUNCTION__, (gint)gj_gtk_progress_get_cb(p), (gint)gj_gtk_progress_get_userdata);

      /* call callback function */
      (gj_gtk_progress_get_cb(p))(TRUE, gj_gtk_progress_get_userdata(p));
    }
     
  return TRUE;
}

void gj_gtk_progress_stop(Prg p)
{
  if(p == NULL)
    {
      g_warning("%s: Prg was NULL", __FUNCTION__);
      return;
    }

  if(gj_gtk_progress_get_cb(p) != NULL)
    {
      g_message("%s: using callback:0x%.8x, with userdata:0x%.8x", 
		__FUNCTION__, (gint)gj_gtk_progress_get_cb(p), (gint)gj_gtk_progress_get_userdata);

      /* call callback function */
      (gj_gtk_progress_get_cb(p))(FALSE, gj_gtk_progress_get_userdata(p));
    }
  
  gj_gtk_progress_unload(p);
}

static gboolean gj_gtk_progress_pulse_cb(gpointer userdata)
{
  gint id = 0;
  Prg p = NULL;

  if(userdata == NULL)
    return FALSE;

  id = GPOINTER_TO_INT(userdata);
  
  if((p = gj_gtk_progress_find_by_id(id)) == NULL)
    return FALSE;

  gtk_progress_bar_pulse(GTK_PROGRESS_BAR(p->controls.progressbar));
  return TRUE;
}

/*
 * actual gui functions
 */
static void on_progress_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  Prg p = NULL;

  if((p = gj_gtk_progress_find_by_widget(GTK_WIDGET(dialog))) == NULL)
    {
      g_warning("%s: could not find window from widget:0x%.8x", __FUNCTION__, (gint)dialog);
      return;
    }

  switch(response) 
    {
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
      gj_gtk_progress_cancel(p);
      break;

    case GTK_RESPONSE_OK:
    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
    case GTK_RESPONSE_APPLY:
      break;
    }

  gj_gtk_progress_unload(p);
}


#ifdef __cplusplus
}
#endif
