/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $RCSfile: gj_gtk_register_new.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/08 00:05:46 $
 *
 * $Revision: 1.28 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "gj_main.h"
#include "gj_stream.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_parser.h"
#include "gj_me.h"
#include "gj_config.h"

#include "gj_gtk_register_new.h"
#include "gj_gtk_support.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_connection_settings.h"


/* local functions */
static gboolean gj_gtk_rn_setup();

static gboolean gj_gtk_rn_passwords_changed();
static gboolean gj_gtk_rn_check();

static gboolean gj_gtk_rn_save_settings();

static gboolean gj_gtk_rn_connection();
static void gj_gtk_rn_connected_response(GjStream stm, gint requestID);
static void gj_gtk_rn_registered_response(guint event_id, guint related_event_id);

/*
 * actual GUI functions
 */
static void on_entry_rn_password_1_changed(GtkEditable *editable, gpointer user_data);
static void on_entry_rn_password_2_changed(GtkEditable *editable, gpointer user_data);

static void on_register_new_response(GtkDialog *dialog, gint response, gpointer user_data);


struct 
{
  /* search */
  GtkWidget *window;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;

  GtkWidget *combo_server;
  GtkWidget *combo_entry_server;
  GtkWidget *spinbutton_port;
  GtkWidget *entry_username;  
  GtkWidget *entry_password1;
  GtkWidget *entry_password2;

  GtkWidget *checkbutton_use_configuration;

  GtkWidget *vbox;

} ctlRN;


struct
{
  GjStream stm;

} optRN;


/*
 * external functions...
 */
gboolean gj_gtk_rn_load()
{
  GladeXML *xml = NULL;

  if(ctlRN.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlRN.window));
      return TRUE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "register_new",
			      NULL,
			      "register_new", &ctlRN.window,
			      "combo_server", &ctlRN.combo_server,
			      "combo-entry_server", &ctlRN.combo_entry_server,
			      "spinbutton_port", &ctlRN.spinbutton_port,
			      "entry_username", &ctlRN.entry_username,
			      "entry_password1", &ctlRN.entry_password1,
			      "entry_password2", &ctlRN.entry_password2,
			      "checkbutton_use_configuration", &ctlRN.checkbutton_use_configuration,
			      "dialog-vbox", &ctlRN.vbox,
			      "button_ok", &ctlRN.button_ok,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "register_new", "response", on_register_new_response,
		       "entry_password1", "changed", on_entry_rn_password_1_changed,
		       "entry_password2", "changed", on_entry_rn_password_2_changed,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(ctlRN.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  gj_gtk_rn_setup();

  return TRUE;
}

gboolean gj_gtk_rn_unload()
{
  if(ctlRN.window == NULL)
    return TRUE;

  gtk_widget_destroy(ctlRN.window);

  ctlRN.window = NULL;

  return TRUE;
}

static gboolean gj_gtk_rn_setup()
{
  GList *servers = NULL;
  const gchar **server_listing = NULL;
  const gchar *str = NULL;
  gint i = 0;

  server_listing = gnome_jabber_get_server_list();
  
  /* set a limit of a 1000 */
  while((str = server_listing[i++]) != NULL && i < 1000)
      servers = g_list_append(servers, (gchar*)str);

  gtk_combo_set_popdown_strings(GTK_COMBO(ctlRN.combo_server), servers);

  gtk_entry_set_text(GTK_ENTRY(ctlRN.combo_entry_server), "jabber.org");

  /* clean up */
  if(servers != NULL) g_list_free(servers);

  return TRUE;
}

static gboolean gj_gtk_rn_passwords_changed()
{
  gboolean match = FALSE;

  G_CONST_RETURN gchar *password1 = NULL;
  G_CONST_RETURN gchar *password2 = NULL;

  password1 = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_password1));
  password2 = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_password2));

  if(password1 != NULL && g_utf8_strlen(password1, -1) > 0 && 
     password2 != NULL && g_utf8_strlen(password2, -1) > 0)
    {
      if(strcmp(password1,password2) == 0)
	match = TRUE;
    }
  else
    {
      gtk_widget_set_sensitive(GTK_WIDGET(ctlRN.button_ok), FALSE);
      return TRUE;
    }
  
  /* change gui based on match */
  if(match == TRUE)
    gtk_widget_set_sensitive(GTK_WIDGET(ctlRN.button_ok), TRUE);
  else
    gtk_widget_set_sensitive(GTK_WIDGET(ctlRN.button_ok), FALSE);

  return TRUE;
}

static gboolean gj_gtk_rn_check()
{
  gboolean passed = FALSE;

  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *server = NULL;

  G_CONST_RETURN gchar *password1 = NULL;
  G_CONST_RETURN gchar *password2 = NULL;

  server = gtk_entry_get_text(GTK_ENTRY(ctlRN.combo_entry_server));
  username = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_username));

  password1 = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_password1));
  password2 = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_password2));

  /* do checks */
  if(server == NULL || g_utf8_strlen(server, -1) < 1)
    {
      /* server not entered */
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Server was not specified."));

      passed = FALSE;
    }
  else if(username == NULL || g_utf8_strlen(username, -1) < 1)
    {
      /* username not entered */
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Username was not specified."));

      passed = FALSE;
    }
  else if(password1 == NULL || password2 == NULL || g_utf8_strlen(password1, -1) < 1 || g_utf8_strlen(password2, -1) < 1)
    {
      /* passwords not entered */
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Password was not specified."));

      passed = FALSE;
    }
  else if(g_strcasecmp(password1,password2) != 0)
    {
      /* passwords do not match */
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Passwords do NOT match."));

      passed = FALSE;
    }
  else
    {
      /* ok */
      passed = TRUE;
    }

  return passed;
}

static gboolean gj_gtk_rn_save_settings()
{
  G_CONST_RETURN gchar *server = NULL;
  gint16 port = 0;
  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *password = NULL;

  /* if checkbutton is selected, change the connection settings */
  server = gtk_entry_get_text(GTK_ENTRY(ctlRN.combo_entry_server));
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlRN.spinbutton_port));
  username = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_username));
  password = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_password1));

  /* set */
  gj_config_set_cs_server(server);
  gj_config_set_cs_port(port);
  gj_config_set_cs_username(username);
  gj_config_set_cs_password(password);

  /* save */
  gj_config_write();

  return TRUE;
}

static gboolean gj_gtk_rn_connection()
{
  GjStream stm = NULL;
  G_CONST_RETURN gchar *server = NULL;
  gint16 port = 0;

  server = gtk_entry_get_text(GTK_ENTRY(ctlRN.combo_entry_server));
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlRN.spinbutton_port));

  if((stm = gj_stream_new(server)) == NULL)
    {
      g_warning("gj_gtk_rn_connection: failed to send request, information query was NULL");
      return FALSE;
    }


  /* set struct info */
  gj_stream_set_to(stm, server);
  gj_stream_set_port(stm, port);
  gj_stream_set_connected_cb(stm,gj_gtk_rn_connected_response);

  if(gj_stream_build(stm) == FALSE)
    {
      g_warning("gj_gtk_rn_connection: failed to send request for authorisation, could not build stream.");
      return FALSE;
    }

  /* set controls state */
  gtk_widget_set_sensitive(ctlRN.vbox, FALSE);
  gtk_widget_set_sensitive(ctlRN.button_ok, FALSE);

  gj_stream_open(stm);
  
  /* make available to others */
  optRN.stm = stm;

  return TRUE;
}


static void gj_gtk_rn_connected_response(GjStream stm, gint requestID)
{
  G_CONST_RETURN gchar *server = NULL;
  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *password = NULL;
  guint port = 5222;

  /* get information */
  server = gtk_entry_get_text(GTK_ENTRY(ctlRN.combo_entry_server));
  port = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ctlRN.spinbutton_port));
  username = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_username));
  password = gtk_entry_get_text(GTK_ENTRY(ctlRN.entry_password1));

  /* set information */
  gj_me_set_connected_host(server);
  gj_me_set_connected_port(port);
  gj_me_set_connected_jid(username);

  /* do request */
  gj_iq_request_register_user(server,
			      username,
			      password,
			      gj_gtk_rn_registered_response,
			      NULL);
}

static void gj_gtk_rn_registered_response(guint event_id, guint related_event_id)
{
  GjInfoquery iq = NULL;
  GjInfoquery iq_related = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  g_message("%s: response with event_id:%d, related event id:%d", __FUNCTION__, event_id, related_event_id);

  gtk_widget_set_sensitive(ctlRN.vbox, TRUE);
  gtk_widget_set_sensitive(ctlRN.button_ok, TRUE);

  g_message("%s: response with event_id:%d", __FUNCTION__, event_id);
  
  if((iq = gj_iq_find(event_id)) == NULL)
    {
      gj_gtk_show_error(_("Failed to Register New Account."),
			1,
			_("Invalid Reponse."));

      g_warning("%s: failed to find IQ based on event_id:%d", __FUNCTION__, event_id);
      return;
    }

  if((iq_related = gj_iq_find(related_event_id)) == NULL)
    {
      gj_gtk_show_error(_("Failed to Register New Account."),
			2,
			_("Invalid Reponse."));

      g_warning("%s: failed to find IQ based on related event id:%d", __FUNCTION__, related_event_id);

      return;
    }

  /* check for errors? */
  node = gj_iq_get_xml(iq_related);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *error_code = NULL;
      xmlChar *error_message = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);
	  gj_iq_free_by_event_id(event_id);
	  return;
	}

      error_code = gj_parser_find_by_attr_and_ret(node, (guchar*)"code");
      error_message = gj_parser_find_by_node_and_ret(node, (guchar*)"error");
      
      gj_gtk_show_error(_("Failed to Register New Account."),
			atoi((gchar*)error_code),
			(gchar*)error_message);
      
      g_warning("%s: IQ type is error, registration failed for event_id:%d", __FUNCTION__, event_id);
      
      /* clean up */
      g_free(detail);
      gj_iq_free_all_related_by_event_id(event_id);
      
      /* find stream and close */
      if(optRN.stm != NULL)
	{
	  gj_stream_free(optRN.stm);
	  optRN.stm = NULL;
	}
      
      return;
    }

  /* ok */
  gj_gtk_msg_new(TRUE,
		 GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_INFORMATION,
		 (MsgCb)NULL,
		 NULL,
		 _("Registration Success."),
		 _("No Errors"),
		 NULL);

  /* clean up */
  gj_iq_free_all_related_by_event_id(event_id);

  if(optRN.stm != NULL)
    {
      /* find stream and close */
      gj_stream_free(optRN.stm);
      optRN.stm = NULL;
    }

  /* should we save the settings to the connection settings ?? */
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ctlRN.checkbutton_use_configuration)) == TRUE)
    gj_gtk_rn_save_settings();
}

/*
 * actual gui functions
 */
static void on_entry_rn_password_1_changed(GtkEditable *editable, gpointer user_data)
{
    gj_gtk_rn_passwords_changed();
}

static void on_entry_rn_password_2_changed(GtkEditable *editable, gpointer user_data)
{
    gj_gtk_rn_passwords_changed();
}

static void on_register_new_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  switch(response) 
    {
    case GTK_RESPONSE_OK:
      if(gj_gtk_rn_check() == FALSE)
	return;
      
      gj_gtk_rn_connection();
      return;

    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
      break;
    }

  gj_gtk_rn_unload();
}

#ifdef __cplusplus
}
#endif
