/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * $RCSfile: gj_gtk_register_service.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/03 21:02:46 $
 *
 * $Revision: 1.23 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* gui */
#include <glade/glade.h>
#include <gtk/gtk.h>

/* local */
#include "common.h"

#include "gj_main.h"
#include "gj_registration.h"
#include "gj_parser.h"

#include "gj_gtk_register_service.h"
#include "gj_gtk_support.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_progress.h"


/*
 * member functions
 */

/* gets */
static const gchar *gj_gtk_rs_get_key();

static gboolean gj_gtk_rs_get_require_username();
static gboolean gj_gtk_rs_get_require_password();
static gboolean gj_gtk_rs_get_require_email();
static gboolean gj_gtk_rs_get_require_nickname();

/* sets */
static gboolean gj_gtk_rs_set_key(const gchar *key);
static gboolean gj_gtk_rs_set_service(const gchar *service);

static gboolean gj_gtk_rs_set_require_username(gboolean require);
static gboolean gj_gtk_rs_set_require_password(gboolean require);
static gboolean gj_gtk_rs_set_require_email(gboolean require);
static gboolean gj_gtk_rs_set_require_nickname(gboolean require);


/*
 * window functions 
 */
static gboolean gj_gtk_rs_setup();
static gboolean gj_gtk_rs_check();

static void gj_gtk_rs_registration_requirements_response(GjRegistration rgr, gint error_code, const gchar *error_reason);
static void gj_gtk_rs_registration_response(GjRegistration rgr, gint error_code, const gchar *error_reason);


/* 
 * internal callbacks
 */
static void gj_gtk_rs_requirements_progress_cb(gboolean cancelled, gpointer userdata);
static void gj_gtk_rs_register_progress_cb(gboolean cancelled, gpointer userdata);

/*
 * actual GUI functions
 */
static void on_register_service_response(GtkDialog *dialog, gint response, gpointer user_data);


struct 
{
  /* search */
  GtkWidget *window;

  GtkWidget *label_instructions;
  GtkWidget *label_service;

  GtkWidget *label_username;
  GtkWidget *label_password;
  GtkWidget *label_email;
  GtkWidget *label_nickname;
  GtkWidget *entry_username;  
  GtkWidget *entry_password;
  GtkWidget *entry_email;
  GtkWidget *entry_nickname;

  GtkWidget *button_ok;
  GtkWidget *button_cancel;

} ctlRS;


struct
{
  gchar *key; /* key used as part of registration */

  gchar *service;

  gint requirements; /* bit set int */

  gboolean require_username;
  gboolean require_password;
  gboolean require_email;
  gboolean require_nickname;

} optRS;


static Prg p = NULL;

/*
 * member functions
 */

/* gets */
static const gchar *gj_gtk_rs_get_key()
{
  return optRS.key;
}

static gboolean gj_gtk_rs_get_require_username()
{
  return optRS.require_username;
}

static gboolean gj_gtk_rs_get_require_password()
{
  return optRS.require_password;
}

static gboolean gj_gtk_rs_get_require_email()
{
  return optRS.require_email;
}

static gboolean gj_gtk_rs_get_require_nickname()
{
  return optRS.require_nickname;
}

/* sets */
static gboolean gj_gtk_rs_set_key(const gchar *key)
{
  if(key == NULL)
    {
      g_warning("gj_gtk_rs_set_key: key was NULL");
      return FALSE;
    }
  
  if(optRS.key != NULL)
    {
      g_free(optRS.key);
      optRS.key = NULL;
    }

  optRS.key = g_strdup(key);
  return TRUE;
}

static gboolean gj_gtk_rs_set_service(const gchar *service)
{
  if(service == NULL)
    {
      g_warning("gj_gtk_rs_set_service: service was NULL");
      return FALSE;
    }
  
  if(optRS.service != NULL)
    {
      g_free(optRS.service);
      optRS.service = NULL;
    }

  optRS.service = g_strdup(service);
  return TRUE;
}

static gboolean gj_gtk_rs_set_require_username(gboolean require)
{
  optRS.require_username = require;
  return TRUE;
}

static gboolean gj_gtk_rs_set_require_password(gboolean require)
{
  optRS.require_password = require;
  return TRUE;
}

static gboolean gj_gtk_rs_set_require_email(gboolean require)
{
  optRS.require_email = require;
  return TRUE;
}

static gboolean gj_gtk_rs_set_require_nickname(gboolean require)
{
  optRS.require_nickname = require;
  return TRUE;
}


/*
 * external functions...
 */
gboolean gj_gtk_rs_load(const gchar *service)
{
  GladeXML *xml = NULL;

  if(ctlRS.window != NULL)
    {
      gtk_window_present(GTK_WINDOW(ctlRS.window));
      return TRUE;
    }

  if(service == NULL)
    {
      g_warning("%s: service was NULL", __FUNCTION__);
      return FALSE;
    }

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "register_service",
			      NULL,
			      "register_service", &ctlRS.window,
			      "label_instructions", &ctlRS.label_instructions,
			      "label_service", &ctlRS.label_service,
			      "label_username", &ctlRS.label_username,
			      "label_password", &ctlRS.label_password,
			      "label_email", &ctlRS.label_email,
			      "label_nickname", &ctlRS.label_nickname,
			      "entry_username", &ctlRS.entry_username,
			      "entry_password", &ctlRS.entry_password,
			      "entry_email", &ctlRS.entry_email,
			      "entry_nickname", &ctlRS.entry_nickname,
			      "button_ok", &ctlRS.button_ok,
			      "button_cancel", &ctlRS.button_cancel,
			      NULL);

  gj_gtk_glade_connect(xml,
		       NULL,
		       "register_service", "response", on_register_service_response,
		       NULL);

  if(!xml) return FALSE;
  g_object_unref(xml);

  /* init vars */
  optRS.key = NULL;
  gj_gtk_rs_set_service((gchar*)service);

  gj_gtk_rs_set_require_username(FALSE);
  gj_gtk_rs_set_require_password(FALSE);
  gj_gtk_rs_set_require_email(FALSE);
  gj_gtk_rs_set_require_nickname(FALSE);

/*   gdk_window_set_decorations (GDK_WINDOW(ctlRS.window->window),  */
/* 			      GDK_DECOR_BORDER | GDK_DECOR_TITLE); */

  gj_gtk_rs_setup(service);

  return TRUE;
}

gboolean gj_gtk_rs_unload()
{
  if(ctlRS.window == NULL)
    return TRUE;

  gtk_widget_destroy(ctlRS.window);
  ctlRS.window = NULL;

  if(optRS.key != NULL)
    {
      g_free(optRS.key);
      optRS.key = NULL;
    }

  if(optRS.service != NULL)
    {
      g_free(optRS.service);
      optRS.service = NULL;
    }

  optRS.require_username = FALSE;
  optRS.require_password = FALSE;
  optRS.require_email = FALSE;
  optRS.require_nickname = FALSE;  

  return TRUE;
}

static gboolean gj_gtk_rs_setup(const gchar *service)
{
  /* set up control */
  gtk_label_set_text(GTK_LABEL(ctlRS.label_service), service);

  p = gj_gtk_progress_start(TRUE,
			    GTK_WINDOW(ctlRS.window),
			    gj_gtk_rs_requirements_progress_cb, 
			    NULL, 
			    _("Requesting Registration Details for Service"), 
			    _("Service is \"%s\""), 
			    service);

  /* get request authentication from server */
  gj_registration_request_requirements(service, gj_gtk_rs_registration_requirements_response);  
  
  return TRUE;
}

static gboolean gj_gtk_rs_check()
{
  gboolean passed = FALSE;

  G_CONST_RETURN gchar *service = NULL;
  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *password = NULL;
  G_CONST_RETURN gchar *email = NULL;
  G_CONST_RETURN gchar *nickname = NULL;
  
  service = gtk_label_get_text(GTK_LABEL(ctlRS.label_service));
  username = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_username));
  password = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_password));
  email = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_email));
  nickname = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_nickname));


  /* do checks */
  if(service == NULL || g_utf8_strlen(service, -1) < 1)
    {
      gj_gtk_show_error(_("Details Incomplete."),
			1,
			_("Service was not specified."));

      passed = FALSE;
    }
  else if(gj_gtk_rs_get_require_username() == TRUE && (username == NULL || g_utf8_strlen(username, -1) < 1))
    {
      /* username not entered */
      gj_gtk_show_error(_("Details Incomplete."),
			2,
			_("Username was not specified."));

      passed = FALSE;
    }
  else if(gj_gtk_rs_get_require_password() == TRUE && (password == NULL || g_utf8_strlen(password, -1) < 1))
    {
      /* passwords not entered */
      gj_gtk_show_error(_("Details Incomplete."),
			3,
			_("Password was not specified."));

      passed = FALSE;
    }
  else if(gj_gtk_rs_get_require_email() == TRUE && (email == NULL || g_utf8_strlen(email, -1) < 1))
    {
      /* email address not entered */
      gj_gtk_show_error(_("Details Incomplete."),
			4,
			_("Email Address was not specified."));

      passed = FALSE;
    }
  else if(gj_gtk_rs_get_require_nickname() == TRUE && (nickname == NULL || g_utf8_strlen(nickname, -1) < 1))
    {
      /* nickname not entered */
      gj_gtk_show_error(_("Details Incomplete."),
			5,
			_("Nickname was not specified."));

      passed = FALSE;
    }
  else
    {
      /* ok */
      passed = TRUE;
    }

  return passed;
}

static void gj_gtk_rs_registration_requirements_response(GjRegistration rgr, 
							 gint error_code, 
							 const gchar *error_reason)
{
  g_message("%s: received response.", __FUNCTION__);

  if(p != NULL)
    {
      gj_gtk_progress_stop(p);
      p = NULL;
    }
  
  if(rgr == NULL)
    {
      g_warning("%s: rgr was NULL", __FUNCTION__);
      return;
    }

  if(error_code != 0 || error_reason != NULL)
    {
      /* if error close dialog */
      gj_gtk_rs_unload();

      /* warn about error - usually a timeout */
      gj_gtk_show_error(_("Failed to Register Service."),
			error_code,
			error_reason);
 
      /* clean up */
      gj_registration_free(rgr);

      return;
    }

  /* show window */
  gtk_widget_show_all(GTK_WIDGET(ctlRS.window));

  gdk_window_set_decorations (GDK_WINDOW(ctlRS.window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE);

  /* enable buttons */
  gtk_widget_show(GTK_WIDGET(ctlRS.button_ok));
  gtk_widget_show(GTK_WIDGET(ctlRS.button_cancel));

  /* set up controls */
  gtk_label_set_text(GTK_LABEL(ctlRS.label_instructions), gj_registration_get_instructions(rgr));

  if(gj_registration_get_require_username(rgr) == TRUE)
    {
      gtk_widget_show(GTK_WIDGET(ctlRS.label_username));
      gtk_widget_show(GTK_WIDGET(ctlRS.entry_username));
      gj_gtk_rs_set_require_username(TRUE);
    }

  if(gj_registration_get_require_password(rgr) == TRUE)
    {
      gtk_widget_show(GTK_WIDGET(ctlRS.label_password));
      gtk_widget_show(GTK_WIDGET(ctlRS.entry_password));
      gj_gtk_rs_set_require_password(TRUE);
    }

  if(gj_registration_get_require_email(rgr) == TRUE)
    {
      gtk_widget_show(GTK_WIDGET(ctlRS.label_email));
      gtk_widget_show(GTK_WIDGET(ctlRS.entry_email));
      gj_gtk_rs_set_require_email(TRUE);
    }

  if(gj_registration_get_require_nickname(rgr) == TRUE)
    {
      gtk_widget_show(GTK_WIDGET(ctlRS.label_nickname));
      gtk_widget_show(GTK_WIDGET(ctlRS.entry_nickname));
      gj_gtk_rs_set_require_nickname(TRUE);
    }

  if(gj_registration_get_key(rgr) == NULL)
    {
      g_warning("%s: no registration key for response", __FUNCTION__);
      return;
    }
  else
    gj_gtk_rs_set_key(gj_registration_get_key(rgr));

  /* clean up */
  gj_registration_free(rgr);
}

static void gj_gtk_rs_registration_response(GjRegistration rgr, 
					    gint error_code, 
					    const gchar *error_reason)
{
  g_message("%s: registration response", __FUNCTION__);
  
  if(p != NULL)
    {
      gj_gtk_progress_stop(p);
      p = NULL;
    }
  
  if(error_code == 0 && error_reason == NULL)
    {
      /* service not entered */
      gj_gtk_msg_new(TRUE,
		     GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_INFORMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("Registration Success."),
		     _("No Errors"),
		     NULL);  
    }
  else
    {
      gj_gtk_show_error(_("Failed to Register Service."),
			error_code,
			error_reason);
    }
  
  /* clean up */
  gj_registration_free(rgr);
}

static void gj_gtk_rs_requirements_progress_cb(gboolean cancelled, gpointer userdata)
{
  g_message("%s: cancelled:%s. userdata:0x%.8x", 
	    __FUNCTION__, cancelled?"yes":"no", (gint)userdata);
  
  if(cancelled == TRUE)
    gj_gtk_rs_unload();
}

static void gj_gtk_rs_register_progress_cb(gboolean cancelled, gpointer userdata)
{
  g_message("%s: cancelled:%s. userdata:0x%.8x",
	    __FUNCTION__, cancelled?"yes":"no", (gint)userdata);

  if(cancelled == TRUE)
    gj_gtk_rs_unload();
}

/*
 * actual gui functions
 */
static void on_register_service_response(GtkDialog *dialog, gint response, gpointer user_data)
{
  G_CONST_RETURN gchar *service = NULL;
  G_CONST_RETURN gchar *username = NULL;
  G_CONST_RETURN gchar *password = NULL;
  G_CONST_RETURN gchar *email = NULL;
  G_CONST_RETURN gchar *nickname = NULL;

  switch(response) 
    {
    case GTK_RESPONSE_OK:
      if(gj_gtk_rs_check() == FALSE)
	return;
      
      service = gtk_label_get_text(GTK_LABEL(ctlRS.label_service));
      username = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_username));
      password = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_password));
      email = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_email));
      nickname = gtk_entry_get_text(GTK_ENTRY(ctlRS.entry_nickname));
      
      p = gj_gtk_progress_start(TRUE,
				GTK_WINDOW(ctlRS.window),
				gj_gtk_rs_register_progress_cb, 
				NULL, 
				_("Attempting to Register Service"), 
				_("Service is \"%s\""), 
				service);
      
      gj_registration_request(service, gj_gtk_rs_registration_response, 
			      gj_gtk_rs_get_key(), 
			      username,
			      password, 
			      email, 
			      nickname);
      break;

    case GTK_RESPONSE_HELP:
    case GTK_RESPONSE_NONE:
    case GTK_RESPONSE_REJECT:
    case GTK_RESPONSE_ACCEPT:
    case GTK_RESPONSE_DELETE_EVENT:
    case GTK_RESPONSE_CANCEL:
    case GTK_RESPONSE_CLOSE:
    case GTK_RESPONSE_YES:
    case GTK_RESPONSE_NO:
    case GTK_RESPONSE_APPLY:
      break;
    }

  gj_gtk_rs_unload();
}


#ifdef __cplusplus
}
#endif
