/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_gtk_roster.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/22 14:52:30 $
 * 
 * $Revision: 1.117 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <string.h>
#include <stdlib.h>

#include <gdk/gdkkeysyms.h>

#include "gj_main.h"
#include "gj_iq.h"
#include "gj_iq_requests.h"
#include "gj_roster.h"
#include "gj_roster_item.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"
#include "gj_autoaway.h"
#include "gj_me.h"
#include "gj_parser.h"
#include "gj_event.h"
#include "gj_history.h"
#include "gj_config.h"
#include "gj_jid.h"
#include "gj_connection.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_support.h"
#include "gj_gtk_msg.h"
#include "gj_gtk_message.h"
#include "gj_gtk_send_message.h"
#include "gj_gtk_chat.h"
#include "gj_gtk_group_chat.h"
#include "gj_gtk_headline.h"
#include "gj_gtk_my_presence.h"
#include "gj_gtk_my_information.h"
#include "gj_gtk_my_preferences.h"
#include "gj_gtk_connection_progress.h"
#include "gj_gtk_connection_settings.h"
#include "gj_gtk_password_change.h"
#include "gj_gtk_log.h"
#include "gj_gtk_user_information.h"
#include "gj_gtk_register_new.h"
#include "gj_gtk_edit_groups.h"
#include "gj_gtk_group_message.h"
#include "gj_gtk_add_contact.h"
#include "gj_gtk_subscription_request.h"
#include "gj_gtk_about.h"
#include "gj_gtk_debug.h"
#include "gj_gtk_event_viewer.h"
#include "gj_gtk_group_chat_join.h"
#include "gj_gtk_browse_services.h"
/* #include "gj_gtk_directory_search.h" */
#include "gj_gtk_docklet.h"
#include "gj_gtk_rename_contact.h"
#include "gj_gtk_rename_group.h"
#include "gj_gtk_stock.h"
#include "gj_gtk_progress.h"


typedef struct 
{
  GjRoster r;

  gchar *group;
  
  gboolean found;
  GtkTreeIter found_iter;

} GjRosterWindowFindGroup;


typedef struct 
{
  GjRoster r;
  GjRosterItem ri;

  GjJID jid;

  gboolean found;
  GList *iters;

} GjRosterWindowFindUser;


/* 
 * roster columns 
 */
enum
  {
    COL_ROSTER_IMAGE = 0,
    COL_ROSTER_IMAGE_FLASH,
    COL_ROSTER_IMAGE_FLASH_ON,
    COL_ROSTER_TEXT,
    COL_ROSTER_ID,
    COL_ROSTER_NAME,
    COL_ROSTER_NOT_GROUP,
    COL_ROSTER_COUNT 
  };


typedef struct 
{
  GtkWidget *window; 

  GtkWidget *treeview;

  GtkWidget *statusbar; 
  GtkWidget *hbox;

/*   GtkWidget *optionmenu_presence; */
  GtkWidget *optionmenu_presence_presets;

  GtkWidget *button_error;

  GtkWidget *mnu_jabber_connect;
  GtkWidget *mnu_jabber_disconnect;
  GtkWidget *mnu_jabber_register_new_account;
  GtkWidget *mnu_jabber_unregister;
  GtkWidget *mnu_jabber_change_password;
  GtkWidget *mnu_jabber_preferences;
  GtkWidget *mnu_jabber_connection_settings;
  GtkWidget *mnu_jabber_my_information;
  GtkWidget *mnu_jabber_quit;

  GtkWidget *mnu_roster;
  GtkWidget *mnu_roster_add;
  GtkWidget *mnu_roster_send_message;
  GtkWidget *mnu_roster_group_chat;
  GtkWidget *mnu_roster_import;
  GtkWidget *mnu_roster_event_viewer;
  GtkWidget *mnu_roster_directory_search;
  GtkWidget *mnu_roster_browse_services;

  GtkWidget *mnu_help_homepage;
  GtkWidget *mnu_help_project_information;
  GtkWidget *mnu_help_log;
  GtkWidget *mnu_help_debug;
  GtkWidget *mnu_help_about;

  GtkWidget *eventbox_statusbar;

  GtkWidget *eventbox_presence;
  GtkWidget *image_presence;
  GtkWidget *label_presence;
  GtkWidget *entry_status;

  /* members */
  gint flash_timeout_id;
  GList *flash_list;

} GjRosterWindow;


const gchar *authors[] = {
  "Martyn Russell (ginxd@btopenworld.com)",
  "Colin Marquardt (gj@marquardt-home.de)",
  NULL
};

const gchar *documenters[] = {
  "-",
  NULL
};

const gchar *translators[] = {
  "Pedro Cuevas (es)",
  "Colin Marquardt (de)",
  /*     "Frdric Brin (fr)", - INVALID UTF8 */
  "Frederic Brin (fr)", 
  "Andrey Lebedev (ru)",
  "Ondra Kudlik (cs)",
  NULL
};


/* states */
static gboolean gj_gtk_roster_set_mode_connected(GjRosterWindow *window);
static gboolean gj_gtk_roster_set_mode_disconnected(GjRosterWindow *window);

static gboolean gj_gtk_roster_set_my_presence(GjRosterWindow *window, 
					      GjPresenceType type, 
					      GjPresenceShow show, 
					      const gchar *status, 
					      gint8 priority,
					      gboolean send_it);

/* model functions */
static gboolean gj_gtk_roster_model_setup();
static gboolean gj_gtk_roster_model_populate_columns(GjRosterWindow *window);

static gboolean gj_gtk_roster_model_add(GjRosterWindow *window, GjRosterItem ri);
static gboolean gj_gtk_roster_model_add_group(GjRosterWindow *window, const gchar *group, GtkTreeIter *iter);
static gboolean gj_gtk_roster_model_add_agent(GjRosterWindow *window, GjRosterItem ri);
static gboolean gj_gtk_roster_model_add_contact_resource(GjRosterWindow *window, GjRosterItem ri);
static gboolean gj_gtk_roster_model_add_contact_not_permanent(GjRosterWindow *window, GjRosterItem ri);
static gboolean gj_gtk_roster_model_add_contact_offline(GjRosterWindow *window, GjRosterItem ri);
static gboolean gj_gtk_roster_model_add_contact_permanent(GjRosterWindow *window, GjRosterItem ri);

static gboolean gj_gtk_roster_model_set_presence(GjRosterWindow *window, 
						 GjRosterItem ri, 
						 GjPresenceType type, 
						 GjPresenceShow show, 
						 const gchar *status);

static gboolean gj_gtk_roster_model_set_message(GjRosterWindow *window, GjRosterItem ri, GjMessageType type);

/* model finds */
static gboolean gj_gtk_roster_model_find_jid(GtkTreeView *view, GjJID jid, GList **iters);
static gboolean gj_gtk_roster_model_find_jid_foreach(GtkTreeModel *model, GtkTreePath *path, 
						     GtkTreeIter *iter, GjRosterWindowFindUser *data);

static gboolean gj_gtk_roster_model_find_group(GtkTreeView *view, const gchar *group, GtkTreeIter *iter);
static gboolean gj_gtk_roster_model_find_group_foreach(GtkTreeModel *model, GtkTreePath *path, 
						       GtkTreeIter *iter, GjRosterWindowFindGroup *data);

static void gj_gtk_roster_model_iter_list_destroy_cb(gpointer data, gpointer user_data);

/* model selections */
static GjRosterItem gj_gtk_roster_model_selection_get_ri(GjRosterWindow *window);
static gboolean gj_gtk_roster_model_selection_get_is_group(GjRosterWindow *window, gboolean *is_group, gchar **group);

/* roster contact functions */
static gboolean gj_gtk_roster_contact_read(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_new_message(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_new_chat(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_headlines(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_history(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_information(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_sub_request(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_sub_cancel(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_edit_groups(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_rename(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_add(GjRosterWindow *window);
static gboolean gj_gtk_roster_contact_remove(GjRosterWindow *window);

/* roster agent functions */
static gboolean gj_gtk_roster_agent_log_in(GjRosterWindow *window);
static gboolean gj_gtk_roster_agent_log_out(GjRosterWindow *window);


/* roster group functions */
static gboolean gj_gtk_roster_group_message(GjRosterWindow *window);
static gboolean gj_gtk_roster_group_rename(GjRosterWindow *window);
static gboolean gj_gtk_roster_group_remove(GjRosterWindow *window);

/* other */
static gboolean gj_gtk_roster_unregister(GjRosterWindow *window);

/* events */
static gboolean gj_gtk_roster_handle_next_event(GjRosterWindow *window, GjRosterItem ri);
/* static gboolean gj_gtk_roster_handle_next_event_display(GjRosterWindow *window, GjRosterItem ri);  */

static gboolean gj_gtk_roster_flash_timeout_add(GjRosterWindow *window, GjRosterItem ri);
static gboolean gj_gtk_roster_flash_timeout_remove(GjRosterWindow *window, GjRosterItem ri);
static gboolean gj_gtk_roster_flash_timeout_cb(GjRosterWindow *window);
static gboolean gj_gtk_roster_flash_timeout_update_roster(GjRosterWindow *window, GjRosterItem ri);

/* menus */
static gboolean gj_gtk_roster_generate_mnu_user(GjRosterWindow *window, GjRosterItem ri, GdkEventButton *gdk_event);
static gboolean gj_gtk_roster_generate_mnu_agent(GjRosterWindow *window, GjRosterItem ri, GdkEventButton *gdk_event);
static gboolean gj_gtk_roster_generate_mnu_group(GjRosterWindow *window, GdkEventButton *gdk_event);

/* callbacks */
static void gj_gtk_roster_event_cb(gint event_id, GjRosterWindow *window);

static void gj_gtk_roster_get_response(const GList *contacts, 
				       const GList *agents, 
				       GjRosterWindow *window);
static void gj_gtk_roster_set_response(const GList *updated_roster_items, 
				       GjRosterWindow *window);

static void gj_gtk_roster_user_changed_cb(guint event_id, guint related_event_id);

static void gj_gtk_roster_autoaway_cb(guint msec, GjRosterWindow *window);

static void gj_gtk_roster_contact_remove_cb(guint event_id, guint related_event_id);
static void gj_gtk_roster_unregister_cb(guint event_id, guint related_event_id);

/* menu callbacks  */
static void gj_gtk_roster_menu_contact_read_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_new_message_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_new_chat_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_headlines_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_history_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_information_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_sub_request_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_sub_cancel_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_edit_groups_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_rename_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_add_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_contact_remove_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_agent_log_in_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_agent_log_out_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_group_message_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_group_rename_cb(GtkMenuItem *menuitem, GjRosterWindow *window);
static void gj_gtk_roster_menu_group_remove_cb(GtkMenuItem *menuitem, GjRosterWindow *window);

static void gj_gtk_roster_presence_menu_cb(GtkMenuItem *menuitem, gpointer user_data);

#if 0
static void gj_gtk_roster_presence_preset_menu_cb(GtkMenuItem *menuitem, gpointer user_data);
#endif


/* gui callbacks */
static void on_mnu_jabber_connect_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_disconnect_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_register_new_account_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_unregister_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_change_password_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_preferences_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_connection_settings_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_my_information_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_jabber_quit_activate(GtkMenuItem *menuitem, GjRosterWindow *window);

static void on_mnu_roster_add_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_roster_send_message_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_roster_group_chat_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_roster_event_viewer_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_roster_directory_search_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_roster_browse_services_activate(GtkMenuItem *menuitem, GjRosterWindow *window);

static void on_mnu_help_homepage_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_help_project_information_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_help_log_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_help_debug_activate(GtkMenuItem *menuitem, GjRosterWindow *window);
static void on_mnu_help_about_activate(GtkMenuItem *menuitem, GjRosterWindow *window);

static void on_treeview_row_inserted(GtkTreeModel *treemodel, GtkTreePath *path, GtkTreeIter *iter, GjRosterWindow *window);
static void on_treeview_row_deleted(GtkTreeModel *treemodel, GtkTreePath *path, GjRosterWindow *window);
static gboolean on_treeview_button_press_event(GtkWidget *widget, GdkEventButton *gdk_event, GjRosterWindow *window);
static gboolean on_treeview_button_release_event(GtkWidget *widget, GdkEventButton *gdk_event, GjRosterWindow *window);

static gboolean on_eventbox_statusbar_button_press_event(GtkWidget *widget, GdkEventButton *event, GjRosterWindow *window);
/* static gboolean on_eventbox_error_button_press_event(GtkWidget *widget, GdkEventButton *event, GjRosterWindow *window); */

static void on_button_error_clicked(GtkButton *button, GjRosterWindow *window);

static void on_eventbox_presence_image_button_press_event(GtkWidget *widget, GdkEventButton *event, GjRosterWindow *window);
static void on_eventbox_presence_text_button_press_event(GtkWidget *widget, GdkEventButton *event, GjRosterWindow *window);
static void on_entry_status_activate(GtkEntry *entry, GjRosterWindow *window);
static gboolean on_entry_status_focus_out_event(GtkWidget *widget, GdkEventFocus *event, GjRosterWindow *window);
static gboolean on_entry_status_key_release_event(GtkWidget *widget, GdkEventKey *event, GjRosterWindow *window);

static void on_destroy(GtkWidget *widget, GjRosterWindow *window);
static gboolean on_delete_event(GtkWidget *widget, GdkEvent *event, GjRosterWindow *window);


static GjRoster roster = NULL;
static GjRosterWindow *current_window = NULL;


void gj_gtk_roster_cb_connected()
{
  GjConnection c = NULL;

  if(!current_window)
    {
      g_warning("%s: current window was NULL", __FUNCTION__);
      return;
    }

  /* new roster code */
  c = gj_connection_find_by_host_and_port(gj_me_get_connected_host(), 
					  gj_me_get_connected_port());
  roster = gj_roster_new(c, 
			 gj_me_get_connected_jid(),
			 (GjRosterCallback) gj_gtk_roster_get_response, 
			 (GjRosterSetCallback) gj_gtk_roster_set_response,
			 current_window);

  gj_roster_request(roster);

  /* set our presence */
  gj_gtk_roster_set_mode_connected(current_window);
}
 
void gj_gtk_roster_cb_disconnected()
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;

  if(!current_window)
    {
      g_warning("%s: current window was NULL", __FUNCTION__);
      return;
    }

  view = GTK_TREE_VIEW(current_window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));

  /* clear roster */
  gtk_tree_store_clear(store);
  
  gj_gtk_roster_set_mode_disconnected(current_window);
}

static void gj_gtk_roster_get_response(const GList *contacts, const GList *agents, GjRosterWindow *window)
{
  gint i = 0;
  gint count = 0;

  if(!window)
    return;

  for(i=0,count=0; i<g_list_length((GList*)contacts); i++)
    {
      GjRosterItem ri = NULL;
      GjJID j = NULL;
      gchar *groups = NULL;

      ri = (GjRosterItem) g_list_nth_data((GList*)contacts, i);
      if(ri == NULL) continue;

      groups = gj_roster_item_get_groups_as_string(ri);
      j = gj_roster_item_get_jid(ri);
      
      g_message("[CONTACT] (%2.2d) jid:'%s', name:'%s', groups:'%s', type:'%d'->'%s', subscription:'%d'->'%s', resource:'%s', agent:'%s'",
		i,
		gj_jid_get_full(j),
		gj_roster_item_get_name(ri),
		groups?groups:"none",
		gj_roster_item_get_type(ri),
		gj_translate_roster_item_type(gj_roster_item_get_type(ri)),
		gj_roster_item_get_subscription(ri),
		gj_translate_roster_item_subscription(gj_roster_item_get_subscription(ri)),
		gj_jid_get_resource(j)?gj_jid_get_resource(j):"",
		gj_jid_get_agent(j)?gj_jid_get_agent(j):"");
      
      gj_gtk_roster_model_add(window, ri);
      count++;

      /* clean up */
      if(groups != NULL) g_free(groups);
    }

  g_message("%s: %d contacts all together", __FUNCTION__, count);

  for(i=0,count=0; i<g_list_length((GList*)agents); i++)
    {
      GjRosterItem ri = NULL;
      GjJID j = NULL;
      gchar *groups = NULL;

      ri = (GjRosterItem) g_list_nth_data((GList*)agents, i);
      if(ri == NULL) continue;

      groups = gj_roster_item_get_groups_as_string(ri);
      j = gj_roster_item_get_jid(ri);
      
      g_message("[AGENT] (%2.2d) jid:'%s', name:'%s', groups:'%s', type:'%d'->'%s', subscription:'%d'->'%s', resource:'%s', agent:'%s'",
		i,
		gj_jid_get_full(j),
		gj_roster_item_get_name(ri),
		groups?groups:"none",
		gj_roster_item_get_type(ri),
		gj_translate_roster_item_type(gj_roster_item_get_type(ri)),
		gj_roster_item_get_subscription(ri),
		gj_translate_roster_item_subscription(gj_roster_item_get_subscription(ri)),
		gj_jid_get_resource(j)?gj_jid_get_resource(j):"",
		gj_jid_get_agent(j)?gj_jid_get_agent(j):"");
      
      gj_gtk_roster_model_add_agent(window, ri);
      count++;

      /* clean up */
      if(groups != NULL) g_free(groups);
    }

  g_message("%s: %d agents all together", __FUNCTION__, count);
}

static void gj_gtk_roster_set_response(const GList *updated_roster_items, GjRosterWindow *window)
{
  gint i = 0;

  if(updated_roster_items == NULL)
    return;

  for(i=0; i<g_list_length((GList*)updated_roster_items); i++)
    {
      GjRosterItem ri = NULL;

      ri = (GjRosterItem) g_list_nth_data((GList*)updated_roster_items, i);
      if(ri == NULL) continue;

      gj_gtk_roster_model_remove(ri);
      
      if(gj_roster_item_get_is_set_for_removal(ri) == FALSE)
	gj_gtk_roster_model_add(window, ri);
    }
}


/*
 * code
 */
gboolean gj_gtk_roster_load()
{
  GladeXML *xml = NULL;
  GdkPixbuf *pb = NULL;

  GjRosterWindow *window = NULL;
  
  window = g_new0(GjRosterWindow, 1);

  window->flash_timeout_id = -1;

  xml = gj_gtk_glade_get_file(gnome_jabber_get_glade_file(),
			      "roster",
			      NULL,
			      "roster", &window->window,
			      "treeview", &window->treeview,
			      "statusbar", &window->statusbar,
			      "mnu_jabber_connect", &window->mnu_jabber_connect,
			      "mnu_jabber_disconnect", &window->mnu_jabber_disconnect,
			      "mnu_jabber_register_new_account", &window->mnu_jabber_register_new_account,
			      "mnu_jabber_unregister", &window->mnu_jabber_unregister,
			      "mnu_jabber_change_password", &window->mnu_jabber_change_password,
			      "mnu_jabber_preferences", &window->mnu_jabber_preferences,
			      "mnu_jabber_connection_settings", &window->mnu_jabber_connection_settings,
			      "mnu_jabber_my_information", &window->mnu_jabber_my_information,
			      "mnu_jabber_quit", &window->mnu_jabber_quit,
			      "mnu_roster", &window->mnu_roster,
			      "mnu_roster_add", &window->mnu_roster_add,
			      "mnu_roster_send_message", &window->mnu_roster_send_message,
			      "mnu_roster_group_chat", &window->mnu_roster_group_chat,
			      "mnu_roster_event_viewer", &window->mnu_roster_event_viewer,
			      "mnu_roster_directory_search", &window->mnu_roster_directory_search,
			      "mnu_roster_browse_services", &window->mnu_roster_browse_services,
			      "mnu_help_homepage", &window->mnu_help_homepage,
			      "mnu_help_project_information", &window->mnu_help_project_information,
			      "mnu_help_log", &window->mnu_help_log,
			      "mnu_help_debug", &window->mnu_help_debug,
			      "mnu_help_about", &window->mnu_help_about,
/* 			      "hbox_presence", &window->hbox, */
/* 			      "optionmenu_presence", &window->optionmenu_presence, */
			      "optionmenu_presence_presets", &window->optionmenu_presence_presets,
			      "button_error", &window->button_error,
			      "eventbox_statusbar", &window->eventbox_statusbar,
			      "image_presence", &window->image_presence,
			      "label_presence", &window->label_presence,
			      "entry_status", &window->entry_status,
			      NULL);

  gj_gtk_glade_connect(xml,
		       window,
		       "roster", "delete_event", on_delete_event,
		       "roster", "destroy", on_destroy,
		       "button_error", "clicked", on_button_error_clicked,
		       "eventbox_statusbar", "button_press_event", on_eventbox_statusbar_button_press_event,
		       "treeview", "button_press_event", on_treeview_button_press_event,
		       "treeview", "button_release_event", on_treeview_button_release_event,
		       "mnu_jabber_connect", "activate", on_mnu_jabber_connect_activate,
		       "mnu_jabber_disconnect", "activate", on_mnu_jabber_disconnect_activate,
		       "mnu_jabber_register_new_account", "activate", on_mnu_jabber_register_new_account_activate,
		       "mnu_jabber_unregister", "activate", on_mnu_jabber_unregister_activate,
		       "mnu_jabber_change_password", "activate", on_mnu_jabber_change_password_activate,
		       "mnu_jabber_preferences", "activate", on_mnu_jabber_preferences_activate,
		       "mnu_jabber_connection_settings", "activate", on_mnu_jabber_connection_settings_activate,
		       "mnu_jabber_my_information", "activate", on_mnu_jabber_my_information_activate,
		       "mnu_jabber_quit", "activate", on_mnu_jabber_quit_activate,
		       "mnu_roster_add", "activate", on_mnu_roster_add_activate,
		       "mnu_roster_send_message", "activate", on_mnu_roster_send_message_activate,
		       "mnu_roster_group_chat", "activate", on_mnu_roster_group_chat_activate,
		       "mnu_roster_event_viewer", "activate", on_mnu_roster_event_viewer_activate,
		       "mnu_roster_directory_search", "activate", on_mnu_roster_directory_search_activate,
		       "mnu_roster_browse_services", "activate", on_mnu_roster_browse_services_activate,
		       "mnu_help_homepage", "activate", on_mnu_help_homepage_activate,
		       "mnu_help_project_information", "activate", on_mnu_help_project_information_activate,
		       "mnu_help_log", "activate", on_mnu_help_log_activate,
		       "mnu_help_debug", "activate", on_mnu_help_debug_activate,
		       "mnu_help_about", "activate", on_mnu_help_about_activate,
		       "eventbox_presence_image", "button_press_event", on_eventbox_presence_image_button_press_event,
		       "eventbox_presence_text", "button_press_event", on_eventbox_presence_text_button_press_event,
		       "entry_status", "activate", on_entry_status_activate,
		       "entry_status", "focus_out_event", on_entry_status_focus_out_event,
		       "entry_status", "key_release_event", on_entry_status_key_release_event,
		       NULL);


  if(!xml) return FALSE;
  g_object_unref(xml);

  gdk_window_set_decorations (GDK_WINDOW(window->window->window), 
			      GDK_DECOR_BORDER | GDK_DECOR_TITLE | 
			      GDK_DECOR_MENU | GDK_DECOR_MINIMIZE | GDK_DECOR_RESIZEH);

  /* remember */
  current_window = window;

  /* set presence */
  gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeUnAvailable, 
					 GjPresenceShowNormal, 
					 gj_config_get_cs_nickname(), 
					 gj_config_get_cs_priority(),
					 FALSE);

  /* set window icon */
  pb = gj_gtk_pixbuf_new_from_stock(GJ_STOCK_PRESENCE_AVAILABLE);
  gtk_window_set_icon(GTK_WINDOW(window->window), pb);
  g_object_unref(G_OBJECT(pb));

  /* start flash timer - for events */
  window->flash_timeout_id = g_timeout_add(1000, (GSourceFunc)gj_gtk_roster_flash_timeout_cb, window);

  /* set up roster */
  if(!gj_gtk_roster_model_setup(window))
    return FALSE;

  /* 
   * set callbacks up for messages/presence 
   */

  /* auto away */
  if(gj_autoaway_init((GjAutoAwayCallback)gj_gtk_roster_autoaway_cb, window) == FALSE)
    g_warning("%s: auto away could not be initialised and so is unavailable", __FUNCTION__);

  gj_me_set_is_auto_away(FALSE);

  /* events init */
  if(gj_event_init((GjEventCallback)gj_gtk_roster_event_cb, window) == FALSE)
    g_warning("%s: events could not be initialised", __FUNCTION__);

  /*
   * start external things:
   *  - event viewer, docklet, etc 
   */
  
  /* start applet */
  gj_gtk_docklet_load();

  /* set event viewer up */
  gj_gtk_ev_load(FALSE);

  /* get connection information */
  gj_config_read();

  /* should we auto connect ? */
  if(gj_config_get_cs_auto_connect_on_startup() == TRUE)
    gj_gtk_cp_load(gj_gtk_roster_cb_connected, gj_gtk_roster_cb_disconnected);

  return TRUE;
}

gboolean gj_gtk_roster_show(gboolean show)
{
  if(!current_window)
    return FALSE;

  if(current_window->window == NULL)
    return FALSE;
  
  if(show)
    gtk_window_present(GTK_WINDOW(current_window->window));
  else
    gtk_widget_hide_all(GTK_WIDGET(current_window->window));

  return TRUE;
}

static gboolean gj_gtk_roster_set_mode_connected(GjRosterWindow *window)
{
  GtkTreeView *view = NULL;
  
  if(!window)
    return FALSE;

  view = GTK_TREE_VIEW(window->treeview);

  /* set presence */
  gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeAvailable, 
					 GjPresenceShowNormal, 
					 gj_config_get_cs_nickname(), 
					 gj_config_get_cs_priority(),
					 TRUE);

  /* enable roster */
  gtk_widget_set_sensitive(window->treeview, TRUE);
  
  /* enable menu */
  gtk_widget_set_sensitive(window->mnu_jabber_connect,FALSE);
  gtk_widget_set_sensitive(window->mnu_jabber_disconnect,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_register_new_account,FALSE);
  gtk_widget_set_sensitive(window->mnu_jabber_unregister,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_change_password,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_preferences,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_connection_settings,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_my_information,TRUE);

  gtk_widget_set_sensitive(window->mnu_roster, TRUE);
  gtk_widget_set_sensitive(window->mnu_roster_event_viewer, TRUE);
  gtk_widget_set_sensitive(window->mnu_roster_directory_search, TRUE);
  gtk_widget_set_sensitive(window->mnu_roster_browse_services, TRUE);

  gtk_widget_set_sensitive(window->mnu_help_debug, TRUE);

  /* enable presence button */
  gtk_widget_set_sensitive(window->hbox,TRUE);

  return TRUE;
}

static gboolean gj_gtk_roster_set_mode_disconnected(GjRosterWindow *window)
{
  GtkTreeView *view = NULL;

  if(!window)
    return FALSE;

  view = GTK_TREE_VIEW(window->treeview);

  /* set presence */
  gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeUnAvailable, 
					 GjPresenceShowNormal, 
					 gj_config_get_cs_nickname(), 
					 gj_config_get_cs_priority(),
					 FALSE);

  /* disable roster */
  gtk_widget_set_sensitive(GTK_WIDGET(view), FALSE);

  /* enable menu */
  gtk_widget_set_sensitive(window->mnu_jabber_connect,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_disconnect,FALSE);
  gtk_widget_set_sensitive(window->mnu_jabber_register_new_account,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_unregister,FALSE);
  gtk_widget_set_sensitive(window->mnu_jabber_change_password,FALSE);
  gtk_widget_set_sensitive(window->mnu_jabber_preferences,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_connection_settings,TRUE);
  gtk_widget_set_sensitive(window->mnu_jabber_my_information,FALSE);

  gtk_widget_set_sensitive(window->mnu_roster, FALSE);
  gtk_widget_set_sensitive(window->mnu_roster_event_viewer, FALSE);
  gtk_widget_set_sensitive(window->mnu_roster_directory_search, FALSE);
  gtk_widget_set_sensitive(window->mnu_roster_browse_services, FALSE);

  gtk_widget_set_sensitive(window->mnu_help_debug, FALSE);

  /* disable presence button */
  gtk_widget_set_sensitive(window->hbox,FALSE);

  return TRUE;
}

void gj_gtk_roster_set_status_event_error(gboolean is_error)
{
  if(!current_window) 
    return;

  if(is_error == TRUE)
    gtk_widget_show(GTK_WIDGET(current_window->button_error));
  else
    gtk_widget_hide(GTK_WIDGET(current_window->button_error));
}

/*
 * MODEL functions
 */
static gboolean gj_gtk_roster_model_setup(GjRosterWindow *window)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeSelection *selection = NULL;

  if(!window)
    return FALSE;

  view = GTK_TREE_VIEW(window->treeview);
  selection = gtk_tree_view_get_selection(view);

  /* store */
  store = gtk_tree_store_new(COL_ROSTER_COUNT,
			     GDK_TYPE_PIXBUF, /* image */
			     GDK_TYPE_PIXBUF, /* old image (used for message/presence icon...etc) */
			     G_TYPE_BOOLEAN,  /* boolean for flash */
			     G_TYPE_STRING,   /* text */
			     G_TYPE_INT,      /* id - roster item id */
			     G_TYPE_STRING,   /* name - used only for groups */
			     G_TYPE_BOOLEAN); /* is group */


  gj_gtk_roster_model_populate_columns(window);
			     
  /* model */
  model = GTK_TREE_MODEL(store);
  gtk_tree_view_set_model(view, model);

  g_signal_connect (G_OBJECT(model), "row-inserted", G_CALLBACK(on_treeview_row_inserted), window);
  g_signal_connect (G_OBJECT(model), "row-deleted", G_CALLBACK(on_treeview_row_deleted), window);

  /* properties */
  gtk_tree_view_set_enable_search(view, TRUE); 
  gtk_tree_view_set_search_column(view, COL_ROSTER_NAME);  
  gtk_tree_view_set_rules_hint(view, FALSE); 
  gtk_tree_view_set_headers_visible(view, FALSE);  
  gtk_tree_view_set_headers_clickable(view, FALSE); 
  gtk_tree_view_set_reorderable(view, FALSE);
  gtk_tree_view_columns_autosize(view); 
  /*    gtk_tree_view_expand_all(view);   */

  /* sort list */
  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(model), COL_ROSTER_TEXT, GTK_SORT_ASCENDING);

  /* clean up */
  g_object_unref(G_OBJECT(store));

  return TRUE;
}

gboolean gj_gtk_roster_model_reload()
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;

  gint i = 0;
  gint count = 0;

  const GList *contacts = NULL;
  const GList *agents = NULL;

  if(!current_window)
    return FALSE;

  contacts = gj_roster_get_contacts(roster);
  agents = gj_roster_get_agents(roster);

  view = GTK_TREE_VIEW(current_window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));

  /* clear roster */
  gtk_tree_store_clear(store);
  
  /* populate roster */
  for(i=0,count=0; i<g_list_length((GList*)contacts); i++)
    {
      GjRosterItem ri = NULL;

      ri = (GjRosterItem) g_list_nth_data((GList*)contacts, i);
      if(ri == NULL) continue;

      gj_gtk_roster_model_add(current_window, ri);
    }

  for(i=0,count=0; i<g_list_length((GList*)agents); i++)
    {
      GjRosterItem ri = NULL;

      ri = (GjRosterItem) g_list_nth_data((GList*)agents, i);
      if(ri == NULL) continue;

      gj_gtk_roster_model_add(current_window, ri);
    }
  
  return TRUE;
}

static gboolean gj_gtk_roster_model_populate_columns(GjRosterWindow *window)
{
  GtkTreeView *view = NULL;
  GtkTreeViewColumn *column = NULL;
  GtkCellRenderer *renderer = NULL;

  if(!window)
    return FALSE;

  view = GTK_TREE_VIEW(window->treeview);

  /* cell renderer's */
  column = gtk_tree_view_column_new();

  /* COL_ROSTER_IMAGE */  
  renderer = gtk_cell_renderer_pixbuf_new();
  gtk_tree_view_column_pack_start(column, renderer, FALSE);
  gtk_tree_view_column_set_attributes(column,
				      renderer,
				      "visible", COL_ROSTER_NOT_GROUP,
				      "pixbuf", COL_ROSTER_IMAGE,
				      NULL);

  /* COL_ROSTER_TEXT */
  renderer = gtk_cell_renderer_text_new();
  gtk_tree_view_column_pack_start(column, renderer, TRUE);
  gtk_tree_view_column_set_attributes(column,
				      renderer,
				      "markup", COL_ROSTER_TEXT,
				      NULL);

  /* insert column */
  gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);

  return TRUE;
}

static gboolean gj_gtk_roster_model_add(GjRosterWindow *window, GjRosterItem ri)
{
  GjPresence pres = NULL;
  GjJID j = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  if((j = gj_roster_item_get_jid(ri)) == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if((pres = gj_roster_item_get_presence(ri)) == NULL)
    {
      g_warning("%s: could not get presence from roster item with JID:'%s'", __FUNCTION__, gj_jid_get_full(j)); 
      return FALSE;
    }

  /* agents */
  if(gj_jid_get_type(j) == GjJIDTypeAgent)
    {
      if(gj_config_get_mpf_hide_agents() == TRUE)
	{
	  g_message("%s: agents not to be shown, not adding jid:'%s'", 
		    __FUNCTION__, gj_jid_get_full(j));
	  return FALSE;
	}

      return gj_gtk_roster_model_add_agent(window, ri);
    }

  /* other resources */
  if(gj_roster_item_get_is_another_resource(ri) == TRUE)
    return gj_gtk_roster_model_add_contact_resource(window, ri);

  /* unknown contacts - was && */
  if(gj_roster_item_get_is_permanent(ri) == FALSE || 
     gj_roster_item_get_subscription(ri) == GjRosterItemSubscriptionNone)
    return gj_gtk_roster_model_add_contact_not_permanent(window, ri);

  g_message("%s: roster item's current presence type:%d->'%s'", 
	    __FUNCTION__, 
	    gj_presence_get_type(pres), 
	    gj_translate_presence_type(gj_presence_get_type(pres)));

  /* offline contacts */
  if(gj_presence_get_type(pres) == GjPresenceTypeUnAvailable || 
     gj_presence_get_type(pres) == GjPresenceTypeSubscribed || 
     gj_presence_get_type(pres) == GjPresenceTypeError)
    return gj_gtk_roster_model_add_contact_offline(window, ri);

  /* all others */
  if(gj_gtk_roster_model_add_contact_permanent(window, ri) == FALSE)
    return FALSE;
  
  /* should we play sound ? */
  if(gj_config_get_mpf_sound_on_user_online() == TRUE)
    gj_gtk_sound_play(gj_config_get_mpf_sound_file_on_user_online());

  return TRUE;
}

static gboolean gj_gtk_roster_model_add_contact_resource(GjRosterWindow *window, GjRosterItem ri)
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeModel *model = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;
  GtkTreeIter iter_child;

  gchar *group = NULL;    /* ROSTER_GROUP_NOT_ON_ROSTER; */
  GdkPixbuf *pb = NULL;

  GjJID j = NULL;
  GjPresence pres = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__); 
      return FALSE;
    }

  j = gj_roster_item_get_jid(ri);
  pres = gj_roster_item_get_presence(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  /* set pixbuf and group */
  pb = gj_gtk_pixbuf_new_from_stock(GJ_STOCK_PRESENCE_AVAILABLE);
  group = g_strdup(ROSTER_GROUP_RESOURCE);

  /* find group */
  if(gj_gtk_roster_model_add_group(window, group, &iter) == FALSE)
    {
      if(group != NULL) g_free(group);
      if(pb != NULL) g_object_unref(G_OBJECT(pb));
      return FALSE;
    }

  /* add to tree */
  gtk_tree_store_insert(store, &iter_child, &iter, 0);
  gtk_tree_store_set(store, &iter_child,
		     COL_ROSTER_IMAGE, pb,
		     COL_ROSTER_IMAGE_FLASH, NULL,
		     COL_ROSTER_IMAGE_FLASH_ON, FALSE,
		     COL_ROSTER_TEXT, gj_jid_get_resource(j),
		     COL_ROSTER_ID, gj_roster_item_get_id(ri),
		     COL_ROSTER_NAME, gj_roster_item_get_name(ri),
		     COL_ROSTER_NOT_GROUP, TRUE,
		     -1);

  path = gtk_tree_model_get_path(model, &iter_child);
  gtk_tree_model_row_changed(model, path, &iter_child);

  /* clean up */
  if(group != NULL) g_free(group);
  if(path != NULL) gtk_tree_path_free(path);
  if(pb != NULL) g_object_unref(G_OBJECT(pb));

  return TRUE;
}

static gboolean gj_gtk_roster_model_add_contact_not_permanent(GjRosterWindow *window, GjRosterItem ri)
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeModel *model = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;
  GtkTreeIter iter_child;

  gchar *group = NULL;    /* ROSTER_GROUP_NOT_ON_ROSTER; */
  GdkPixbuf *pb = NULL;

  GjPresence pres = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__); 
      return FALSE;
    }

  pres = gj_roster_item_get_presence(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  /* set pixbuf and group */
  pb = gj_gtk_pixbuf_new_from_stock(GJ_STOCK_PRESENCE_NONE);
  group = g_strdup(ROSTER_GROUP_NOT_ON_ROSTER);

  /* find group */
  if(gj_gtk_roster_model_add_group(window, group, &iter) == FALSE)
    {
      if(group != NULL) g_free(group);
      if(pb != NULL) g_object_unref(G_OBJECT(pb));
      return FALSE;
    }

  /* add to tree */
  gtk_tree_store_insert(store, &iter_child, &iter, 0);
  gtk_tree_store_set(store, &iter_child,
		     COL_ROSTER_IMAGE, pb,
		     COL_ROSTER_IMAGE_FLASH, NULL,
		     COL_ROSTER_IMAGE_FLASH_ON, FALSE,
		     COL_ROSTER_TEXT, gj_roster_item_get_name(ri),
		     COL_ROSTER_ID, gj_roster_item_get_id(ri),
		     COL_ROSTER_NAME, gj_roster_item_get_name(ri),
		     COL_ROSTER_NOT_GROUP, TRUE,
		     -1);

  path = gtk_tree_model_get_path(model, &iter_child);
  gtk_tree_model_row_changed(model, path, &iter_child);

  /* clean up */
  if(group != NULL) g_free(group);
  if(path != NULL) gtk_tree_path_free(path);
  if(pb != NULL) g_object_unref(G_OBJECT(pb));
      
  return TRUE;
}

static gboolean gj_gtk_roster_model_add_contact_offline(GjRosterWindow *window, GjRosterItem ri)
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeModel *model = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;
  GtkTreeIter iter_child;

  gchar *group = ROSTER_GROUP_OFFLINE;
  gchar *group_list = NULL;
  gchar *name = NULL;
  GdkPixbuf *pb = NULL;

  GjPresence pres = NULL;
  GjJID j = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__); 
      return FALSE;
    }

  pres = gj_roster_item_get_presence(ri);
  j = gj_roster_item_get_jid(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);


  /* if hide offline contacts do not show */ 
  /* if hide offline group do not show */
  /* if show offline group put contacts in offline group */
  /* if show offline contacts and hide offline group show as normal */

  /*
    if hide offline contacts and hide offline group return
    if hide offline contacts and show offline group SHOULD NOT GET HERE. - so return
    if show offline contacts and hide offline group call gj_gtk_roster_model_add_contact_permanent
    if show offline contacts and show offline group continue
  */

  if(gj_config_get_mpf_hide_offline_contacts() == TRUE && gj_config_get_mpf_hide_offline_group() == TRUE)
    {
      g_message("%s: hiding offline contacts, hiding offline group, not adding jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
      return FALSE;
    }

  if(gj_config_get_mpf_hide_offline_contacts() == TRUE && gj_config_get_mpf_hide_offline_group() == FALSE)
    {
      g_message("%s: hiding offline contacts, showing offline group, not adding jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
      return FALSE;
    }

  if(gj_config_get_mpf_hide_offline_contacts() == FALSE && gj_config_get_mpf_hide_offline_group() == TRUE)
    {
      g_message("%s: offline group hidden, showing offline contacts, adding permanent contact jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
      return gj_gtk_roster_model_add_contact_permanent(window, ri);
    }

  /* find group */
  if(gj_gtk_roster_model_add_group(window, group, &iter) == FALSE)
    return FALSE;
 
  /* set pixbuf */
  pb = gj_gtk_presence_to_pixbuf_new(gj_presence_get_type(pres), gj_presence_get_show(pres));

  /* set name - include groups */
  group_list = gj_roster_item_get_groups_as_string(ri);
  name = g_strdup_printf("%s %s%s%s",
			 gj_roster_item_get_name(ri),
			 group_list?"[":"",
			 group_list?group_list:"",
			 group_list?"]":"");
  
  if(group_list != NULL) g_free(group_list);

  /* add to tree */
  gtk_tree_store_insert(store, &iter_child, &iter, 0);
  gtk_tree_store_set(store, &iter_child,
		     COL_ROSTER_IMAGE, pb,
		     COL_ROSTER_IMAGE_FLASH, NULL,
		     COL_ROSTER_IMAGE_FLASH_ON, FALSE,
		     COL_ROSTER_TEXT, name,
		     COL_ROSTER_ID, gj_roster_item_get_id(ri),
		     COL_ROSTER_NAME, name,
		     COL_ROSTER_NOT_GROUP, TRUE, 
		     -1);

  path = gtk_tree_model_get_path(model, &iter_child);
  gtk_tree_model_row_changed(model, path, &iter_child);

  /* clean up */
  if(path != NULL) gtk_tree_path_free(path);
  if(pb != NULL) g_object_unref(G_OBJECT(pb));
      
  return TRUE;
}

static gboolean gj_gtk_roster_model_add_contact_permanent(GjRosterWindow *window, GjRosterItem ri)
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeModel *model = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter_child;
  
  gint i = 0;

  GjPresence pres = NULL;
  GjJID j = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__); 
      return FALSE;
    }

  j = gj_roster_item_get_jid(ri);
  pres = gj_roster_item_get_presence(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  g_message("%s: contact:'%s' belongs to %d groups", 
	    __FUNCTION__, gj_jid_get_full(j), g_list_length(gj_roster_item_get_groups(ri)));
  
  /* loop through and add contact for each group they belong to */
  for(i=0;gj_roster_item_get_groups(ri) != NULL && i < g_list_length(gj_roster_item_get_groups(ri));i++)
    {
      GtkTreeIter iter;

      GdkPixbuf *pb = NULL;
      gchar *group = NULL;
      gchar *name = NULL;

      group = (gchar*) g_list_nth_data(gj_roster_item_get_groups(ri), i); 
      g_message("%s: looking for group:'%s', index:%d", __FUNCTION__, group, i);
      
      /* find group */
      if(gj_gtk_roster_model_add_group(window, group, &iter) == FALSE)
	return FALSE;

      /* pixbuf */
      pb = gj_gtk_presence_to_pixbuf_new(gj_presence_get_type(pres), gj_presence_get_show(pres));

      if(gj_config_get_mpf_hide_contact_presence_details() == FALSE &&
	 gj_jid_get_type(j) == GjJIDTypeContact)
	{
	  gchar *s1 = NULL;
	  gchar *s2 = NULL;

	  if(gj_roster_item_get_name(ri) != NULL)
	    s1 = g_markup_escape_text(gj_roster_item_get_name(ri), -1);
	  if(gj_presence_get_status(pres) != NULL)
	    s2 = g_markup_escape_text(gj_presence_get_status(pres), -1);
	  if(s2 == NULL)
	    s2 = gj_gtk_presence_to_status_text_new(gj_presence_get_type(pres), gj_presence_get_show(pres));
	  
	  name = g_strdup_printf(ROSTER_CONTACT_PRESENCE_DETAILS_FORMAT, s1, s2==NULL?"":s2);
	  
	  if(s1 != NULL) g_free(s1);
	  if(s2 != NULL) g_free(s2);
	}

      /* add to store */
      gtk_tree_store_insert(store, &iter_child, &iter, 0);

      /* set store */
      gtk_tree_store_set(store, &iter_child,
			 COL_ROSTER_IMAGE, pb,
			 COL_ROSTER_IMAGE_FLASH, NULL,
			 COL_ROSTER_IMAGE_FLASH_ON, FALSE,
			 COL_ROSTER_TEXT, name!=NULL?name:gj_roster_item_get_name(ri),
			 COL_ROSTER_ID, gj_roster_item_get_id(ri),
			 COL_ROSTER_NAME, name!=NULL?name:gj_roster_item_get_name(ri),
			 COL_ROSTER_NOT_GROUP, TRUE,
			 -1);

      path = gtk_tree_model_get_path(model, &iter_child);
      gtk_tree_model_row_changed(model, path, &iter_child);

      /* clean up */
      if(path != NULL) gtk_tree_path_free(path);
      if(pb != NULL) g_object_unref(G_OBJECT(pb));
      if(name != NULL) g_free(name);

      path = NULL;
    }
  
  /*
    NOTE: this section is used for contacts which do not have a group
    and although this is totally plausable, future considerations 
    may involve contacts having a default pool/group when they do 
    not already belong to one.
  */
  if(gj_roster_item_get_groups(ri) == NULL)
    {
      GdkPixbuf *pb = NULL;
      gchar *name = NULL;

      g_message("%s: contact:'%s' has no group", __FUNCTION__, gj_jid_get_full(j));

      /* get image */
      pb = gj_gtk_presence_to_pixbuf_new(gj_presence_get_type(pres), gj_presence_get_show(pres));

      if(gj_config_get_mpf_hide_contact_presence_details() == FALSE &&
	 gj_jid_get_type(j) == GjJIDTypeContact)
	{
	  gchar *s1 = NULL;
	  gchar *s2 = NULL;
	  
	  if(gj_roster_item_get_name(ri) != NULL)
	    s1 = g_markup_escape_text(gj_roster_item_get_name(ri), -1);
	  if(gj_presence_get_status(pres) != NULL)
	    s2 = g_markup_escape_text(gj_presence_get_status(pres), -1);
	  if(s2 == NULL)
	    s2 = gj_gtk_presence_to_status_text_new(gj_presence_get_type(pres), gj_presence_get_show(pres));
	  
	  name = g_strdup_printf(ROSTER_CONTACT_PRESENCE_DETAILS_FORMAT, s1, s2==NULL?"":s2);
	  
	  if(s1 != NULL) g_free(s1);
	  if(s2 != NULL) g_free(s2);
	}

      /* add to store */
      gtk_tree_store_insert(store, &iter_child, NULL, 0);
      gtk_tree_store_set(store, &iter_child,
			 COL_ROSTER_IMAGE, pb,
			 COL_ROSTER_IMAGE_FLASH, NULL,
			 COL_ROSTER_IMAGE_FLASH_ON, FALSE,
			 COL_ROSTER_TEXT, name!=NULL?name:gj_roster_item_get_name(ri),
			 COL_ROSTER_ID, gj_roster_item_get_id(ri),
			 COL_ROSTER_NAME, name!=NULL?name:gj_roster_item_get_name(ri),
			 COL_ROSTER_NOT_GROUP, TRUE,
			 -1);

      path = gtk_tree_model_get_path(model, &iter_child);
      gtk_tree_model_row_changed(model, path, &iter_child);

      /* clean up */
      if(path != NULL) gtk_tree_path_free(path);
      if(pb != NULL) g_object_unref(G_OBJECT(pb));
      if(name != NULL) g_free(name);
    }

  return TRUE;
}

static gboolean gj_gtk_roster_model_add_group(GjRosterWindow *window, const gchar *group, GtkTreeIter *iter)
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeModel *model = NULL;
  GtkTreePath *path = NULL;

  gchar *s1 = NULL;
  gchar *markup = NULL;

  if(!window)
    return FALSE;

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  if(group == NULL)
    {
      g_warning("%s: group was NULL", __FUNCTION__);
      return FALSE;
    }

  if(iter == NULL)
    {
      g_warning("%s: iter was NULL", __FUNCTION__);
      return FALSE;
    }

  if(gj_gtk_roster_model_find_group(view, group, iter) == TRUE)
    {
      g_message("%s: group:'%s' already exists", __FUNCTION__, group); 
      return TRUE;
    } 

  s1 = g_markup_escape_text(group, -1);
  markup = g_strdup_printf("<b>%s</b>", s1);
      
  /* add contact to store */
  gtk_tree_store_insert(store, iter, NULL, 0);
  gtk_tree_store_set(store, iter,
		     COL_ROSTER_IMAGE, NULL, /* pb */
		     COL_ROSTER_IMAGE_FLASH, NULL,
		     COL_ROSTER_IMAGE_FLASH_ON, FALSE,
		     COL_ROSTER_TEXT, markup,
		     COL_ROSTER_ID, -1,
		     COL_ROSTER_NAME, s1,
		     COL_ROSTER_NOT_GROUP, FALSE,
		     -1);

  path = gtk_tree_model_get_path(model, iter);
  gtk_tree_model_row_changed(model, path, iter);

  /* clean up */
  if(path != NULL) gtk_tree_path_free(path);
  if(s1 != NULL) g_free(s1);
  if(markup != NULL) g_free(markup);

  return TRUE;
}

static gboolean gj_gtk_roster_model_add_agent(GjRosterWindow *window, GjRosterItem ri)
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeModel *model = NULL;
  GtkTreePath *path = NULL;
  GtkTreeIter iter;
  GtkTreeIter iter_child;

  gchar *group = ROSTER_GROUP_AGENTS;
  GdkPixbuf *pb = NULL;

  GjPresence pres = NULL;
  GjJID j = NULL;

  if(!window)
    return FALSE;
  
  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__); 
      return FALSE;
    }
  
  j = gj_roster_item_get_jid(ri);
  pres = gj_roster_item_get_presence(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  if(gj_config_get_mpf_hide_agents() == TRUE)
    {
      g_message("%s: hiding offline agents, not adding jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
      return FALSE;
    }

  /* find group */
  if(gj_gtk_roster_model_add_group(window, group, &iter) == FALSE)
    return FALSE;
 
  /* set pixbuf */
  pb = gj_gtk_presence_to_pixbuf_new(gj_presence_get_type(pres), gj_presence_get_show(pres));

  /* add to tree */
  gtk_tree_store_insert(store, &iter_child, &iter, 0);
  gtk_tree_store_set(store, &iter_child,
		     COL_ROSTER_IMAGE, pb,
		     COL_ROSTER_IMAGE_FLASH, NULL,
		     COL_ROSTER_IMAGE_FLASH_ON, FALSE,
		     COL_ROSTER_TEXT, gj_roster_item_get_name(ri),
		     COL_ROSTER_ID, gj_roster_item_get_id(ri),
		     COL_ROSTER_NAME, gj_roster_item_get_name(ri),
		     COL_ROSTER_NOT_GROUP, TRUE, 
		     -1);

  path = gtk_tree_model_get_path(model, &iter_child);
  gtk_tree_model_row_changed(model, path, &iter_child);

  /* clean up */
  if(path != NULL) gtk_tree_path_free(path);
  if(pb != NULL) g_object_unref(G_OBJECT(pb));

  return TRUE;
}

gboolean gj_gtk_roster_model_remove(GjRosterItem ri)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeIter iter;

  gboolean valid = FALSE;
  gboolean removed = FALSE;
  gint count = 0;

  GjJID j = NULL;

  if(!current_window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  j = gj_roster_item_get_jid(ri);

  view = GTK_TREE_VIEW(current_window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  /* NOTE: at this point, we need to continue searching
     the roster for other copies of the same JID for different
     groups. */

  /* get the first iter in the list */
  valid = gtk_tree_model_get_iter_first(model, &iter);

  for(; valid == TRUE; removed = FALSE)
    {
      /* check for children */
      if(gtk_tree_model_iter_has_child(model, &iter) == TRUE)
	{
	  GtkTreeIter iter_child;
	  gint children = 0;
	  gint index = 0;

	  children = gtk_tree_model_iter_n_children(model, &iter);

	  for(index=0; index<children; index++)
	    {
	      if((valid = gtk_tree_model_iter_nth_child(model, &iter_child, &iter, index)) == TRUE)
		{
		  gint id = 0;
		  
		  gtk_tree_model_get(model, &iter_child, COL_ROSTER_ID, &id, -1);
		  if(id == gj_roster_item_get_id(ri))
		    {
		      g_message("%s: removing jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
			  
		      if(children <= 1)
			valid = gtk_tree_store_remove(store, &iter);
		      else
			valid = gtk_tree_store_remove(store, &iter_child);

		      removed = TRUE;
		      count++;
		    }
		}
	    }
	}
      else
	{
	  gint id = 0;
	  
	  gtk_tree_model_get(model, &iter, COL_ROSTER_ID, &id, -1);
	  if(id == gj_roster_item_get_id(ri))
	    {
	      g_message("%s: removing jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
	      valid = gtk_tree_store_remove(store, &iter);
	      
	      removed = TRUE;
	      count++;
	    }
	}  

      /* this has to be done here... because of the condition 
	 if an item is removed, the next item is what the iter is set to. */
      if(removed == FALSE) valid = gtk_tree_model_iter_next(model, &iter);
    }

  if(count == 0)
    g_warning("%s: jid:'%s' NOT found", __FUNCTION__, gj_jid_get_full(j));
  
  return TRUE;
}

static gboolean gj_gtk_roster_model_find_jid(GtkTreeView *view, GjJID jid, GList **iters)
{
  GtkTreeModel *model = NULL;
  GjRosterWindowFindUser data;
  
  if(view == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if(jid == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }

  data.r = NULL;
  data.ri = NULL;
  data.found = FALSE;
  data.jid = jid;
  data.iters = NULL;
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  
  gtk_tree_model_foreach(model,
			 (GtkTreeModelForeachFunc) gj_gtk_roster_model_find_jid_foreach,
			 &data);
  
  if(data.found == TRUE) 
    {
      *iters = data.iters;
      return TRUE;
    }
  
  return FALSE;
}

static gboolean gj_gtk_roster_model_find_jid_foreach(GtkTreeModel *model, GtkTreePath *path, 
						     GtkTreeIter *iter, GjRosterWindowFindUser *data)
{
  gint id = -1;
  GjRosterItem ri = NULL;

  gtk_tree_model_get(model, iter, COL_ROSTER_ID, &id, -1);
  if(id == -1) return FALSE;

  ri = gj_rosters_find_item_by_id(id);
  if(ri == NULL) return FALSE;

  if(data->jid && gj_jid_equals(data->jid, gj_roster_item_get_jid(ri)) == TRUE)
    {
      data->found = TRUE;
      data->iters = g_list_append(data->iters, gtk_tree_iter_copy(iter));

      /* we would return TRUE here to stop, but there may be more ... */
    }
  
  return FALSE;
}

static gboolean gj_gtk_roster_model_find_group(GtkTreeView *view, const gchar *group, GtkTreeIter *iter)
{
  GtkTreeModel *model = NULL;
  GjRosterWindowFindGroup data;
  
  if(view == NULL)
    {
      g_warning("%s: view was NULL", __FUNCTION__);
      return FALSE;
    }
  
  if(group == NULL)
    {
      g_warning("%s: jid was NULL", __FUNCTION__);
      return FALSE;
    }
  
  data.r = NULL;
  data.group = (gchar*)group;
  data.found = FALSE;
  /*   data.found_iter = NULL; */
  
  model = gtk_tree_view_get_model(GTK_TREE_VIEW(view));
  
  gtk_tree_model_foreach(model,
			 (GtkTreeModelForeachFunc) gj_gtk_roster_model_find_group_foreach,
			 &data);
  
  if(data.found == TRUE) 
    {
      *iter = data.found_iter;
      return TRUE;
    }
  
  return FALSE;
}

static gboolean gj_gtk_roster_model_find_group_foreach(GtkTreeModel *model, GtkTreePath *path, 
						       GtkTreeIter *iter, GjRosterWindowFindGroup *data) 
{
  gboolean not_group = TRUE;
  gchar *group = NULL;

  gtk_tree_model_get(model, iter, 
		     COL_ROSTER_NOT_GROUP, &not_group, 
		     COL_ROSTER_NAME, &group,
		     -1);
  
  if(group == NULL) return FALSE;
  if(not_group) 
    {
      g_free(group);
      return FALSE;
    }

  if(data->group && g_strcasecmp(data->group, group) == 0)
    {
      data->found = TRUE;
      data->found_iter = *iter;
      return TRUE;
    }
  
  return FALSE;
}

static void gj_gtk_roster_model_iter_list_destroy_cb(gpointer data, gpointer user_data)
{
  GtkTreeIter *iter = NULL;

  iter = (GtkTreeIter*) data;
  if(iter == NULL) return;
  
  gtk_tree_iter_free(iter);
}

static gboolean gj_gtk_roster_model_selection_get_is_group(GjRosterWindow *window, gboolean *not_group, gchar **group)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeIter iter;

  if(!window)
    return FALSE;

  view = GTK_TREE_VIEW(window->treeview);
  model = gtk_tree_view_get_model(view);
  selection = gtk_tree_view_get_selection(view);

  if(not_group == NULL || group == NULL)
    {
      g_warning("%s: is_group or group pointers passed were NULL", __FUNCTION__);
      return FALSE;
    }

  if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
    return FALSE;
  
  gtk_tree_model_get(model, &iter, 
		     COL_ROSTER_NAME, group,
		     COL_ROSTER_NOT_GROUP, not_group, 
		     -1);     
  return TRUE;
}

static GjRosterItem gj_gtk_roster_model_selection_get_ri(GjRosterWindow *window)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeIter iter;

  gint id = 0;
 
  GjRosterItem ri = NULL;

  if(!window)
    return NULL;

  view = GTK_TREE_VIEW(window->treeview);
  model = gtk_tree_view_get_model(view);
  selection = gtk_tree_view_get_selection(view);

  if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
    return NULL;
  
  gtk_tree_model_get(model, &iter, COL_ROSTER_ID, &id, -1);     
  ri = gj_rosters_find_item_by_id(id);

  return ri;
}

/* 
 * ROSTER properties
 */
static gboolean gj_gtk_roster_model_set_presence(GjRosterWindow *window,
						 GjRosterItem ri, 
						 GjPresenceType type, 
						 GjPresenceShow show, 
						 const gchar *status)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeStore *store = NULL;

  GList *iter_list = NULL;

  gboolean ignore_presence = FALSE;

  GdkPixbuf *pb = NULL;
  gchar *text = NULL;
  gint i = 0;

  GjJID j = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  j = gj_roster_item_get_jid(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  if(gj_roster_item_get_type(ri) != GjRosterItemTypeContact && 
     gj_roster_item_get_type(ri) != GjRosterItemTypeAgent)
    {
      g_message("%s: roster item type:%d is not GjRosterItemTypeContact, GjRosterItemTypeAgent, ignoring this event", __FUNCTION__, gj_roster_item_get_type(ri));
      return FALSE;
    }

  /* this statement should only be true when removing an agent.
     for some reason, the presence of the agent is sent AFTER 
     you request to remove it and your subscription from the roster */
  if(type == GjPresenceTypeUnAvailable && status != NULL && strcmp(status,"Unregistered") == 0 &&
     gj_roster_item_get_subscription(ri) == GjRosterItemSubscriptionNone &&
     gj_roster_item_get_is_permanent(ri) == FALSE)
    {
      ignore_presence = TRUE;
    }
      
  /* the event viewer handles presence errors */
  if(type == GjPresenceTypeError)
    ignore_presence = TRUE;

  /* throw this call out if conditions are met */
  if(ignore_presence == TRUE)
    {
      g_message("%s: presence type was error or presence was for a roster item just removed, not adding to the roster, ignoring this event", __FUNCTION__);
      return TRUE;
    }

  /*
   * do we keep it or remove it? 
   */
  if(type == GjPresenceTypeUnAvailable)
    {
      gint remove = 0;

      if(gj_jid_get_type(j) == GjJIDTypeAgent && gj_config_get_mpf_hide_agents() == TRUE)
	remove += 1;
      
      if(gj_roster_item_get_is_permanent(ri) == TRUE && gj_roster_item_get_subscription(ri) != GjRosterItemSubscriptionNone)
	remove += 1;

      if(gj_roster_item_get_is_another_resource(ri) == TRUE)
	remove += 1;
    
      /* if hide offline contacts remove from list and return */
      /* if show offline contacts and show offline group, remove and add again and return */
      /* if show offline contacts and hide offline group, remove from list and continue */

      /* agents remain constantly there UNLESS they are hidden 
	 in preferences - but then if that was the case, we wouldnt
	 have to worry about it being removed anyway. */
      if(gj_jid_get_type(j) == GjJIDTypeContact && remove > 0)
	{
	  g_message("%s: removing jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
	  gj_gtk_roster_model_remove(ri);

	  if(gj_roster_item_get_is_another_resource(ri) == TRUE)
	    {
	      g_message("%s: removed resource from the roster... no further action needed", __FUNCTION__);
	      return TRUE;
	    }
	  else if(gj_config_get_mpf_hide_offline_contacts() == FALSE && 
		  gj_config_get_mpf_hide_offline_group() == FALSE)
	    {
	      g_message("%s: re-adding jid:'%s', into offline group", __FUNCTION__, gj_jid_get_full(j));
	      gj_gtk_roster_model_add(window, ri);
	      return TRUE;
	    }
	  else if(gj_config_get_mpf_hide_offline_group() == TRUE)
	    {
	      /* continue */
	      g_message("%s: continuing to set presence for jid:'%s'", __FUNCTION__, gj_jid_get_full(j));
	    }
	  else
	    return TRUE;
	}
    }
  else
    {
      /* if hide offline contacts do nothing, contact will be added */
      /* if show offline contacts and show offline group remove contact so it will be re-added */
      /* if show offline contacts and hide offline group do nothing, contact will be updated */

      if(gj_config_get_mpf_hide_offline_contacts() == FALSE && 
	 gj_config_get_mpf_hide_offline_group() == FALSE &&
	 gj_jid_get_type(j) == GjJIDTypeContact)
	{
	  GjPresence last_pres = NULL;
	  GjPresenceType last_type = GjPresenceTypeEnd;

	  last_pres = gj_roster_item_get_last_presence(ri);
	  last_type = gj_presence_get_type(last_pres);

	  if((last_type == GjPresenceTypeUnAvailable && type == GjPresenceTypeAvailable) || 
	     (last_type == GjPresenceTypeSubscribed))
	    {
	      g_message("%s: removing jid:'%s' from offline group, item will be re-added.", 
			__FUNCTION__, gj_jid_get_full(j));
	      gj_gtk_roster_model_remove(ri);
	    }
	}
    }
  
  if(gj_gtk_roster_model_find_jid(view, gj_roster_item_get_jid(ri), &iter_list) == FALSE)
    {
      g_message("%s: jid:'%s' is not on the roster, adding...", __FUNCTION__, gj_jid_get_full(j));
      if(gj_gtk_roster_model_add(window, ri) == FALSE)
	return FALSE;

      if(gj_gtk_roster_model_find_jid(view, gj_roster_item_get_jid(ri), &iter_list) == FALSE)
	return FALSE;
    }

  /* get new text/image */
  if(gj_config_get_mpf_hide_contact_presence_details() == FALSE &&
     gj_jid_get_type(j) == GjJIDTypeContact)
    {
      gchar *s1 = NULL;
      gchar *s2 = NULL;
      gchar *s3 = NULL;
      
      if(gj_roster_item_get_is_another_resource(ri) == TRUE)
	s1 = g_strdup(gj_jid_get_resource(j));
      else
	s1 = g_strdup(gj_roster_item_get_name(ri));

      if(s1 != NULL)
	s2 = g_markup_escape_text(s1, -1);
      if(status != NULL)
	s3 = g_markup_escape_text(status, -1);
      if(s3 == NULL)
	s3 = gj_gtk_presence_to_status_text_new(type, show);
      
      text = g_strdup_printf(ROSTER_CONTACT_PRESENCE_DETAILS_FORMAT, s2, s3==NULL?"":s3);
      
      if(s1 != NULL) g_free(s1);
      if(s2 != NULL) g_free(s2);
      if(s3 != NULL) g_free(s3);
    }

  if(gj_roster_item_get_subscription(ri) != GjRosterItemSubscriptionNone)
    pb = gj_gtk_presence_to_pixbuf_new(type, show);
  else
    pb = gj_gtk_presence_to_pixbuf_new(GjPresenceTypeNone, 0);

  /* NOTE: at this point, we need to continue searching
     the roster for other copies of the same JID for different
     groups. */

  for(i=0; i<g_list_length(iter_list); i++)
    {
      GtkTreeIter *iter = NULL;
      GtkTreePath *path = NULL;

      gchar *name = NULL;
      
      iter = (GtkTreeIter*) g_list_nth_data(iter_list, i);

      gtk_tree_model_get(model, iter, COL_ROSTER_NAME, &name, -1);
      gtk_tree_store_set(store, iter,
			 COL_ROSTER_IMAGE, pb,
			 COL_ROSTER_IMAGE_FLASH, NULL,
			 COL_ROSTER_IMAGE_FLASH_ON, FALSE,
			 COL_ROSTER_TEXT, text == NULL ? name : text,
			 -1);
	  
      path = gtk_tree_model_get_path(model, iter);
      gtk_tree_model_row_changed(model, path, iter);
      if(path != NULL) gtk_tree_path_free(path);
      
      if(name != NULL) g_free(name);
    }

  g_list_foreach(iter_list, gj_gtk_roster_model_iter_list_destroy_cb, NULL);
  g_list_free(iter_list);

  /* clean up */
  if(text != NULL) g_free(text);
  if(pb != NULL) g_object_unref(G_OBJECT(pb));

  return TRUE;
}

gboolean gj_gtk_roster_model_set_message(GjRosterWindow *window, GjRosterItem ri, GjMessageType type)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeStore *store = NULL;

  GList *iter_list = NULL;
  gint i = 0;

  GdkPixbuf *pb = NULL;

  GjJID j = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  j = gj_roster_item_get_jid(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  /* get image */
  pb = gj_gtk_message_to_pixbuf_new(type);

  if(gj_gtk_roster_model_find_jid(view, gj_roster_item_get_jid(ri), &iter_list) == FALSE)
    {
      g_message("%s: could not find jid:'%s' on roster", __FUNCTION__, gj_jid_get_full(j));
      if(gj_gtk_roster_model_add(window, ri) == FALSE)
	return FALSE;
    }

  for(i=0; i<g_list_length(iter_list); i++)
    {
      GtkTreeIter *iter = NULL;
      GtkTreePath *path = NULL;
      
      iter = (GtkTreeIter*) g_list_nth_data(iter_list, i);

      gtk_tree_store_set(store, iter,
			 COL_ROSTER_IMAGE_FLASH, pb,
			 COL_ROSTER_IMAGE_FLASH_ON, FALSE,
			 -1);
      
      path = gtk_tree_model_get_path(model, iter);
      gtk_tree_model_row_changed(model, path, iter);
      
      if(path != NULL) gtk_tree_path_free(path);
    }

  g_list_foreach(iter_list, gj_gtk_roster_model_iter_list_destroy_cb, NULL);
  g_list_free(iter_list);

  if(pb != NULL) g_object_unref(G_OBJECT(pb));

  return TRUE;
}

static gboolean gj_gtk_roster_set_my_presence(GjRosterWindow *window,
					      GjPresenceType type, 
					      GjPresenceShow show, 
					      const gchar *status, 
					      const gint8 priority,
					      gboolean send_it)
{
  if(!window)
    return FALSE;

  gtk_image_set_from_stock(GTK_IMAGE(window->image_presence), 
			   gj_gtk_presence_to_stock_id(type, show), 
			   GTK_ICON_SIZE_BUTTON);

  if(status != NULL)
    {
      gchar *s1 = NULL;
      gchar *s2 = NULL;
      gchar *s3 = NULL;
      gchar *text = NULL;

      s1 = gj_gtk_presence_to_show_text_new(type, show);

      if(s1 != NULL)
	s2 = g_markup_escape_text(s1, -1);
      if(status != NULL)
	s3 = g_markup_escape_text(status, -1);
      if(s3 == NULL)
	s3 = gj_gtk_presence_to_status_text_new(type, show);
      
      text = g_strdup_printf(ROSTER_CONTACT_PRESENCE_DETAILS_FORMAT, s2, s3==NULL?"":s3);
      gtk_label_set_markup(GTK_LABEL(window->label_presence), text);
      
      if(s1 != NULL) g_free(s1);
      if(s2 != NULL) g_free(s2);
      if(s3 != NULL) g_free(s3);
      if(text != NULL) g_free(text);
    }
			     
  /* send the status */
  if(send_it)
    gj_presence_send(type, show, status, priority);

  return TRUE;
}

gboolean gj_gtk_roster_set_my_presence_and_save(GjPresenceType type, 
						GjPresenceShow show, 
						const gchar *status, 
						gint8 priority,
						gboolean send_it)
{
  gchar *status_dup = NULL;

  if(gj_gtk_roster_set_my_presence(current_window, type, show, status, priority, send_it) == FALSE)
    {
      g_warning("%s: not saving config, setting presence failed", __FUNCTION__);
      return FALSE;
    }

  status_dup = g_strdup(status);

  /* remember details */
  gj_me_set_presence_type(type);
  gj_me_set_presence_show(show);
  gj_me_set_presence_status(status_dup);  
  gj_me_set_presence_priority(priority);  
  
  return TRUE;
}

static void gj_gtk_roster_autoaway_cb(guint msec, GjRosterWindow *window)
{
  if(!window)
    return;

  /* if not connected - do nothing */
  if(gj_me_get_is_connected() == FALSE)
    return;

  if(msec > (60 * 1000))
    {
      guint x = msec;
      guint seconds = 0;
      guint minutes = 0;
      guint hours = 0;
      guint days = 0;

      gchar *text = NULL;
      
      /* convert ms to seconds */
      x /= 1000;

      /* calculate from there */
      seconds = x % 60;
      x /= 60;
      minutes = x % 60;
      x /= 60;
      hours = x % 24;
      x /= 24;
      days = x;

      /* alert the user */
      if(days==0)
	{
	  /* To translators: hours, minutes, seconds */
	  text = g_strdup_printf(_("AutoAway %2.2d:%2.2d:%2.2d"),
				 hours, minutes, seconds);
	}
      else
	{
	  /* To translators: hours, minutes, seconds, days, "day/days" */
	  text = g_strdup_printf(_("AutoAway %2.2d:%2.2d:%2.2d (%d %s)"), 
				 hours, minutes, seconds, days, days==1?_("day"):_("days"));
	}

      /*       gj_gtk_roster_set_status(text); */
      g_free(text);

      /* set to autoaway mode */
      gj_me_set_is_auto_away(TRUE);
	  
      /* check the options */
      if(gj_config_get_mpf_use_auto_away() == TRUE &&
	 (msec/1000) >= (gj_config_get_mpf_auto_away_after() * 60) && 
	 (msec/1000) < (gj_config_get_mpf_auto_xa_after() * 60) && 
	 gj_me_get_auto_away_presence_show() != GjPresenceShowAway && 
	 gj_me_get_auto_away_presence_show() != GjPresenceShowXA)
	{
	  g_message("%s: setting presence to automatically be away, idle time: %d seconds, configured auto away time:%d minutes", __FUNCTION__, msec / 1000, gj_config_get_mpf_auto_away_after());

	  /* set to away */
	  gj_gtk_roster_set_my_presence(window, 
					GjPresenceTypeAvailable,
					GjPresenceShowAway,
					gj_config_get_mpf_auto_away_status(),
					gj_config_get_cs_priority(),
					TRUE);

	  /* set state */
	  gj_me_set_auto_away_presence_show(GjPresenceShowAway);
	} 
      else if(gj_config_get_mpf_use_auto_away() == TRUE && 
	      (msec/1000) >= (gj_config_get_mpf_auto_xa_after() * 60) && 
	      gj_me_get_auto_away_presence_show() != GjPresenceShowXA)
	{
	  g_message("%s: setting presence to automatically be xa, idle time: %d seconds, configured auto xa time:%d minutes", __FUNCTION__, msec / 1000, gj_config_get_mpf_auto_xa_after());

	  /* set to xa */
	  gj_gtk_roster_set_my_presence(window,
					GjPresenceTypeAvailable,
					GjPresenceShowXA,
					gj_config_get_mpf_auto_xa_status(),
					gj_config_get_cs_priority(),
					TRUE);

	  /* set state */
	  gj_me_set_auto_away_presence_show(GjPresenceShowXA);
	}
    }
  else
    {
      if(gj_me_get_is_auto_away() == TRUE)
	{
	  GjPresenceShow previous_presence_show = 0;
	  GjPresenceType previous_presence_type = 0;
	  const gchar *previous_presence_status = NULL;

	  g_message("%s: restoring presence to original state.", __FUNCTION__);

	  /* change presence */
	  /* what was it before?? - check the store? */
	  previous_presence_show = gj_me_get_presence_show();
	  previous_presence_type = gj_me_get_presence_type();
	  previous_presence_status = gj_me_get_presence_status();

	  /* set to normal */
	  gj_gtk_roster_set_my_presence(window,
					previous_presence_type,
					previous_presence_show,
					previous_presence_status,
					gj_config_get_cs_priority(),
					TRUE);
 
	  /* set to autoaway mode */
	  gj_me_set_is_auto_away(FALSE);
	  gj_me_set_auto_away_presence_show(previous_presence_show);
	}
    }
}

/* roster contact functions */
static gboolean gj_gtk_roster_contact_read(GjRosterWindow *window)
{
  GtkTreeView *view = NULL;
  GtkTreeSelection *selection = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;

  GjRosterItem ri = NULL;

  gint id = 0;

  if(!window)
    return FALSE;

  view = GTK_TREE_VIEW(window->treeview);
  model = gtk_tree_view_get_model(view);
  selection = gtk_tree_view_get_selection(view);

  if(gtk_tree_selection_get_selected(selection, &model, &iter) == FALSE)
    return FALSE;
  
  gtk_tree_model_get(model, &iter, COL_ROSTER_ID, &id, -1);      
  ri = gj_rosters_find_item_by_id(id);
  
  gj_gtk_roster_handle_next_event(window, ri);
  gj_gtk_roster_handle_next_event_display(ri);
  
  return TRUE;
}

static gboolean gj_gtk_roster_contact_new_message(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  gj_gtk_message_send_new(ri); 
  return TRUE;
}

static gboolean gj_gtk_roster_contact_new_chat(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;

  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  gj_gtk_chat_send_new(ri);
  return TRUE;
}

static gboolean gj_gtk_roster_contact_headlines(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;

  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  if(gj_gtk_hw_load_by_ri(ri) == FALSE)
    {
      gj_gtk_msg_new(FALSE,
		     GJ_GTK_MSG_BUTTON_OK | GJ_GTK_MSG_ICON_EXCLAIMATION,
		     (MsgCb)NULL,
		     NULL,
		     _("No Headlines."),
		     _("There are no headlines for this user."),
		     NULL);
    }
  
  return TRUE;
}

static gboolean gj_gtk_roster_contact_history(GjRosterWindow *window)
{
  GjHistory hist = NULL;
  GjRosterItem ri = NULL;

  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  if((hist = gj_history_find(gj_roster_item_get_id(ri))) == NULL)
    {
      g_warning("%s: could not find history based on roster item id:%d", __FUNCTION__, gj_roster_item_get_id(ri));
      return FALSE;
    }
  
  gj_gtk_open_url(gj_history_get_url(hist));
  return TRUE;
}

static gboolean gj_gtk_roster_contact_information(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  gj_gtk_ui_load(ri);
  return TRUE;
}

static gboolean gj_gtk_roster_contact_sub_request(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  GjJID j = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  j = gj_roster_item_get_jid(ri);
  gj_presence_send_to(j, GjPresenceTypeSubscribe, GjPresenceShowEnd, NULL, 0);

  return TRUE;
}

static gboolean gj_gtk_roster_contact_sub_cancel(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  GjJID j = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  j = gj_roster_item_get_jid(ri);
  gj_presence_send_to(j, GjPresenceTypeUnSubscribed, GjPresenceShowEnd, NULL, 0);

  return TRUE;
}

static gboolean gj_gtk_roster_contact_edit_groups(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  gj_gtk_eg_load(ri);
  return TRUE;
}

static gboolean gj_gtk_roster_contact_rename(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  gj_gtk_rc_load(ri);
  return TRUE;
}

static gboolean gj_gtk_roster_contact_add(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  gj_gtk_ac_load(roster, ri);
  return TRUE;
}

static gboolean gj_gtk_roster_contact_remove(GjRosterWindow *window)
{
  GtkWidget *dialog = NULL;
  GtkWidget *checkbox = NULL;

  GjRosterItem ri = NULL;
  GjJID j = NULL;
  
  GList *list = NULL;

  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  j = gj_roster_item_get_jid(ri);

  if(gj_jid_get_type(j) == GjJIDTypeAgent)
    list = gj_roster_get_contacts_by_agent(roster, gj_jid_get_without_resource(j)); 

  if(list == NULL)
    {
      dialog = gj_gtk_hig_dialog_new(GTK_WINDOW(window->window),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_QUESTION,
				     GTK_BUTTONS_OK_CANCEL,
				     _("You are about to remove a contact."),
				     _("Proceed to remove contact \"%s\"?"), 
				     gj_jid_get_without_resource(j));
    }
  else
    {
      gchar *text = NULL;

      if(g_list_length(list) == 1)
	text = g_strdup_printf(_("This agent is responsible for 1 contact!\nProceed to remove agent \"%s\"?"),
			       gj_jid_get_without_resource(j));
      else
	text = g_strdup_printf(_("This agent is responsible for %d contacts!\nProceed to remove agent \"%s\"?"), 
			       g_list_length(list), gj_jid_get_without_resource(j));
				     
      dialog = gj_gtk_hig_dialog_new(GTK_WINDOW(window->window),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_MESSAGE_QUESTION,
				     GTK_BUTTONS_OK_CANCEL,
				     _("You are about to remove an agent."),
				     text);

      checkbox = gtk_check_button_new_with_label(_("Remove all contacts this agent is responsible for?"));
      gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), checkbox, FALSE, FALSE, 6);
      
      gtk_widget_show_all(dialog);

      if(text) g_free(text);
    }
  
  /* run dialog */
  if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) 
    {
      GjRosterItem this_ri = NULL;
      GjJID this_j = NULL;
      gint i = 0;
      
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbox))) 
	{
	  for(i=0; i<g_list_length(list); i++)
	    {
	      this_ri = (GjRosterItem) g_list_nth_data(list, i);
	      if(this_ri == NULL) continue;
	      this_j = gj_roster_item_get_jid(this_ri);

	      gj_iq_request_roster_remove_contact(gj_jid_get_without_resource(this_j), 
						  gj_gtk_roster_contact_remove_cb,
						  NULL);

	    }
	}

      gj_iq_request_roster_remove_contact(gj_jid_get_without_resource(j), 
					  gj_gtk_roster_contact_remove_cb,
					  NULL);
    }

  gtk_widget_destroy(dialog);
 
  /* clean up */
  if(list) g_list_free(list);

  return TRUE;
}

static gboolean gj_gtk_roster_agent_log_in(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  GjJID j = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  j = gj_roster_item_get_jid(ri);
  gj_presence_send_to(j, GjPresenceTypeAvailable, GjPresenceShowNormal, NULL, 0);

  return TRUE;
}

static gboolean gj_gtk_roster_agent_log_out(GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  GjJID j = NULL;
  
  if(!window)
    return FALSE;

  if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
    return FALSE;

  j = gj_roster_item_get_jid(ri);
  gj_presence_send_to(j, GjPresenceTypeUnAvailable, GjPresenceShowNormal, NULL, 0);

  return TRUE;
}

static gboolean gj_gtk_roster_group_message(GjRosterWindow *window)
{
  gboolean not_group = TRUE;
  gchar *group = NULL;
  
  if(!window)
    return FALSE;

  if(gj_gtk_roster_model_selection_get_is_group(window, &not_group, &group) == FALSE)
    return FALSE;
  
  gj_gtk_gm_load(roster, group);

  if(group) g_free(group);
  return TRUE;
}

static gboolean gj_gtk_roster_group_rename(GjRosterWindow *window)
{
  gboolean not_group = TRUE;
  gchar *group = NULL;
  
  if(!window)
    return FALSE;

  if(gj_gtk_roster_model_selection_get_is_group(window, &not_group, &group) == FALSE)
    return FALSE;
  
  gj_gtk_rg_load(roster, group);

  if(group) g_free(group);
  return TRUE;
}

static gboolean gj_gtk_roster_group_remove(GjRosterWindow *window)
{
  GtkWidget *dialog = NULL;
  GtkWidget *checkbox = NULL;

  gboolean not_group = TRUE;
  gchar *group = NULL;
  
  GList *list = NULL;
  gchar *text = NULL;

  if(!window)
    return FALSE;

  if(gj_gtk_roster_model_selection_get_is_group(window, &not_group, &group) == FALSE)
    return FALSE;
  
  if(group == NULL || g_utf8_strlen(group, -1) < 1)
    {
      g_warning("%s: group was NULL or len was < 1", __FUNCTION__);
      return FALSE;
    }
  
  dialog = gj_gtk_hig_dialog_new(GTK_WINDOW(window->window),
				 GTK_DIALOG_DESTROY_WITH_PARENT,
				 GTK_MESSAGE_QUESTION,
				 GTK_BUTTONS_OK_CANCEL,
				 _("You are about to remove a group."),
				 _("Proceed to remove group \"%s\"?"), 
				 group);


  list = gj_roster_get_contacts_by_group(roster, group);
  if(g_list_length(list) == 1)
    text = g_strdup_printf(_("Remove the contact in this group from roster?"));
  else
    text = g_strdup_printf(_("Remove all %d contacts in this group from roster?"), g_list_length(list));
  
  checkbox = gtk_check_button_new_with_label(text);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), checkbox, FALSE, FALSE, 6);

  gtk_widget_show_all(dialog);
  
  if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) 
    {
      GjRosterItem ri = NULL;
      GjJID j = NULL;
      gint i = 0;
      
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbox))) 
	{
	  for(i=0; i<g_list_length(list); i++)
	    {
	      ri = (GjRosterItem) g_list_nth_data(list, i);
	      if(ri == NULL) continue;
	      j = gj_roster_item_get_jid(ri);

	      gj_iq_request_roster_remove_contact(gj_jid_get_without_resource(j), 
						  gj_gtk_roster_contact_remove_cb,
						  NULL);

	      /* remove roster item ? */
	    }
	}
      else
	{
	  for(i=0; i<g_list_length(list); i++)
	    {
	      ri = (GjRosterItem) g_list_nth_data(list, i);
	      if(ri == NULL) continue;

	      j = gj_roster_item_get_jid(ri);
	      gj_roster_item_groups_del(ri, group);

	      /* using the list, modify all the users groups to include the new group */
	      gj_iq_request_user_change(gj_jid_get_full(j), 
					gj_roster_item_get_name(ri), 
					gj_roster_item_get_groups(ri),
					gj_gtk_roster_user_changed_cb,
					NULL);
	    }
	}
    }

  /* clean up */
  if(text) g_free(text);
  if(list) g_list_free(list);
  if(group) g_free(group);
  
  gtk_widget_destroy(dialog);
  
  return TRUE;
}

static gboolean gj_gtk_roster_unregister(GjRosterWindow *window)
{
  GtkWidget *dialog = NULL;

  if(!window)
    return FALSE;

  dialog = gj_gtk_hig_dialog_new(GTK_WINDOW(window->window),
				 GTK_DIALOG_DESTROY_WITH_PARENT,
				 GTK_MESSAGE_QUESTION,
				 GTK_BUTTONS_OK_CANCEL,
				 _("Unregister this account?"),
				 _("If you continue you will be disconnected from the server immediately."));

  /* run dialog */
  if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) 
    {
      gj_iq_request_unregister(gj_config_get_cs_server(),
			       gj_gtk_roster_unregister_cb,
			       NULL);    
    }

  gtk_widget_destroy(dialog);
 
  return TRUE;
}

static gboolean gj_gtk_roster_unsubscribed(GjRosterWindow *window, GjPresence pres)
{
  GtkWidget *dialog = NULL;
  GtkWidget *checkbox = NULL;
  GjJID j = NULL;

  if(!window)
    return FALSE;

  if(pres == NULL)
    {
      g_warning("%s: presence was NULL", __FUNCTION__);
      return FALSE;
    }

  j = gj_presence_get_from_jid(pres);

  dialog = gj_gtk_hig_dialog_new(GTK_WINDOW(window->window),
				 GTK_DIALOG_DESTROY_WITH_PARENT,
				 GTK_MESSAGE_QUESTION,
				 GTK_BUTTONS_OK,
				 _("Subscription Canceled"),
				 _("You have been denied status information from \"%s\"."),
				 gj_jid_get_full(j));

  checkbox = gtk_check_button_new_with_label(_("Do you want to remove them from your roster?"));
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), checkbox, FALSE, FALSE, 6);
  
  gtk_widget_show_all(dialog);

  /* run dialog */
  gtk_dialog_run(GTK_DIALOG(dialog));

  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbox))) 
    {
      g_message("%s: removing unsubscribed contact:'%s'", __FUNCTION__, gj_jid_get_full(j));
      gj_iq_request_roster_remove_contact(gj_jid_get_without_resource(j), 
					  gj_gtk_roster_contact_remove_cb,
					  NULL);
    }

  gtk_widget_destroy(dialog);
 
  return TRUE;
}

/*
 * EVENT HANDLING
 */
static gboolean gj_gtk_roster_handle_next_event(GjRosterWindow *window, GjRosterItem ri)
{
  GjEvent ev = NULL;
  GjEventType type = -1;
  
  gint id = -1;
  gint count = 0;
  
  /* if no events waiting, do default action */
  if((count = gj_event_count(ri)) < 1)
    {
      g_message("%s: %d events waiting, therefore starting new chat...", __FUNCTION__, count);

      /* we can start a chat... */
      if(gj_config_get_mpf_double_click_for_chat() == TRUE)
	gj_gtk_chat_send_new(ri);
      else
	gj_gtk_message_send_new(ri);

      return TRUE;
    }

  if((id = gj_event_next(ri)) < 0)
    {
      g_warning("%s: failed to get eventID from roster item", __FUNCTION__);
      return FALSE;
    }
  
  if((ev = gj_event_find_by_id(id)) == NULL)
    {
      g_warning("%s: failed to find event by ID:%d", __FUNCTION__, id);
      return FALSE;
    }

  /* get event type */
  type = gj_event_get_type(ev);

  g_message("%s: handling eventID:%d, type:%d", __FUNCTION__, id, type);
  
  switch(type)
    {
    case GjEventTypeMessage:
      {
	GjMessage m = (GjMessage) gj_event_get_data(ev);
	
	if(gj_message_get_type(m) == GjMessageTypeNormal)
	  {
	    /* DONT free the event, that is done when it has been handled */
	    if(!gj_gtk_message_post_by_event(ev, TRUE))
	      break;
	  }
	else if(gj_message_get_type(m) == GjMessageTypeChat)
	  {
	    if(gj_gtk_chat_post_by_event(ev, TRUE) == TRUE)
	      gj_event_free(ev);
	    else
	      break;

	    /* look for futher events of type 'chat' in queue */
	    while((ev = gj_event_find_next_by_type_and_message_type(ri, type, GjMessageTypeChat)) != NULL)
	      {
		m = (GjMessage) gj_event_get_data(ev);
		id = gj_event_get_id(ev);

		if(m == NULL)
		  {
		    g_warning("%s: msg data was NULL, not handling...", __FUNCTION__);

		    /* free event - no use to me now */
		    gj_event_free(ev);
		    continue;
		  }
		
		g_message("%s: handling next event id:%d, type:%d->'%s'", 
			  __FUNCTION__, id, type, gj_translate_event_type(type));

		if(gj_gtk_chat_post_by_event(ev, TRUE) == TRUE)
		  gj_event_free(ev);
		else
		  break;
	      }
	  }
	else if(gj_message_get_type(m) == GjMessageTypeHeadline)
	  {
	    HeadlineWindow hw = NULL;
	    
	    /* notify of headline */
	    if((hw = gj_gtk_hw_notification_new(id, m)) == NULL)
	      {
		g_warning("%s: new notification for eventID:%d returned NULL chat window", __FUNCTION__, id);
		return FALSE;

	      }

	    /* free event */
	    gj_event_free(ev);
	    
	    /* look for futher events of type 'headline' in queue */
	    while((ev = gj_event_find_next_by_type_and_message_type(ri,type,GjMessageTypeHeadline)) != NULL)
	      {
		m = (GjMessage) gj_event_get_data(ev);
		id = gj_event_get_id(ev);

		if(m == NULL)
		  {
		    g_warning("%s: m data was NULL, not handling...", __FUNCTION__);

		    /* free event - no use to me now */
		    gj_event_free(ev);
		    continue;
		  }
		
		g_message("%s: handling next event of type:%d, id:%d", __FUNCTION__, type, id);
		gj_gtk_hw_notification_new(id, m);
	
		/* free event */
		gj_event_free(ev);
	      }	    
	  }
	else
	  {
	    g_message("%s: unhandled message type:%d", __FUNCTION__, gj_message_get_type(m));

	    /* if we dont handle certain types of messages (e.g. headlines, groupchats...), free it. */
	    gj_event_free(ev);
	  }
      }
      break;

    default:
      g_warning("%s: unhandled event type:%d->'%s', freeing...", __FUNCTION__, type, gj_translate_event_type(type));
      
      /* if we dont know about it, free it. */
      gj_event_free(ev);

      break;
    }

  gj_event_print_all(ri);
  
  return TRUE;
}

gboolean gj_gtk_roster_handle_next_event_display(GjRosterItem ri)
{
  GjEvent ev = NULL;
  GjEventType type = -1;
  GjRosterWindow *window = NULL;

  gint id = 0;

  if(!(window = current_window))
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }
  
  id = gj_event_next(ri);
  if(id != -1) ev = gj_event_find_by_id(id);
  if(ev != NULL) type = gj_event_get_type(ev);
 
  switch(type)
    {
    case GjEventTypeMessage:
      {
	GjMessage m = (GjMessage) gj_event_get_data(ev);
	
	/* start flashing */
	if(gj_message_get_type(m) != GjMessageTypeError &&
	   gj_message_get_type(m) != GjMessageTypeXEvent)
	  {
	    /* returns true if added false if not */
	    if(gj_gtk_roster_flash_timeout_add(window, ri) == TRUE)
	      gj_gtk_roster_model_set_message(window, ri, gj_message_get_type(m));
	  }
	else
	  {
	    gj_gtk_roster_model_set_message(window, ri, gj_message_get_type(m));
	  }
      }
      break;

    default:
      {
	GjPresence pres = NULL;

	/* stop flashing */
	gj_gtk_roster_flash_timeout_remove(window, ri);
	
	if((pres = gj_roster_item_get_presence(ri)) == NULL)
	  {
	    g_warning("%s: could not get presence from roster item with id:%d", 
		      __FUNCTION__, gj_roster_item_get_id(ri)); 
	    return FALSE;
	  }
	
	/* no events pending, set icon to status */
	gj_gtk_roster_model_set_presence(window, 
					 ri, 
					 gj_presence_get_type(pres), 
					 gj_presence_get_show(pres), 
					 gj_presence_get_status(pres)); 
      }
      break;
    }

  /* check for ANY events... if so - flash notification icon */
  if(gj_event_count_all() > 0)
    gj_gtk_docklet_set_active(TRUE);
  else
    gj_gtk_docklet_set_active(FALSE);

  return TRUE;
}

static gboolean gj_gtk_roster_flash_timeout_add(GjRosterWindow *window, GjRosterItem ri)
{
  gint i = 0;
  gboolean found = FALSE;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  for(i=0; i<g_list_length(window->flash_list) && !found; i++)
    {
      gint ri_id = 0;
      gpointer data = NULL;

      data = (gpointer) g_list_nth_data(window->flash_list, i);
      if(data == NULL) continue;

      ri_id = GPOINTER_TO_INT(data);
      if(ri_id == gj_roster_item_get_id(ri)) found = TRUE;
    }
   
  if(!found)
    {
      /* add to list */
      window->flash_list = g_list_append(window->flash_list, GINT_TO_POINTER(gj_roster_item_get_id(ri)));
      return TRUE;
    }
  
  return FALSE;
}

static gboolean gj_gtk_roster_flash_timeout_remove(GjRosterWindow *window, GjRosterItem ri)
{ 
  gint i = 0;

  /* NOTE: this should REALLY be a HASH TABLE not a LIST */

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }
 
  if(window->flash_list == NULL)
    return TRUE;

  for(i=0; i<g_list_length(window->flash_list); i++)
    {
      gint ri_id = 0;
      gpointer data = NULL;

      data = (gpointer) g_list_nth_data(window->flash_list, i);
      if(data == NULL) continue;

      ri_id = GPOINTER_TO_INT(data);
      if(ri_id == gj_roster_item_get_id(ri)) 
	window->flash_list = g_list_remove(window->flash_list, data);
    }

  return TRUE;
}

static gboolean gj_gtk_roster_flash_timeout_update_roster(GjRosterWindow *window, GjRosterItem ri)
{
  GtkTreeView *view = NULL;
  GtkTreeModel *model = NULL;
  GtkTreeStore *store = NULL;

  GList *iter_list = NULL;
  gint i = 0;

  GjJID j = NULL;
  GjPresence pres = NULL;

  if(!window)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }
  
  j = gj_roster_item_get_jid(ri);
  pres = gj_roster_item_get_presence(ri);

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));
  model = gtk_tree_view_get_model(view);

  if(gj_gtk_roster_model_find_jid(view, gj_roster_item_get_jid(ri), &iter_list) == FALSE)
    {
      g_message("%s: could not find jid:'%s' on roster", __FUNCTION__, gj_jid_get_full(j));
      if(gj_gtk_roster_model_add(window, ri) == FALSE)
	return FALSE;
    }

  for(i=0; i<g_list_length(iter_list); i++)
    {
      GtkTreeIter *iter = NULL;
      GtkTreePath *path = NULL;
      
      GdkPixbuf *pb_flash = NULL;

      gboolean flash = FALSE;
      
      iter = (GtkTreeIter*) g_list_nth_data(iter_list, i);
      
      gtk_tree_model_get(model, iter,
			 COL_ROSTER_IMAGE_FLASH, &pb_flash,
			 COL_ROSTER_IMAGE_FLASH_ON, &flash,
			 -1);
      
      /* FALSE = draw presence, TRUE = draw image */
      if(!flash)
	{
	  GdkPixbuf *pb_presence = NULL;

	  if(gj_roster_item_get_subscription(ri) != GjRosterItemSubscriptionNone)
	    pb_presence = gj_gtk_presence_to_pixbuf_new(gj_presence_get_type(pres), gj_presence_get_show(pres));
	  else
	    pb_presence = gj_gtk_presence_to_pixbuf_new(GjPresenceTypeNone, 0);
	  
	  gtk_tree_store_set(store, iter,
			     COL_ROSTER_IMAGE, pb_presence,
			     COL_ROSTER_IMAGE_FLASH_ON, TRUE,
			     -1);

	  if(pb_presence != NULL) g_object_unref(G_OBJECT(pb_presence));
	}
      else
	{
	  gtk_tree_store_set(store, iter,
			     COL_ROSTER_IMAGE, pb_flash,
			     COL_ROSTER_IMAGE_FLASH_ON, FALSE,
			     -1);
	}
      
      path = gtk_tree_model_get_path(model, iter);
      gtk_tree_model_row_changed(model, path, iter);
      
      /* clean up */
      if(path != NULL) gtk_tree_path_free(path);
      if(pb_flash != NULL) g_object_unref(G_OBJECT(pb_flash));
    }
  
  g_list_foreach(iter_list, gj_gtk_roster_model_iter_list_destroy_cb, NULL);
  g_list_free(iter_list);

  /* return FALSE to stop timeout */
  return TRUE;
}

static gboolean gj_gtk_roster_flash_timeout_cb(GjRosterWindow *window)
{
  gint i = 0;

  if(!window)
    return FALSE;

  if(window->flash_list == NULL)
    return TRUE;

  for(i=0; i<g_list_length(window->flash_list); i++)
    {
      gint ri_id = 0;
      gpointer data = NULL;

      GjRosterItem ri = NULL;

      data = (gpointer) g_list_nth_data(window->flash_list, i);
      if(data == NULL) continue;

      ri_id = GPOINTER_TO_INT(data);

      ri = gj_rosters_find_item_by_id(ri_id);
      if(ri == NULL) continue;
      
      if(gj_gtk_roster_flash_timeout_update_roster(window, ri) == FALSE)
	return FALSE;
    }

  return TRUE;
}



/*
 * CALLBACK functions
 */
static void gj_gtk_roster_event_cb(gint event_id, GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  GjJID j = NULL;
  GjEvent ev = NULL;
  GjEventType type = 0;
  
  if(!window)
    return;

  if((ev = gj_event_find_by_id(event_id)) == NULL)
    {
      g_warning("%s: failed to find event by ID:%d", __FUNCTION__, event_id);
      return;
    }
  
  /* event type */
  type = gj_event_get_type(ev);
  g_message("%s: received new event, event_id:%d, type:%d->'%s'", 
	    __FUNCTION__, event_id, type, gj_translate_event_type(type));
  
  /* find roster item */
  if((ri = gj_event_get_roster_item(ev)) == NULL)
    {
      g_warning("%s: no roster item for this event, this is a *SERIOUS* error.", __FUNCTION__);
      return;
    }

  /* get jid */
  j = gj_roster_item_get_jid(ri);
  
  /* notify event viewer */
  gj_gtk_ev_model_add_from_event(ev);

  /* this is for group chats */
  switch(gj_roster_item_get_type(ri))
    {
    case GjRosterItemTypeGroupChat:
      gj_gtk_gcw_post_by_event(ev);
      gj_event_free(ev);
      
      /* we should return here really - 
	 we are not interested in roster events for group chat */ 
      return;
      break;
      
    default:
      
      /* handle event */
      switch(type)
	{
	case GjEventTypePresence:
	  {
	    GjPresence pres = NULL;
	    pres = (GjPresence) gj_event_get_data(ev);

	    /* what if there are MORE than one chat window available in the future? */
	    gj_gtk_chat_presence_update(ri, (GjPresence) gj_event_get_data(ev));
	    
	    /* exceptions are subscriptions */
	    if(gj_presence_get_type(pres) == GjPresenceTypeSubscribe)
	      {
		/* we may not add here... it is up to the user
		   therefore we show the dialog and let the dialog add the user... */
		
		gj_gtk_sr_load(pres);
	      }
	    else if(gj_presence_get_type(pres) == GjPresenceTypeUnSubscribed)
	      {
		gj_gtk_roster_unsubscribed(window, pres);
	      }
	    else
	      gj_roster_item_set_presence(ri, pres);
	    
	    
	    gj_event_free(ev);
	  }
	  break;
	case GjEventTypeMessage:
	  {
	    GjMessage m = (GjMessage) gj_event_get_data(ev);
	    
	    if(gj_message_get_type(m) == GjMessageTypeNormal)
	      {
		gj_gtk_message_post_by_event(ev, FALSE);

		/* should we play sound ? */
		if(gj_config_get_mpf_sound_on_receive_message() == TRUE)
		  gj_gtk_sound_play(gj_config_get_mpf_sound_file_on_receive_message());
	      }
	    else if(gj_message_get_type(m) == GjMessageTypeChat)
	      {
		if(gj_gtk_chat_post_by_event(ev, FALSE) == TRUE)
		  gj_event_free(ev);
		
		/* dont play sound for xevents */
		if(gj_config_get_mpf_sound_on_receive_chat() == TRUE)
		  gj_gtk_sound_play(gj_config_get_mpf_sound_file_on_receive_chat());
	      }
	    else if(gj_message_get_type(m) == GjMessageTypeXEvent)
	      {
		gj_gtk_chat_post_by_event(ev, FALSE);
		gj_event_free(ev);
	      } 
	    else if(gj_message_get_type(m) == GjMessageTypeHeadline)
	      {
		/* should we play sound ? */
		if(gj_config_get_mpf_sound_on_receive_headline() == TRUE)
		  gj_gtk_sound_play(gj_config_get_mpf_sound_file_on_receive_headline());
	      }
	    else if(gj_message_get_type(m) == GjMessageTypeError)
	      {
		/* should we play sound ? */
		if(gj_config_get_mpf_sound_on_receive_error() == TRUE)
		  gj_gtk_sound_play(gj_config_get_mpf_sound_file_on_receive_error());
	      }
	    else
	      {
		/* do we free the event because it is not handled ?? */
		g_message("%s: freeing unhandled event", __FUNCTION__);
		gj_event_free(ev);
	      }
	  }
	  break;
	  
	default:
	  g_warning("%s: unhandled event type:%d->'%s'", __FUNCTION__, type, gj_translate_event_type(type));
	  break;
	}
    }
      
  gj_gtk_roster_handle_next_event_display(ri);
}

static gboolean gj_gtk_roster_generate_mnu_user(GjRosterWindow *window, GjRosterItem ri, GdkEventButton *gdk_event)
{
  GtkWidget *menu = NULL;
  GtkWidget *submenu = NULL;
  GtkWidget *item = NULL;
  GtkWidget *image = NULL;

  GjEvent ev = NULL;

  if(!window)
    return FALSE;

  if(gdk_event == NULL)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  ev = gj_event_find_next_by_type(ri, GjEventTypeMessage);

  menu = gtk_menu_new();
  
  item = gtk_menu_item_new_with_mnemonic(_("Read..."));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_read_cb), window);
  
  if(ev == NULL)
    gtk_widget_set_sensitive(GTK_WIDGET(item), FALSE);

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

  item = gtk_image_menu_item_new_with_mnemonic("New _Message");
  image = gtk_image_new_from_stock(GJ_STOCK_MESSAGE_NORMAL, GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_new_message_cb), window);

  item = gtk_image_menu_item_new_with_mnemonic(_("New _Chat")); 
  image = gtk_image_new_from_stock(GJ_STOCK_MESSAGE_CHAT, GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_new_chat_cb), window);

  item = gtk_image_menu_item_new_with_mnemonic(_("Vie_w Headlines"));
  image = gtk_image_new_from_stock(GJ_STOCK_MESSAGE_HEADLINE, GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_headlines_cb), window);
  
  if(gj_roster_item_get_is_another_resource(ri) == FALSE)
    {
      item = gtk_separator_menu_item_new();
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
      
      item = gtk_image_menu_item_new_with_mnemonic(_("_History"));
      image = gtk_image_new_from_stock(GTK_STOCK_JUSTIFY_FILL, GTK_ICON_SIZE_MENU);
      gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
      g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_history_cb), window);
      
      item = gtk_image_menu_item_new_from_stock(GTK_STOCK_DIALOG_INFO, NULL);
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);  
      g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_information_cb), window);
      
      if(gj_roster_item_get_is_permanent(ri) == TRUE)
	{
	  item = gtk_menu_item_new_with_mnemonic(_("Edit _Groups..."));
	  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
	  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_edit_groups_cb), window);
	  
	  item = gtk_menu_item_new_with_mnemonic(_("Rename..."));
	  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
	  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_rename_cb), window);
	}
      
      item = gtk_separator_menu_item_new();
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
      
      submenu = gtk_menu_new();
      
      item = gtk_menu_item_new_with_mnemonic(_("_Request"));
      gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item); 
      g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_sub_request_cb), window);
	      
      item = gtk_menu_item_new_with_mnemonic(_("_Cancel"));
      gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item); 
      g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_sub_cancel_cb), window);
      
      item = gtk_menu_item_new_with_mnemonic(_("Subscription"));
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
      
      gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);
      
      item = gtk_separator_menu_item_new();
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
      
      if(gj_roster_item_get_is_permanent(ri) == FALSE)
	{
	  item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD, NULL);
	  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
	  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_add_cb), window);
	}

      item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, NULL);
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
      g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_remove_cb), window);
    }
  
  /* show */
  gtk_widget_show_all(GTK_WIDGET(menu));
  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, gdk_event->button, gdk_event->time);

  /* clean up */
  gtk_object_sink(GTK_OBJECT(menu));

  return TRUE;
}

static gboolean gj_gtk_roster_generate_mnu_agent(GjRosterWindow *window, GjRosterItem ri, GdkEventButton *gdk_event)
{
  GtkWidget *menu = NULL;
  GtkWidget *submenu = NULL;
  GtkWidget *item = NULL;
  GtkWidget *image = NULL;

  if(!window)
    return FALSE;

  if(gdk_event == NULL)
    return FALSE;

  if(ri == NULL)
    {
      g_warning("%s: roster item was NULL", __FUNCTION__);
      return FALSE;
    }

  menu = gtk_menu_new();
      
  item = gtk_image_menu_item_new_from_stock(GTK_STOCK_DIALOG_INFO, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);  
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_information_cb), window);

  item = gtk_menu_item_new_with_mnemonic(_("Rename..."));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_rename_cb), window);

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  
  item = gtk_image_menu_item_new_with_mnemonic(_("Log _In"));
  image = gtk_image_new_from_stock(GTK_STOCK_YES, GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);  
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_agent_log_in_cb), window);

  item = gtk_image_menu_item_new_with_mnemonic(_("Log _Out"));
  image = gtk_image_new_from_stock(GTK_STOCK_NO, GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);   
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_agent_log_out_cb), window);

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

  submenu = gtk_menu_new();

  item = gtk_menu_item_new_with_mnemonic(_("_Request"));
  gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item); 
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_sub_request_cb), window);

  item = gtk_menu_item_new_with_mnemonic(_("_Cancel"));
  gtk_menu_shell_append(GTK_MENU_SHELL(submenu), item); 
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_sub_cancel_cb), window);

  item = gtk_menu_item_new_with_mnemonic(_("Subscription"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
  
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), submenu);

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);

  item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_contact_remove_cb), window);

  /* show */
  gtk_widget_show_all(GTK_WIDGET(menu));
  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, gdk_event->button, gdk_event->time);

  /* clean up */
  gtk_object_sink(GTK_OBJECT(menu));

  return TRUE;
}

static gboolean gj_gtk_roster_generate_mnu_group(GjRosterWindow *window, GdkEventButton *gdk_event)
{
  GtkWidget *menu = NULL;
  GtkWidget *item = NULL;
  GtkWidget *image = NULL;
  
  if(!window)
    return FALSE;

  if(gdk_event == NULL)
      return FALSE;

  menu = gtk_menu_new();
      
  item = gtk_image_menu_item_new_with_mnemonic(_("Send _Message to All"));
  image = gtk_image_new_from_stock(GJ_STOCK_MESSAGE_NORMAL, GTK_ICON_SIZE_MENU);
  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_group_message_cb), window);

  item = gtk_separator_menu_item_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  
  item = gtk_menu_item_new_with_mnemonic(_("Rename"));
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item); 
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_group_rename_cb), window);

  item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, NULL);
  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
  g_signal_connect ((gpointer) item, "activate", G_CALLBACK(gj_gtk_roster_menu_group_remove_cb), window);

  /* show */
  gtk_widget_show_all(GTK_WIDGET(menu));
  gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, gdk_event->button, gdk_event->time);

  /* clean up */
  gtk_object_sink(GTK_OBJECT(menu));

  return TRUE;
}

/*
 * roster IQ callbacks
 */
void gj_gtk_roster_contact_remove_cb(guint event_id, guint related_event_id)
{
  GjInfoquery iq = NULL;
  GjInfoquery iq_related = NULL;

  xmlNodePtr node = NULL;

  xmlChar *type = NULL;

  g_message("%s: response with event id:%d, related event id:%d", __FUNCTION__, event_id, related_event_id);

  if((iq = gj_iq_find(related_event_id)) == NULL)
    {
      g_warning("%s: failed to find infoquery based on related event id:%d", __FUNCTION__, related_event_id);

      return;
    }

  if((iq_related = gj_iq_find(event_id)) == NULL)
    return;

  /* check for errors? */
  node = gj_iq_get_xml(iq);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      gchar *detail = NULL;

      xmlChar *error_code = NULL;
      xmlChar *error_message = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);
	  gj_iq_free_all_related_by_event_id(related_event_id);
	  return;
	}

      error_code = gj_parser_find_by_attr_and_ret(node, (guchar*)"code");
      error_message = gj_parser_find_by_node_and_ret(node, (guchar*)"error"); 

      gj_gtk_show_error(_("Failed to Remove Contact."),
			atoi((gchar*)error_code),
			(gchar*)error_message);
       
      g_warning("%s: infoquery type is error, registration failed for event id:%d", __FUNCTION__, event_id);
        
      /* clean up */
      g_free(detail);
      gj_iq_free_all_related_by_event_id(event_id);
         
      return;
    }

  /* NOTE: we do not remove from the roster here... 
     we are sent a message to do this */
  
  /* clean up */
  gj_iq_free_all_related_by_event_id(event_id);
}

void gj_gtk_roster_unregister_cb(guint event_id, guint related_event_id)
{
  GjInfoquery iq = NULL;
  GjInfoquery iq_related = NULL;

  xmlChar *type = NULL;
  xmlNodePtr node = NULL;

  g_message("%s: response with event id:%d, related event id:%d", __FUNCTION__, event_id, related_event_id);
  
  if((iq = gj_iq_find(event_id)) == NULL)
    {
      gj_gtk_show_error(_("Failed to Unregister."),
			1,
			_("Invalid Response Received."));
       
      g_warning("%s: failed to find infoquery based on event id:%d", __FUNCTION__, event_id);
      return;
    }

  if((iq_related = gj_iq_find(related_event_id)) == NULL)
    {
      gj_gtk_show_error(_("Failed to Unregister."),
			2,
			_("Invalid Response Received."));

      g_warning("%s: failed to find infoquery based on related event id:%d", __FUNCTION__, related_event_id);
      return;
    }

  /* check for errors? */
  node = gj_iq_get_xml(iq_related);
  type = gj_parser_find_by_attr_and_ret(node, (guchar*)"type");

  if(type != NULL && xmlStrcmp(type, (guchar*)"error") == 0)
    {
      xmlChar *error_code = NULL;
      xmlChar *error_message = NULL;

      if((node = gj_parser_find_by_node(node, (guchar*)"error")) == NULL)
	{
	  g_warning("%s: failed to find node 'error'", __FUNCTION__);

	  gj_iq_free_all_related_by_event_id(event_id);
	  return;
	}

      error_code = gj_parser_find_by_attr_and_ret(node, (guchar*)"code");
      error_message = gj_parser_find_by_node_and_ret(node, (guchar*)"error");

      gj_gtk_show_error(_("Failed to Unregister."),
			atoi((gchar*)error_code),
			(gchar*)error_message);
      
      g_warning("%s: infoquery type is error, registration failed for event id:%d", __FUNCTION__, event_id);
        
      /* clean up */
      gj_iq_free_all_related_by_event_id(event_id);
       
      return;
    }

  gj_gtk_msg_new(FALSE,
		 GJ_GTK_MSG_BUTTON_CLOSE | GJ_GTK_MSG_ICON_INFORMATION,
		 (MsgCb)NULL,
		 NULL,
		 _("Account unregistered successfully."),
		 _("No errors."),
		 NULL);

  /* clean up */
  gj_iq_free_all_related_by_event_id(event_id);

  /* disconnect */
  gj_iq_request_goodbye(gj_me_get_connected_host());
}

static void gj_gtk_roster_user_changed_cb(guint event_id, guint related_event_id)
{
  GjInfoquery iq = NULL;

  g_message("%s: response with event id:%d and related event id:%d", __FUNCTION__, event_id, related_event_id);

  if((iq = gj_iq_find(related_event_id)) == NULL)
    {
      g_warning("%s: failed to find IQ based on related event id:%d", __FUNCTION__, related_event_id);
      return;
    }

  /* clean up */
  gj_iq_free_all_related_by_event_id(related_event_id);
}


/*
 * GUI events
 */
static void on_treeview_row_inserted(GtkTreeModel *treemodel, GtkTreePath *path, GtkTreeIter *iter, GjRosterWindow *window)
{
  GtkTreeIter iter_new;
  GtkTreePath *path_new = NULL;
  GtkTreePath *path_to_emit = NULL;

  gboolean not_group = TRUE;
  gchar *group = NULL;
  gchar *title = NULL;
  guint count = 0;
  gchar *path_str = NULL;

  gchar *s1 = NULL;

  if(!window)
    return;

  if(path == NULL)
    {
      g_warning("%s: path was NULL", __FUNCTION__);
      return;
    }

  if(iter == NULL)
    {
      g_warning("%s: iter was NULL", __FUNCTION__);
      return;
    }

  if(gtk_tree_model_iter_parent(GTK_TREE_MODEL(treemodel), &iter_new, iter) == FALSE)
    {
      g_message("%s: iter is at the toplevel already, doing nothing...", __FUNCTION__);
      return;
    }
  
  if((path_new = gtk_tree_model_get_path(GTK_TREE_MODEL(treemodel), &iter_new)) == NULL)
    {
      g_warning("%s: could not get path from toplevel iterator", __FUNCTION__);
      return;
    }

  path_str = gtk_tree_path_to_string(path_new);

  if(path_str != NULL)
    {
      g_message("%s: path is:'%s'", __FUNCTION__, path_str);
      g_free(path_str);
      path_str = NULL;
    }

  count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(treemodel), &iter_new);
  g_message("%s: %d contacts in group", __FUNCTION__, count);

  gtk_tree_model_get(GTK_TREE_MODEL(treemodel), &iter_new,
		     COL_ROSTER_NAME, &group,
		     COL_ROSTER_NOT_GROUP, &not_group,
		     -1);
  
  if(not_group == TRUE) return;

  s1 = g_markup_escape_text(group, -1);
  title = g_strdup_printf("<b>%s</b> (%d)",s1,count);

  gtk_tree_store_set(GTK_TREE_STORE(treemodel), &iter_new,
		     COL_ROSTER_TEXT, title,
		     -1);

  path_to_emit = gtk_tree_model_get_path(treemodel, &iter_new);
  gtk_tree_model_row_changed(treemodel, path_to_emit, &iter_new);

  if(group != NULL && strcmp(group, ROSTER_GROUP_OFFLINE) != 0)
    {
      /* expand */
      if(gtk_tree_view_row_expanded(GTK_TREE_VIEW(window->treeview), path_new) == FALSE)
	gtk_tree_view_expand_row(GTK_TREE_VIEW(window->treeview), path_new, FALSE);
    }

  /* clean up */
  if(path_to_emit != NULL) gtk_tree_path_free(path_to_emit);
  if(s1 != NULL) g_free(s1);
  if(group != NULL) g_free(group);
  if(title != NULL) g_free(title);
       
  return;
}

static void on_treeview_row_deleted(GtkTreeModel *treemodel, GtkTreePath *path, GjRosterWindow *window)
{
  GtkTreeView *view = NULL;
  GtkTreeStore *store = NULL;
  GtkTreeIter iter_parent; 
  
  gchar *group = NULL; 
  gchar *title = NULL; 
  gchar *path_str = NULL;
  gchar **v_path_str = NULL;
  guint count = 0; 
  gboolean can_delete = FALSE;

  if(!window) 
    return;

  view = GTK_TREE_VIEW(window->treeview);
  store = GTK_TREE_STORE(gtk_tree_view_get_model(view));

  if(path == NULL)
    {
      g_warning("%s: path was NULL", __FUNCTION__);
      return;
    }

  path_str = gtk_tree_path_to_string(path);
  if(path_str != NULL)
    {
      g_message("%s: path is:'%s'", __FUNCTION__, path_str);

      /* hack for the get parent function which doesnt work correctly. */
      if((v_path_str = g_strsplit(path_str, ":", -1)) == NULL)
	g_warning("%s: could not split path str:'%s'", __FUNCTION__, path_str);

      /* clean up */
      g_free(path_str);
      path_str = NULL;

      if(v_path_str == NULL)
	return;

      if(v_path_str[1] != NULL)
	can_delete = TRUE;

      /* clean up */
      g_strfreev(v_path_str);
    }

  /* get parent */
  if(gtk_tree_path_up(path) == FALSE || can_delete == FALSE)
    {
      g_message("%s: no parent associated with row deleted.", __FUNCTION__);
      return;
    }

  if(gtk_tree_model_get_iter(GTK_TREE_MODEL(treemodel), &iter_parent, path) == FALSE)
    {
      g_warning("%s: could not get iter for parent associated with row deleted.", __FUNCTION__);
      return;
    }

  /* get count */
  count = gtk_tree_model_iter_n_children(GTK_TREE_MODEL(treemodel), &iter_parent);
  g_message("%s: %d contacts in this group", __FUNCTION__, count);

  if(count < 1)
    {
      g_message("%s: removing group, it is empty", __FUNCTION__);
    }
  else
    {
      GtkTreePath *path = NULL;
      gchar *s1 = NULL;

      gtk_tree_model_get(treemodel, &iter_parent,
			 COL_ROSTER_NAME, &group,
			 -1);

      s1 = g_markup_escape_text(group, -1);
      title = g_strdup_printf("<b>%s</b> (%d)", s1, count);
      
      gtk_tree_store_set(store, &iter_parent,
			 COL_ROSTER_TEXT, title,
			 -1);

      path = gtk_tree_model_get_path(treemodel, &iter_parent);
      gtk_tree_model_row_changed(treemodel, path, &iter_parent);

      /* clean up */
      if(path != NULL) gtk_tree_path_free(path);
      if(s1 != NULL) g_free(s1);
      if(group != NULL) g_free(group);  
      if(title != NULL) g_free(title); 
    }
}

static gboolean on_treeview_button_press_event(GtkWidget *widget, GdkEventButton *gdk_event, GjRosterWindow *window)
{
  if(gdk_event->type == GDK_2BUTTON_PRESS && gdk_event->button == 1)
    {
      GjRosterItem ri = NULL;

      if((ri = gj_gtk_roster_model_selection_get_ri(window)) == NULL)
	return FALSE;
      
      gj_gtk_roster_handle_next_event(window, ri);
      gj_gtk_roster_handle_next_event_display(ri);
    }

  return FALSE;
}

static gboolean on_treeview_button_release_event(GtkWidget *widget, GdkEventButton *gdk_event, GjRosterWindow *window)
{
  GjRosterItem ri = NULL;
  GjJID j = NULL;
  
  ri = gj_gtk_roster_model_selection_get_ri(window);

  if(gdk_event->type == GDK_BUTTON_RELEASE && gdk_event->button == 3)
    {
      /* 
       * RIGHT SINGLE CLICK 
       */
      gboolean not_group = TRUE;
      gchar *group = NULL;

      gj_gtk_roster_model_selection_get_is_group(window, &not_group, &group);
      
      if(not_group)
	{
	  if(!ri) return FALSE;
	  j = gj_roster_item_get_jid(ri);

	  if(gj_jid_get_type(j) != GjJIDTypeAgent)
	    gj_gtk_roster_generate_mnu_user(window, ri, gdk_event);
	  else
	    gj_gtk_roster_generate_mnu_agent(window, ri, gdk_event);
	}
      else
	{
	  if(group != NULL &&
	     strcmp(group, ROSTER_GROUP_OFFLINE) != 0 &&
	     strcmp(group, ROSTER_GROUP_RESOURCE) != 0 &&
	     strcmp(group, ROSTER_GROUP_SUBSCRIBING) != 0 &&
	     strcmp(group, ROSTER_GROUP_AGENTS) != 0 &&
	     strcmp(group, ROSTER_GROUP_NOT_ON_ROSTER) != 0)
	    {
	      gj_gtk_roster_generate_mnu_group(window, gdk_event);
	    }
	}
    }
  else if(gdk_event->type == GDK_2BUTTON_PRESS && gdk_event->button == 1)
    { 
      /* 
       * LEFT DOUBLE CLICK 
       */
      gj_gtk_roster_handle_next_event(window, ri);
      gj_gtk_roster_handle_next_event_display(ri);
    }
  
  return FALSE;
}

static void on_mnu_jabber_connect_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_cp_load(gj_gtk_roster_cb_connected, gj_gtk_roster_cb_disconnected);
}

static void on_mnu_jabber_disconnect_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_iq_request_goodbye(gj_me_get_connected_host());
}

static void on_mnu_jabber_register_new_account_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_rn_load();
}

static void on_mnu_jabber_unregister_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_unregister(window);
}

static void on_mnu_jabber_change_password_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_pc_load();
}

static void on_mnu_jabber_preferences_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_mpf_load();
}

static void on_mnu_jabber_connection_settings_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_cs_load();
}

static void on_mnu_jabber_my_information_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_mi_load();
}

static void on_mnu_jabber_quit_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gnome_jabber_quit();
}

static void on_mnu_roster_add_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_ac_load(roster, NULL);
}

static void on_mnu_roster_send_message_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_sm_load(roster);
}

static void on_mnu_roster_event_viewer_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_ev_load(TRUE);
}

static void on_mnu_roster_group_chat_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_gcj_load();
}

static void on_mnu_roster_directory_search_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  /*   gj_gtk_ds_load(); */
}

static void on_mnu_roster_browse_services_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_bs_load();
}

static void on_mnu_help_homepage_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_open_url("http://gnome-jabber.sourceforge.net/");
}

static void on_mnu_help_project_information_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_open_url(PACKAGE_BUGREPORT);
}

static void on_mnu_help_log_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_log_load();
}

static void on_mnu_help_debug_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_debug_load();
}

static void on_mnu_help_about_activate(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_about_load(_("About"),
		    PACKAGE_NAME,
		    "Instant Messenger for the Jabber Protocol",
		    VERSION, /* major version */
		    BUILD_ID,
		    "Protected Under the GNU General Public License.\n"
		    "http://www.gnu.org/licenses/gpl.txt",
		    "http://sourceforge.net/projects/gnome-jabber/\n"
		    "http://www.jabber.org",
		    NULL,
		    authors,
		    documenters,
		    translators);
}

static void gj_gtk_roster_menu_contact_read_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_read(window);
}

static void gj_gtk_roster_menu_contact_new_message_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_new_message(window);
}

static void gj_gtk_roster_menu_contact_new_chat_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_new_chat(window);
}

static void gj_gtk_roster_menu_contact_headlines_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_headlines(window);
}

static void gj_gtk_roster_menu_contact_history_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_history(window);
}

static void gj_gtk_roster_menu_contact_information_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_information(window);
}

static void gj_gtk_roster_menu_contact_sub_request_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_sub_request(window);
}

static void gj_gtk_roster_menu_contact_sub_cancel_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_sub_cancel(window);
}

static void gj_gtk_roster_menu_contact_edit_groups_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_edit_groups(window);
}

static void gj_gtk_roster_menu_contact_rename_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_rename(window);
}

static void gj_gtk_roster_menu_contact_add_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_add(window);
}

static void gj_gtk_roster_menu_contact_remove_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_contact_remove(window);
}

static void gj_gtk_roster_menu_agent_log_in_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_agent_log_in(window);
}

static void gj_gtk_roster_menu_agent_log_out_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_agent_log_out(window);
}

static void gj_gtk_roster_menu_group_message_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_group_message(window);
}

static void gj_gtk_roster_menu_group_rename_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_group_rename(window);
}

static void gj_gtk_roster_menu_group_remove_cb(GtkMenuItem *menuitem, GjRosterWindow *window)
{
  gj_gtk_roster_group_remove(window);
}

static void gj_gtk_roster_presence_menu_cb(GtkMenuItem *menuitem, gpointer user_data)
{
  gint callback_action = -1;

  if(user_data == NULL)
    {
      g_warning("%s: user data was NULL", __FUNCTION__);
      return;
    }

  /* get action */
  callback_action = GPOINTER_TO_INT(user_data);

  g_message("%s: agent menu activated menuitem:0x%.8x, callback action:%d", 
	    __FUNCTION__, (gint)menuitem, callback_action);

  switch(callback_action)
    {
    case GjPresenceShowNormal:
      gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeAvailable, 
					     GjPresenceShowNormal, 
					     gj_me_get_presence_status(),
					     gj_me_get_presence_priority(),
					     TRUE);
      break;
    case GjPresenceShowChat:
      gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeAvailable, 
					     GjPresenceShowChat, 
					     gj_me_get_presence_status(),
					     gj_me_get_presence_priority(),
					     TRUE);
      break;
    case GjPresenceShowAway:
      gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeAvailable, 
					       GjPresenceShowAway, 
					       gj_me_get_presence_status(),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case GjPresenceShowXA:
      gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeAvailable, 
					       GjPresenceShowXA, 
					       gj_me_get_presence_status(),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case GjPresenceShowDND:
      gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeAvailable, 
					       GjPresenceShowDND, 
					       gj_me_get_presence_status(),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case GjPresenceShowInvisible:
      gj_gtk_roster_set_my_presence_and_save(GjPresenceTypeAvailable, 
					       GjPresenceShowInvisible, 
					       gj_me_get_presence_status(),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;


      /*     case gj_presence_menu_status: */
      /*       gj_gtk_mp_load(gj_me_get_presence_type(),  gj_me_get_presence_show()); */
    }
}

#if 0
static void gj_gtk_roster_presence_preset_menu_cb(GtkMenuItem *menuitem, gpointer user_data)
{
  gint callback_action = -1;

  if(user_data == NULL)
    {
      g_warning("%s: user data was NULL", __FUNCTION__);
      return;
    }

  /* get action */
  callback_action = GPOINTER_TO_INT(user_data);

  g_message("%s: agent menu activated menuitem:0x%.8x, callback action:%d", 
	    __FUNCTION__, (gint)menuitem, callback_action);

  switch(callback_action)
    {
    case gj_presence_preset_menu_custom:
      gj_gtk_mp_load(gj_me_get_presence_type(),  gj_me_get_presence_show());
      break;
    case gj_presence_preset_menu_idle:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("I'm here."),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case gj_presence_preset_menu_around:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("About the place..."),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case gj_presence_preset_menu_eating:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("Eating"),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case gj_presence_preset_menu_out:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("Out for a bit"),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case gj_presence_preset_menu_gaming:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("Playing games..."),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case gj_presence_preset_menu_showering:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("In the shower"),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case gj_presence_preset_menu_sleeping:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("Sleeping... zZzZ"),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;
    case gj_presence_preset_menu_working:
      gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					       gj_me_get_presence_show(), 
					       _("Working!"),
					       gj_me_get_presence_priority(),
					       TRUE);
      break;

    default:
      break;
    }
}
#endif

static gboolean on_eventbox_statusbar_button_press_event(GtkWidget *widget, GdkEventButton *event, GjRosterWindow *window)
{
  if(event->type == GDK_BUTTON_PRESS && event->button == 1)
    gj_gtk_ev_load(TRUE);

  return FALSE;
}

static void on_button_error_clicked(GtkButton *button, GjRosterWindow *window)
{
  gj_gtk_ev_load(TRUE);
}

static void on_eventbox_presence_image_button_press_event(GtkWidget *widget, GdkEventButton *event, GjRosterWindow *window)
{
  if(!window)
    return;

  if(event->type == GDK_BUTTON_PRESS && event->button == 1 &&
     gj_me_get_is_connected() == TRUE)
    {
      GtkWidget *menu = NULL;

      menu = gj_gtk_presence_menu_new(gj_gtk_roster_presence_menu_cb);
      gtk_menu_popup(GTK_MENU(menu), NULL, NULL,
		     NULL, NULL,
		     1, gtk_get_current_event_time());

      if(menu) gtk_object_sink(GTK_OBJECT(menu));
    }
}

static void on_eventbox_presence_text_button_press_event(GtkWidget *widget, GdkEventButton *event, GjRosterWindow *window)
{
  if(!window)
    return;

  if(event->type == GDK_BUTTON_PRESS && event->button == 1 &&
     gj_me_get_is_connected() == TRUE)
    {
      gtk_widget_hide(window->label_presence);
      gtk_widget_show(window->entry_status);
      
      gtk_entry_set_text(GTK_ENTRY(window->entry_status), 
			 gj_me_get_presence_status());

      gtk_widget_grab_focus(window->entry_status);
    }
}

static void on_entry_status_activate(GtkEntry *entry, GjRosterWindow *window)
{
  if(!window)
    return;

  G_CONST_RETURN gchar *text = NULL;

  if(gj_me_get_is_connected() == FALSE)
    return;

  gtk_widget_hide(window->entry_status);
  gtk_widget_show(window->label_presence);

  /* get text */
  text = gtk_entry_get_text(entry);
  if(text == NULL || text[0] == '\0')
    return;

  /* set presence */
  gj_gtk_roster_set_my_presence_and_save(gj_me_get_presence_type(), 
					 gj_me_get_presence_show(), 
					 text,
					 gj_me_get_presence_priority(),
					 TRUE);
}

static gboolean on_entry_status_focus_out_event(GtkWidget *widget, GdkEventFocus *event, GjRosterWindow *window)
{
  if(!window)
    return FALSE;

  gtk_widget_hide(window->entry_status);
  gtk_widget_show(window->label_presence); 

  return FALSE;
}

static gboolean on_entry_status_key_release_event(GtkWidget *widget, GdkEventKey *event, GjRosterWindow *window)
{
  if(!window)
    return FALSE;

  if(event->keyval == GDK_Escape)
    {
      gtk_widget_hide(window->entry_status);
      gtk_widget_show(window->label_presence); 
    }

  return FALSE;
}

static void on_destroy(GtkWidget *widget, GjRosterWindow *window)
{
  current_window = NULL;
  if(!window)
    return;

  /* stop receiving events */
  gj_event_term();

  /* stop receiving autoaway updates */
  gj_autoaway_term();

  /* stop all flashings */
  if(window->flash_list == NULL)
    {
      gint i = 0;

      for(i=0; i<g_list_length(window->flash_list); i++)
	{
	  gint ri_id = 0;
	  gpointer data = NULL;
	  
	  data = (gpointer) g_list_nth_data(window->flash_list, i);
	  if(data == NULL) continue;
	  
	  ri_id = GPOINTER_TO_INT(data);
	  window->flash_list = g_list_remove(window->flash_list, data);
	}

      g_free(window->flash_list);
    }

  if(window->flash_timeout_id != -1)
    {
      g_source_remove(window->flash_timeout_id);
      window->flash_timeout_id = -1;
    }

  g_free(window);

  /* quit app */
  gnome_jabber_quit();
}

static gboolean on_delete_event(GtkWidget *widget, GdkEvent *event, GjRosterWindow *window)
{
  if(!window)
    return TRUE;

  gtk_widget_destroy(window->window);
 
  return TRUE;
}

#ifdef __cplusplus
}
#endif
