/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_log.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/09 20:42:54 $
 * 
 * $Revision: 1.19 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdio.h>
#include <string.h>

#include <glib.h>

#include "common.h"

#include "gj_log.h"
#include "gj_support.h"
#include "gj_config.h"
#include "gj_main.h"

#include "gj_gtk_log.h"


gboolean log_initialised = FALSE;
gboolean log_started = FALSE;
gchar *log_path = NULL;

GIOChannel *log_channel = NULL;


static gchar *gj_log_check_dir();


gboolean gj_log_init()
{
  /* GLib log levels 

  G_LOG_LEVEL_ERROR             = 1 << 2,        - always fatal 
  G_LOG_LEVEL_CRITICAL          = 1 << 3,
  G_LOG_LEVEL_WARNING           = 1 << 4,
  G_LOG_LEVEL_MESSAGE           = 1 << 5,
  G_LOG_LEVEL_INFO              = 1 << 6,
  G_LOG_LEVEL_DEBUG             = 1 << 7,
  
  */

  /* set up event handlers */
  g_log_set_handler(NULL, 
		    G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("XML", 
		    G_LOG_LEVEL_INFO | G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);

  g_log_set_handler("libglade", 
		    G_LOG_LEVEL_INFO | G_LOG_LEVEL_MESSAGE | G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("Gtk", 
		    G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL | G_LOG_FLAG_FATAL,  
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("GLib", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("GModule", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("GThreads", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("Glade", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("GLib-GObject", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);
  g_log_set_handler("GObject", 
		    G_LOG_LEVEL_MASK | G_LOG_FLAG_FATAL, 
		    gj_log_handler, 
		    NULL);

  log_initialised = TRUE;

  return TRUE;
}

static gchar *gj_log_check_dir()
{
  gchar *dir = NULL;
  
  dir = g_build_filename(g_get_home_dir(), PATH_GNOME_CONFIG, NULL); 
  if(g_file_test(dir, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) == FALSE);
    gj_create_dir(dir);
  g_free (dir);
  
  dir = g_build_filename(g_get_home_dir(), PATH_GNOME_CONFIG, PACKAGE, NULL);
  if(!g_file_test(dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
    gj_create_dir(dir);
  g_free (dir);

  dir = g_build_filename(g_get_home_dir(), PATH_GNOME_CONFIG, PACKAGE, PATH_LOG, NULL);
  if(!g_file_test(dir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
    gj_create_dir(dir);
  
  return dir;
}

gboolean gj_log_start()
{
  gsize bytes;
  gchar *message = NULL;

  gchar *filename = NULL;
  const gchar *path = NULL;

  const gchar *saved_path = NULL;
  saved_path = gj_config_get_mpf_log_location();
  
  if(saved_path == NULL || g_utf8_strlen(saved_path, -1) < 1)
    {
      gchar *path = NULL;

      path = gj_log_check_dir();
      gj_config_set_mpf_log_location(path);

      log_path = g_strdup(path);
      g_free(path);
    }
  else
    {
      /* use saved path */
      log_path = g_strdup(saved_path);
    }

  path = gj_config_get_mpf_log_location();
  g_message("gj_log_init: using log path:'%s'", path);
  
/*   filename = g_build_filename(path, g_utf8_strlen(path,-1) > 0 ? G_DIR_SEPARATOR_S : "", FILE_LOG, NULL); */
  filename = g_build_filename(g_get_home_dir(),
			      PATH_GNOME_CONFIG, PACKAGE, PATH_LOG,
			      FILE_LOG,
			      NULL);

  g_message("gj_log_init: using log filename:'%s'", filename);
  log_channel = g_io_channel_new_file(filename,"w",NULL);

  if(log_channel == NULL)
    {
      g_error("gj_log_init: failed to create new file:'%s' (channel) for log (path:'%s')",filename,path);
      return FALSE;
    }

  /* start top of log */
  message = g_strdup_printf("Log started on %s at %s\n\n", gj_get_datestamp(), gj_get_timestamp());

  g_io_channel_write_chars(log_channel,
			   message,
			   -1,
			   &bytes,
			   NULL);

  /* clean up */
  g_free(message);

  gj_gtk_log_load();

  /* set to inialised */
  log_started = TRUE;

  return TRUE;
}

gboolean gj_log_term()
{
  /* close file */
  g_io_channel_shutdown(log_channel,TRUE,NULL);

  /* uninitialise logging */
  log_started = FALSE;

  return TRUE;
}

void gj_log_handler(const gchar *log_domain, GLogLevelFlags log_level, const gchar *log_message, gpointer user_data)
{
    gchar *message = NULL;
    gchar *message_log_level = NULL;
    gchar *extra = NULL;
    gchar *reformatted = NULL;
    guint bytes = 0;
    gboolean valid = FALSE;

    if(log_message == NULL)
    {
      fprintf(stderr, "could not log message it was NULL");
      fflush(stderr);
      return;
    }

    if(log_level == GJ_LOG_LEVEL_WARNING)
	message_log_level = g_strdup_printf(" ** WARNING ** ");
    else if(log_level == GJ_LOG_LEVEL_ERROR)
	message_log_level = g_strdup_printf(" ** ERROR ** ");
    else if(log_level == GJ_LOG_LEVEL_CRITICAL)
	message_log_level = g_strdup_printf(" ** CRITICAL ** ");

#ifdef WIN32
    extra = "\r";
#endif

    reformatted = gj_find_in_str_and_replace("%", log_message, "%%"); 

    /* format buffer */
    message = g_strdup_printf("%s:%s:%s%s%s%s\n",
			      gj_get_timestamp(),
			      log_domain?log_domain:"GJ",
			      message_log_level?message_log_level:"",
			      message_log_level?" ":"\t",
			      reformatted,
			      extra != NULL?extra:"");	

    if(log_initialised == 0 || log_started == 0)
    {
	/* if NOT initialised, used stderr */
	fprintf(stderr, message);
	fflush(stderr);
    }
    else
      {
	/* write to file */
	if((valid = g_utf8_validate(message, -1, NULL)) == FALSE)
	  {
	    fprintf(stderr, "[CRITICAL ERROR] !!!! INVALID UTF8 - WRITING LOG\n");
	    fflush(stderr);
	  }
	else
	  {
	    g_io_channel_write_chars(log_channel,
				     message,
				     -1,
				     &bytes,
				     NULL);
	    
	  }

	if(gnome_jabber_get_log_to_screen() == TRUE)
	  {
	    gboolean print_it = TRUE;

	    if(log_level == G_LOG_LEVEL_DEBUG && gnome_jabber_get_debugging() == FALSE)
	      print_it = FALSE;

	    if(print_it == TRUE)
	      {
		fprintf(stdout, message);
		fflush(stdout);
	      }
	  }
      }

    /* for some reason, using g_io_channel_write_chars interprets the %, so
       you need %%.  But the log function doesnt, so you just use the original 
       string. */

    if(valid == TRUE)
      gj_gtk_log_new(log_domain, log_level, log_message, NULL);

    /* clean up */
    if(message_log_level != NULL) g_free(message);
    if(message_log_level != NULL) g_free(message_log_level);
    if(reformatted != NULL) g_free(reformatted);
}

#ifdef __cplusplus
}
#endif
 
