/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_main.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/09 20:42:55 $
 * 
 * $Revision: 1.52 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdio.h>

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_LOCALE_H
#  include <locale.h>
#endif

#include <glib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <gnet.h>

#ifndef WIN32
#  include <gconf/gconf-client.h>
#  include <libgnome/gnome-init.h>
#  include <libgnome/gnome-i18n.h>
#  include <libgnome/gnome-program.h>
#endif

#include "common.h"

#include "gj_main.h"
#include "gj_log.h"
#include "gj_config.h"
#include "gj_event.h"
#include "gj_support.h"

#include "gj_gtk_roster.h"
#include "gj_gtk_my_preferences.h"
#include "gj_gtk_first_time_run.h"
#include "gj_gtk_stock.h"


gboolean gnome_jabber_init();
gboolean gnome_jabber_term();

gboolean gnome_jabber_first_time_check();
gboolean gnome_jabber_files_check();

struct
{
  gchar *file_config;
  gchar *file_glade;

  gchar *dir_config;  

  gboolean first_time_run;

  gboolean debugging;
  gboolean log_to_screen;

} gnome_jabber;


/***********************************************************************
 * JABBER SERVERS AND CONFERENCES / ROOMS
 ***********************************************************************/

const gchar *gnome_jabber_servers[] = {
  "12jabber.com",
  "aedsolucoes.com.br",
  "akl.lt",
  "amessage.at",
  "amessage.be",
  "amessage.ch",
  "amessage.de",
  "amessage.info",
  "amessage.li",
  "bloodyxml.com",
  "bluoptic.com",
  "borderlinenormal.com",
  "dhbit.ca",
  "eifelansinn.net",
  "es.tipic.com",
  "geodude.timmins.net",
  "ibericachat.com",
  "illx.org",
  "im.pizza.net",
  "jabber-fr.net",
  "jabber.2000-plus.pl",
  "jabber.anywise.com",
  "jabber.at",
  "jabber.atari-source.com",
  "jabber.bettercom.de",
  "jabber.cn",
  "jabber.com",
  "jabber.cyber-junk.de",
  "jabber.cz",
  "jabber.daemon.sh",
  "jabber.dk",
  "jabber.earth.li",
  "jabber.freenet.de",
  "jabber.gda.pl",
  "jabber.hu",
  "jabber.kelkoo.net",
  "jabber.kicks-ass.org",
  "jabber.kolbsoft.com",
  "jabber.linux.it",
  "jabber.lumina.ro",
  "jabber.mailclean.net",
  "jabber.mandog.com",
  "jabber.netsample.com",
  "jabber.ngnet.it",
  "jabber.noze.it",
  "jabber.or.id",
  "jabber.or.kr",
  "jabber.org",
  "jabber.palomine.net",
  "jabber.projecto-oasis.cx",
  "jabber.ro",
  "jabber.ru",
  "jabber.sk",
  "jabber.toble.com",
  "jabber.unesco.kz",
  "jabber.wiretrip.org",
  "jabber.wp.pl",
  "jabberlawine.de",
  "jabberpl.org",
  "kanga.nu",
  "myjabber.net",
  "nabla.net",
  "netmindz.net",
  "netnewmedia.de",
  "njs.netlab.cz",
  "nureality.ca",
  "openaether.org",
  "oracle.sweeney.ath.cx",
  "planet-linux.org",
  "rhymbox.com",
  "sml.dyndns.org",
  "sourcecode.de",
  "sucs.org",
  "swissjabber.ch",
  "syndicon.de",
  "theoretic.com",
  "tipic.com",
  "tipic.it",
  "xasa.com",
  NULL 
};

const gchar *gnome_jabber_conferences[] = {
  "jdev@conference.jabber.org",
  "jabber@conference.jabber.org",
  NULL,
};

/****************************************************************************/

int main(int argc, char **argv)
{
  gboolean log_to_screen = FALSE;
  gboolean debugging = FALSE;

#ifndef WIN32
  GnomeProgram *program = NULL;

  struct poptOption options[] = 
    {
      {
	"debugging",
	'd',
	POPT_ARG_NONE,
	&debugging,
	0,
	N_("Logs more, uses local glade file, etc."),
	NULL
      },
      
      {
	"log-to-screen",
	'l',
	POPT_ARG_NONE,
	&log_to_screen,
	0,
	N_("Logs to screen (stdout)."),
	NULL
      },
      
      { NULL, '\0', 0, NULL, 0, NULL, NULL }
    };
#endif

  /* init variables */
  gnome_jabber_init();


  /* gtk / gdk / gnome initiations */
  if(gtk_init_check(&argc,&argv) == FALSE)
    {
      g_error("%s: failed to initiate gtk", __FUNCTION__);
      exit(0);
    }
  else
    g_message("%s: initiated gtk", __FUNCTION__);

  if(gdk_init_check(&argc,&argv) == FALSE)
    {
      g_error("%s: failed to initiate gdk", __FUNCTION__);
      exit(0);
    }
  else
    g_message("%s: initiated gdk", __FUNCTION__);

  gnet_init();
  g_message("%s: initiated gnet", __FUNCTION__);

  /* sets the log handler */
  gj_log_init();

#ifdef HAVE_LOCALE_H
  /* internationalisation */
  setlocale(LC_ALL, "");
#endif

#ifdef HAVE_GETTEXT
  bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
  textdomain(GETTEXT_PACKAGE);
  bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
#endif

#ifndef WIN32

  program = gnome_program_init (PACKAGE, VERSION,
				LIBGNOME_MODULE,
				argc, argv,
				GNOME_PROGRAM_STANDARD_PROPERTIES,
				GNOME_PARAM_ENABLE_SOUND, TRUE,
 				GNOME_PARAM_POPT_TABLE, options, 
				GNOME_PARAM_HUMAN_READABLE_NAME, "GnomeJabber",
				GNOME_PARAM_NONE);



  gnome_jabber_set_debugging(debugging);
  gnome_jabber_set_log_to_screen(log_to_screen);

#endif

  /* check to see if we have run this app before? */
  gnome_jabber_files_check();

  if(gnome_jabber_get_debugging() == TRUE)
    {
      gnome_jabber_set_glade_file("gnome-jabber.glade");
      g_message("%s: using local gnome-jabber.glade", __FUNCTION__);
    }

  /* config */
  gj_config_init(gnome_jabber_get_config_file());
  g_message("%s: initiated config", __FUNCTION__);

  /* get config */
  gj_config_read();

  /* logging */
  gj_log_start();
  g_message("%s: initiated logging", __FUNCTION__);

  /* stock icon init */
  if(gj_gtk_stock_init() == FALSE)
    g_warning("%s: stock icons could not be initialised", __FUNCTION__);
  else
    g_message("%s: initialised stock icons", __FUNCTION__);

  /* check for first time run */
  if(gnome_jabber_get_first_time_run() == TRUE)
    {
      g_message("%s: running for the first time...", __FUNCTION__);
      
      /* so we load the connection settings too */
      gj_gtk_roster_load();
      gj_gtk_ftr_load();
    }
  else
    {
      /* roster */
      gj_gtk_roster_load();
      g_message("%s: initiated roster", __FUNCTION__);
    }
 
  /* start event loop */
  g_message("%s: ready!", __FUNCTION__);
  gtk_main();

  /* clean up variables */
  gnome_jabber_term();

  /* end application */
  g_message("%s: exiting...", __FUNCTION__);

  return 0;  
}

gboolean gnome_jabber_first_time_check()
{
  
    
  return FALSE;
}

gboolean gnome_jabber_files_check()
{
  G_CONST_RETURN gchar *home_dir = g_get_home_dir();
  gchar *config_dir = NULL;
  gchar *history_dir = NULL;
  gchar *file_config = NULL;
/*   gchar *file_css = NULL; */

  g_message("%s: home directory is:'%s'", __FUNCTION__, home_dir);

  config_dir = g_build_path(G_DIR_SEPARATOR_S, home_dir, PATH_USERDIR, NULL);
  history_dir = g_build_path(G_DIR_SEPARATOR_S, home_dir, PATH_FULL_HISTORY, NULL);
  file_config = g_build_filename(home_dir, FILE_FULL_CONFIG, NULL);
/*   file_css = g_build_filename(home_dir, FILE_FULL_CSS, NULL); */

#ifdef WIN32
  /* if this directory does not exist, it has not been run by this user before */
  if(g_file_test(config_dir, (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) == FALSE)
    {
      g_message("%s: directory '%s' does NOT exist, creating...", __FUNCTION__, config_dir);
      if(!gj_create_dir(config_dir)) return FALSE;

      gnome_jabber_set_first_time_run(TRUE);
    }
#endif

#if 0
  /* we create this because it will be the default history directory on first time runs */
  if(g_file_test(history_dir,  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR)) == FALSE)
    {
      g_message("%s: directory '%s' does NOT exist, creating...", __FUNCTION__, history_dir);
      if(!gj_create_dir(history_dir)) return FALSE;
    }
#endif

#ifdef WIN32
  /* if this file does not exist, it is possible this is a first time run */
  if(g_file_test(file_config,  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) == FALSE)
    {
      g_message("%s: file '%s' does NOT exist", __FUNCTION__, file_config);
      gnome_jabber_set_first_time_run(TRUE);
    }
#endif

  /* if this file does not exist, formatting will be the only issue with the history */
  if(g_file_test(FILE_FULL_CSS,  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) == FALSE)
    g_message("%s: file '%s' does NOT exist", __FUNCTION__, FILE_FULL_CSS);
  
  /* error if not found because the application is completely dependant on it. */
  if(g_file_test(FILE_FULL_GLADE,  (G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR)) == FALSE)
    g_error("%s: file '%s' does NOT exist", __FUNCTION__, FILE_FULL_GLADE);

  /* save */
  gnome_jabber_set_config_file(file_config);
  gnome_jabber_set_glade_file(FILE_FULL_GLADE);
  gnome_jabber_set_config_dir(config_dir);

  /* clean up */
  if(config_dir != NULL) g_free(config_dir);
  if(history_dir != NULL) g_free(history_dir);
  if(file_config != NULL) g_free(file_config);
/*   if(file_css != NULL) g_free(file_css); */

  return TRUE;
}

/*
 * member functions 
 */

/* gets */
gchar *gnome_jabber_get_config_file()
{
  return gnome_jabber.file_config;
}

gchar *gnome_jabber_get_glade_file()
{
  return gnome_jabber.file_glade;
}

gchar *gnome_jabber_get_config_dir()
{
  return gnome_jabber.dir_config;
}

gboolean gnome_jabber_get_first_time_run()
{
  return gnome_jabber.first_time_run;
}

gboolean gnome_jabber_get_debugging()
{
  return gnome_jabber.debugging;
}

gboolean gnome_jabber_get_log_to_screen()
{
  return gnome_jabber.log_to_screen;
}

const gchar **gnome_jabber_get_server_list()
{
  return gnome_jabber_servers;
}

/* sets */
gboolean gnome_jabber_set_config_file(gchar *filename)
{
  if(filename == NULL)
    {
      g_warning("gnome_jabber_set_config_file: filename was NULL");
      return FALSE;
    }

  if(gnome_jabber.file_config != NULL)
    {
      g_free(gnome_jabber.file_config);
      gnome_jabber.file_config = NULL;
    }

  gnome_jabber.file_config = g_strdup(filename);
  return TRUE;
}

gboolean gnome_jabber_set_glade_file(gchar *filename)
{
  if(filename == NULL)
    {
      g_warning("gnome_jabber_set_glade_file: filename was NULL");
      return FALSE;
    }

  if(gnome_jabber.file_glade != NULL)
    {
      g_free(gnome_jabber.file_glade);
      gnome_jabber.file_glade = NULL;
    }

  gnome_jabber.file_glade = g_strdup(filename);
  return TRUE;
}

gboolean gnome_jabber_set_config_dir(gchar *dir)
{
  if(dir == NULL)
    {
      g_warning("gnome_jabber_set_config_dir: dir was NULL");
      return FALSE;
    }

  if(gnome_jabber.dir_config != NULL)
    {
      g_free(gnome_jabber.dir_config);
      gnome_jabber.dir_config = NULL;
    }

  gnome_jabber.dir_config = g_strdup(dir);
  return TRUE;
}

gboolean gnome_jabber_set_first_time_run(gboolean first_time_run)
{
  gnome_jabber.first_time_run = first_time_run;
  return TRUE;
}

gboolean gnome_jabber_set_debugging(gboolean debugging)
{
  gnome_jabber.debugging = debugging;
  return TRUE;
}

gboolean gnome_jabber_set_log_to_screen(gboolean log_to_screen)
{
  gnome_jabber.log_to_screen = log_to_screen;
  return TRUE;
}

gboolean gnome_jabber_init()
{
  gnome_jabber.file_config = NULL;
  gnome_jabber.file_glade = NULL;
  gnome_jabber.dir_config = NULL;

  gnome_jabber.first_time_run = FALSE;

  gnome_jabber.debugging = FALSE;
  gnome_jabber.log_to_screen = FALSE;

  return TRUE;
}

gboolean gnome_jabber_term()
{
  if(gnome_jabber.file_config != NULL)
    {
      g_free(gnome_jabber.file_config);
      gnome_jabber.file_config = NULL;
    }
    
  if(gnome_jabber.file_glade != NULL)
    {
      g_free(gnome_jabber.file_glade);
      gnome_jabber.file_glade = NULL;
    }
    
  if(gnome_jabber.dir_config != NULL)
    {
      g_free(gnome_jabber.dir_config);
      gnome_jabber.dir_config = NULL;
    }

  gnome_jabber.first_time_run = FALSE;
 
  gnome_jabber.debugging = FALSE;
  gnome_jabber.log_to_screen = FALSE;
  
  return TRUE;
}

gboolean gnome_jabber_quit()
{
  g_message("received quit request");
  gtk_main_quit();

  return TRUE;
}


#ifdef __cplusplus
}
#endif



