/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_registration.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/30 20:38:28 $
 * 
 * $Revision: 1.9 $ 
 *
 */
#ifndef __gj_register_h
#define __gj_register_h

#include "gj_typedefs.h"
 

/* list functions */
GList *gj_registration_list_get();

/* item functions */
GjRegistration gj_registration_new();
void gj_registration_free(GjRegistration rgr);

/* find */
GjRegistration gj_registration_find_by_id(const gchar *id);
GjRegistration gj_registration_find_by_to(const gchar *to);
GjRegistration gj_registration_find_by_from(const gchar *from);

/* sets */
gboolean gj_registration_set_id(GjRegistration rgr, const gchar *id);
gboolean gj_registration_set_to(GjRegistration rgr, const gchar *to);
gboolean gj_registration_set_from(GjRegistration rgr, const gchar *to);
gboolean gj_registration_set_instructions(GjRegistration rgr, const gchar *instructions);
gboolean gj_registration_set_username(GjRegistration rgr, const gchar *username);
gboolean gj_registration_set_password(GjRegistration rgr, const gchar *password);
gboolean gj_registration_set_email(GjRegistration rgr, const gchar *email);
gboolean gj_registration_set_require_username(GjRegistration rgr, const gboolean required);
gboolean gj_registration_set_require_password(GjRegistration rgr, const gboolean required);
gboolean gj_registration_set_require_email(GjRegistration rgr, const gboolean required);
gboolean gj_registration_set_require_nickname(GjRegistration rgr, const gboolean required);
gboolean gj_registration_set_key(GjRegistration rgr, const gchar *key);
gboolean gj_registration_set_callback(GjRegistration rgr, GjRegistrationCallback cb);

/* gets */
const gchar *gj_registration_get_iD(GjRegistration rgr);
const gchar *gj_registration_get_to(GjRegistration rgr);
const gchar *gj_registration_get_from(GjRegistration rgr);
const gchar *gj_registration_get_instructions(GjRegistration rgr);
const gchar *gj_registration_get_username(GjRegistration rgr);
const gchar *gj_registration_get_password(GjRegistration rgr);
const gchar *gj_registration_get_email(GjRegistration rgr);
gboolean gj_registration_get_require_username(GjRegistration rgr);
gboolean gj_registration_get_require_password(GjRegistration rgr);
gboolean gj_registration_get_require_email(GjRegistration rgr);
gboolean gj_registration_get_require_nickname(GjRegistration rgr);
const gchar *gj_registration_get_key(GjRegistration rgr);
GjRegistrationCallback gj_registration_get_callback(GjRegistration rgr);

/*
 * parsing functions  
 */
gboolean gj_registration_request_requirements(const gchar *server, 
					      GjRegistrationRequestCallback cb);
gboolean gj_registration_request(const gchar *server, 
				 GjRegistrationRequestCallback cb, 
				 const gchar *key, 
				 const gchar *username, 
				 const gchar *password, 
				 const gchar *email, 
				 const gchar *nickname);

#endif

