/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_stream.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/30 20:38:28 $
 * 
 * $Revision: 1.14 $
 *
 */

#ifndef __gj_stream_h
#define __gj_stream_h

#include "gj_typedefs.h"


/*
 * item functions
 */
GjStream gj_stream_new(const char *host);
gboolean gj_stream_free(GjStream stm);

/* find */
GjStream gj_stream_find_by_id(const gchar *id);
GjStream gj_stream_find_by_unique_id(const guint id);
GjStream gj_stream_find_by_connection(GjConnection c);
GjStream gj_stream_find_by_request_id(gint requestID);
GjStream gj_stream_find_by_to(const gchar *to);

/*
 * member functions
 */

/* set */
gboolean gj_stream_set_to(GjStream stm, const gchar *to);
gboolean gj_stream_set_port(GjStream stm, guint port);

gboolean gj_stream_set_connected_cb(GjStream stm, GjStreamCallback cb);
gboolean gj_stream_set_disconnected_cb(GjStream stm, GjStreamCallback cb);
gboolean gj_stream_set_error_cb(GjStream stm, GjStreamCallback cb);

/* get */
const gchar *gj_stream_get_id(GjStream stm);
guint gj_stream_get_unique_id(GjStream stm);
gint gj_stream_get_request_id(GjStream stm);

const gchar *gj_stream_get_to(GjStream stm);
guint gj_stream_get_port(GjStream stm);

gboolean gj_stream_get_is_disconnect_planned(GjStream stm);

GjStreamCallback gj_stream_get_connected_cb(GjStream stm);
GjStreamCallback gj_stream_get_disconnected_cb(GjStream stm);
GjStreamCallback gj_stream_get_error_cb(GjStream stm);

const gchar *gj_stream_get_stream(GjStream stm);
xmlDocPtr gj_stream_get_xml(GjStream stm);

GjConnection gj_stream_get_connection(GjStream stm);

/* building */
gboolean gj_stream_open(GjStream stm);  /* build then open the stream...! */
gboolean gj_stream_close(GjStream stm);
gboolean gj_stream_close_by_to(const gchar *to);
gboolean gj_stream_send(GjStream stm, const gchar *text);
gboolean gj_stream_send_by_to(const gchar *to, const gchar *text);

gboolean gj_stream_build(GjStream stm);

/* input from parser */
void gj_stream_receive_old(GjConnection c, xmlNodePtr node); 
void gj_stream_receive(GjConnection c, gchar *from, gchar *id);
void gj_stream_receive_error(GjConnection c, xmlNodePtr node);

/*
 * debugging 
 */
void gj_stream_print_all();
void gj_stream_print(GjStream stm);

#endif

