/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_translate.c,v $
 *
 * $Author: ginxd $
 * $Date: 2003/10/11 14:31:30 $
 * 
 * $Revision: 1.17 $
 *
 */

#ifdef __cplusplus
extern "C" {
#if 0
}
#endif
#endif

#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include "common.h"

#include "gj_roster.h"
#include "gj_presence.h"
#include "gj_message.h"
#include "gj_translate.h"
#include "gj_event.h"

/*
 * NOTE: should we allocate memory for each function which returns a value?
 *       or should we simply return "my translation" instead?
 */

const gchar *gj_translate_roster_item_type(GjRosterItemType type)
{
  switch(type)
    {
    case GjRosterItemTypeNone: return _("none");
    case GjRosterItemTypeContact: return _("contact");
    case GjRosterItemTypeAgent: return _("agent");
    case GjRosterItemTypeGroupChat: return _("group chat");
    case GjRosterItemTypeEnd: return "ROSTER ITEM TYPE NOT SET";
    }

  return _("unknown");
}

const gchar *gj_translate_roster_item_subscription(GjRosterItemSubscription subscription)
{
  switch(subscription)
    {
    case GjRosterItemSubscriptionNone: return _("None");
    case GjRosterItemSubscriptionTo: return _("To");
    case GjRosterItemSubscriptionFrom: return _("From");
    case GjRosterItemSubscriptionBoth: return _("Both");
    case GjRosterItemSubscriptionEnd: return "ROSTER ITEM SUBSCRIPTION NOT SET";
    }

  return _("unknown");
}

const gchar *gj_translate_roster_item_subscription_as_description(GjRosterItemSubscription subscription)
{
  switch(subscription)
    {
    case GjRosterItemSubscriptionNone: return _("Neither users have presence.");
    case GjRosterItemSubscriptionTo: return _("You can see the users presence.");
    case GjRosterItemSubscriptionFrom: return _("The user can see your presence.");
    case GjRosterItemSubscriptionBoth: return _("Presence is available to and from the user.");
    case GjRosterItemSubscriptionEnd: return _("Subscription has not been set.");
    }

  return _("unknown");
}

const gchar *gj_translate_event_type(GjEventType type)
{
  switch(type)
    {
    case GjEventTypePresence: return _("Presence");
    case GjEventTypeMessage: return _("Message");
    case GjEventTypeEnd: return "EVENT TYPE NOT SET";
    }

  return _("unknown");
}

const gchar *gj_translate_presence_type(GjPresenceType type)
{
  switch(type)
    {
    case GjPresenceTypeAvailable: return _("Available");
    case GjPresenceTypeUnAvailable: return _("Unavailable");
    case GjPresenceTypeProbe: return _("Probe");
    case GjPresenceTypeSubscribe: return _("Subscribe");
    case GjPresenceTypeUnSubscribe: return _("Unsubscribe");
    case GjPresenceTypeSubscribed: return _("Subscribed");
    case GjPresenceTypeUnSubscribed: return _("Unsubscribed");
    case GjPresenceTypeError: return _("Error");
    case GjPresenceTypeNone: return _("None");
    case GjPresenceTypeEnd: return "PRESENCE TYPE NOT SET";
    }

  return _("unknown");
}

const gchar *gj_translate_presence_show(GjPresenceShow show)
{
  switch(show)
    {
    case GjPresenceShowAway: return _("Away");
    case GjPresenceShowChat: return _("Chat");
    case GjPresenceShowDND: return _("Busy");
    case GjPresenceShowNormal: return _("Available");
    case GjPresenceShowXA: return _("Not Available");
    case GjPresenceShowInvisible: return _("Invisible");
    case GjPresenceShowEnd: return "PRESENCE SHOW NOT SET";
    }

  return _("unknown");
}

const gchar *gj_translate_message_type(GjMessageType type)
{
  switch(type)
    {
    case GjMessageTypeNormal: return _("Normal");
    case GjMessageTypeChat: return _("Chat");
    case GjMessageTypeGroupChat: return _("Groupchat");
    case GjMessageTypeHeadline: return _("Headline");
    case GjMessageTypeError: return _("Error");
    case GjMessageTypeXEvent: return _("X Event");
    case GjMessageTypeEnd: return "MESSAGE TYPE NOT SET";
    }

  return _("unknown");
}

gchar *gj_translate_message_x_event_type(gint type)
{
    gchar *retval = "";

    if(type & GjMessageXEventTypeCancelled)
	return g_strdup_printf("cancelled");

    if(type & GjMessageXEventTypeOffline)
	retval = g_strconcat(retval, "offline | ", NULL);
    if(type & GjMessageXEventTypeDelivered)
	retval = g_strconcat(retval, "delivered | ", NULL);
    if(type & GjMessageXEventTypeDisplayed)
	retval = g_strconcat(retval, "displayed | ", NULL);
    if(type & GjMessageXEventTypeComposing) 
	retval = g_strconcat(retval, "composing | ", NULL);

    return retval;
}

#ifdef __cplusplus
}
#endif
