/*
 * Copyright (C) 2003      Martyn Russell <ginxd@btopenworld.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* 
 * $RCSfile: gj_typedefs.h,v $
 *
 * $Author: ginxd $
 * $Date: 2003/11/22 14:52:30 $
 * 
 * $Revision: 1.4 $
 *
 */

#ifndef __gj_typedefs_h
#define __gj_typedefs_h

#include <time.h> 

#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <glade/glade.h>

#include <gnet.h>

#include <libxml/tree.h>
#include <libxml/parser.h>

#include "common.h"


/*
 * JID
 */
typedef struct t_jid *GjJID;

typedef enum {
  GjJIDTypeNone = 1,
  GjJIDTypeContact,
  GjJIDTypeAgent,

  GjJIDTypeEnd 
} GjJIDType;

/*
 * presence
 */
typedef struct t_presence *GjPresence;

typedef void (*GjPresenceFromCallback)(GjPresence pres);
typedef void (*GjPresenceToCallback)(gint requestID);

typedef struct t_presenceTo *GjPresenceTo; 

typedef enum {
  GjPresenceTypeAvailable = 1,   /* online */  
  GjPresenceTypeUnAvailable,     /* offline */
  GjPresenceTypeProbe,           /* ask another user's status */
  GjPresenceTypeSubscribe,       /* request to subscribe to presence of user */
  GjPresenceTypeUnSubscribe,     /* request to unsubscribe to presence of user */
  GjPresenceTypeSubscribed,      /* result to subscribe */ 
  GjPresenceTypeUnSubscribed,    /* result to unsubscribe */
  GjPresenceTypeError,           /* error */
  GjPresenceTypeNone,            /* none - we are not given presence */
  
  GjPresenceTypeEnd              /* end of types */
} GjPresenceType;

typedef enum {
  GjPresenceShowAway = 1,    /* temporarily away */
  GjPresenceShowChat,        /* similar to normal except in a conversation */
  GjPresenceShowDND,         /* do not disturb */ 
  GjPresenceShowNormal,      /* default */
  GjPresenceShowXA,          /* eXtended Away, almost unavailable */
  GjPresenceShowInvisible,   /* Invisible, - this is like being disconnected except you arent */
  
  GjPresenceShowEnd          /* end of types */
} GjPresenceShow;


/*
 * message
 */
typedef struct t_message *GjMessage;

typedef void (*GjMessageFromCallback)(GjMessage m);
typedef void (*GjMessageToCallback)(gint requestID);

typedef enum {
  GjMessageTypeNormal = 1,
  GjMessageTypeChat,
  GjMessageTypeGroupChat,
  GjMessageTypeHeadline,
  GjMessageTypeError,
  GjMessageTypeXEvent,
    
  GjMessageTypeEnd
} GjMessageType;
 

#define GjMessageXEventTypeOffline      1 << 1
#define GjMessageXEventTypeDelivered    1 << 2
#define GjMessageXEventTypeDisplayed    1 << 3 
#define GjMessageXEventTypeComposing    1 << 4
#define GjMessageXEventTypeCancelled    1 << 5


/*
 * history
 */
typedef struct t_history *GjHistory;


/* 
 * roster
 */
typedef struct t_roster *GjRoster;

typedef void (*GjRosterCallback)(const GList *contacts, const GList *agents, gpointer user_data);
typedef void (*GjRosterSetCallback)(const GList *updated_roster_items, gpointer user_data);


/*
 * roster item
 */
typedef struct t_roster_item *GjRosterItem; 

typedef enum {
  GjRosterItemSubscriptionNone = 1,
  GjRosterItemSubscriptionTo,
  GjRosterItemSubscriptionFrom,
  GjRosterItemSubscriptionBoth,
  
  GjRosterItemSubscriptionEnd
} GjRosterItemSubscription;

typedef enum {
  GjRosterItemTypeNone = 1,
  GjRosterItemTypeContact,
  GjRosterItemTypeAgent,
  GjRosterItemTypeGroupChat, /* used for group chats */
  
  GjRosterItemTypeEnd
} GjRosterItemType;


/* 
 * agents 
 */
typedef struct t_agent *GjAgent;

typedef void (*GjAgentCallback)(GList *agents, gint error_code, gchar *error_message, void *userdata);


/*
 * Registration (new jabber accounts)
 */
typedef struct t_registration *GjRegistration;

typedef void (*GjRegistrationCallback)(GjRegistration rgr, gint error_code, const gchar *error_reason);
typedef void (*GjRegistrationRequestCallback)(GjRegistration rgr, gint error_code, const gchar *error_reason);


/*
 * Search (JUD)
 */
typedef struct t_search *search;

typedef void (*searchCallback)(search rgr, gint error_code, gchar *error_reason);
typedef void (*searchRequestCallback)(search rgr, gint error_code, gchar *error_reason);


/*
 * gatherdata 
 */
typedef struct t_gatherdata *GjData;
typedef struct t_gatherdata_field *GjDataField;
typedef struct t_gatherdata_field_option *GjDataFieldOption;

typedef void (*GjDataCallback)(GjData gd, gint error_code, gchar *error_reason);
typedef void (*GjDataRequestCallback)(GList *list, gint error_code, gchar *error_reason);


typedef enum {
  GjDataFormTypeForm = 1,        /* This packet contains a form to fill out. Display it to the user */    
  GjDataFormTypeSubmit,          /* The form is filled out, and this is the data that is being returned from the form. */
  GjDataFormTypeCancel,          /* The form was cancelled. Tell the asker that piece of information. */
  GjDataFormTypeResult,          /* Data results being returned from a search, or some other query. */

  GjDataFormTypeEnd

} GjDataFormType;

typedef enum {
  
  GjDataFieldTypeTextSingle = 1, /* single line or word of text */
  GjDataFieldTypeTextPrivate,    /* instead of showing the user what they typed, you show ***** to protect it */
  GjDataFieldTypeTextMulti,      /* multiple lines of text entry */
  GjDataFieldTypeListSingle,     /* given a list of choices, pick one */
  GjDataFieldTypeListMulti,      /* given a list of choices, pick one or more */
  GjDataFieldTypeBoolean,        /* 0 or 1, true or false, yes or no. Default value is 0 */
  GjDataFieldTypeFixed,          /* fixed for putting in text to show sections, 
				    or just advertise your web site in the middle of the form */
  GjDataFieldTypeHidden,         /* is not given to the user at all, but returned with the questionnaire
				    (e.g., a unique number at the top of a sheet of paper to identify the form) */
  GjDataFieldTypeJIDSingle,      /* Jabber ID - choosing a JID from your roster, and entering one based on the rules for a JID. */
  GjDataFieldTypeJIDMulti,       /* multiple entries for JIDs */

  GjDataFieldTypeEnd
  
} GjDataFieldType;



/*
 * IQs
 */
typedef struct t_infoquery *GjInfoquery;

typedef void (*GjInfoqueryCallback)(guint related_event_id, guint event_id);

typedef enum {
  GjInfoqueryTypeNull = 0,
  GjInfoqueryTypeGet,
  GjInfoqueryTypeSet,
  GjInfoqueryTypeResult,
  GjInfoqueryTypeError,
  
  GjInfoqueryTypeEnd
} GjInfoqueryType;


/*
 * events
 */
typedef enum {
  GjEventTypePresence = 1,
  GjEventTypeMessage,

  GjEventTypeEnd
} GjEventType;


typedef struct t_event *GjEvent;

typedef void (*GjEventCallback)(gint event_id, gpointer user_data);


/* 
 * parser
 */
typedef struct t_parser *GjParser;

typedef void (*GjParserStreamCallback)(GjParser p, const gchar *from, const gchar *id);
typedef void (*GjParserMessageCallback)(GjParser p, xmlNodePtr n);
typedef void (*GjParserPresenceCallback)(GjParser p, xmlNodePtr n);
typedef void (*GjParserIQCallback)(GjParser p, xmlNodePtr n);
typedef void (*GjParserErrorCallback)(GjParser p, xmlNodePtr n);
typedef void (*GjParserOtherCallback)(GjParser p, xmlNodePtr n);

/*
 * stream
 */
typedef struct t_stream *GjStream;

typedef void (*GjStreamCallback)(GjStream stm, gint error);


/*
 * connection
 */
typedef struct t_connection *GjConnection;

typedef void (*GjConnectionCallbackIn)(GjConnection c, gchar *text);
typedef void (*GjConnectionCallbackHup)(GjConnection c);
typedef void (*GjConnectionCallbackConnect)(GjConnection c);
typedef void (*GjConnectionCallbackDisconnect)(GjConnection c);
typedef void (*GjConnectionCallbackError)(GjConnection c, GTcpSocketConnectAsyncStatus status);


/*
 * AutoAway
 */
typedef void (*GjAutoAwayCallback)(guint msec, gpointer user_data);


#endif

